/***************************************************************************
* C Header:	    mrbt.h
*	Version:	v1.1
*	%created_by:	sks %
*	%date_created: Wed July 22 11:11:09 2009 %
*
***************************************************************************/
/**********************************************************************
*  Copyright (C) 2000-2002 Dialogic Corporation.
*  All Rights Reserved
*
*  All names, products, and services mentioned herein are the 
*  trademarks or registered trademarks of their respective 
*  organizations and are the sole property of their respective owners.
**********************************************************************/
#ifndef _mrbt_h
#define _mrbt_h


/* OS Header Files */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <sys/timeb.h>
//#include <conio.h>

/* OS Socket API Headers */
#include <sys/select.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <stropts.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <ctype.h>
#include <ncurses.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <curses.h>

/* Dialogic Header Files */
#include <gcip.h>
#include <gclib.h>
#include <gcisdn.h>
#include <srllib.h>
// SKS
#include <gcipmlib.h>
#include <dxxxlib.h>
#include <cc_s7.h>
#include <pthread.h>
#include <fcntl.h>


// SKS
#include <m3glib.h>
#include <m3gevts.h>
#include <m3gerrs.h>
#include <port_connect.h>
#include <mmlib.h>


#define UNROUTE			0
#define ROUTE				1

// connection type to connect two devices
#define FULL_DUP		4
#define HALF_DUP		2

#define AUDIO_LCN 				1
#define VIDEO_LCN 				2
#define AUDIO_VIDEO_LCN 	3

#define DLGC_XCODER_QCIF_DCI_STR "000001b008000001b509000001000000012000845d4c282c2090a28f"

static const char* portMediaTypeStr[] =
{
      "NONE",
      "AUDIO",
      "VIDEO",
      "H223/NBUP"
};
enum	app_state_t 
{
       APP_STATE_INITIAL           						= 0,
       APP_STATE_H245_STARTED      						= 1,
       APP_STATE_MM_M3G_CONNECTED         		= 2,
       APP_STATE_DTI_M3G_CONNECTED      			= 3,
       APP_STATE_M3G_M3G_CONNECTED   					= 4,
       APP_STATE_MM_M3G_DISCONNECTED         	= 5,
       APP_STATE_DTI_M3G_DISCONNECTED      		= 6,
       APP_STATE_M3G_M3G_DISCONNECTED   			= 7,
       APP_STATE_H245_STOPPED     						= 8,
			 APP_STATE_MRBT_PLAY_STARTED						= 9,
			 APP_STATE_MRBT_PLAY_STOPPED						= 10,
			 APP_STATE_MEDIA_STARTED								= 11,
			 APP_STATE_MEDIA_IDLE										= 12,
			 APP_STATE_MEDIA_STOPPED								= 13,
			 APP_STATE_ENDPOINT_CONNECTED						= 14,
			 APP_STATE_FRAMING_LOST									= 15,
			 APP_STATE_UNKNOWN											= 16
};

static const char* appStatesStr[] =
{
		"APP_STATE_INITIAL",
		"APP_STATE_H245_STARTED",
		"APP_STATE_MM_M3G_CONNECTED",
		"APP_STATE_DTI_M3G_CONNECTED",
		"APP_STATE_M3G_M3G_CONNECTED",
		"APP_STATE_MM_M3G_DISCONNECTED",
		"APP_STATE_DTI_M3G_DISCONNECTED",
		"APP_STATE_M3G_M3G_DISCONNECTED",
		"APP_STATE_H245_STOPPED",
		"APP_STATE_MRBT_PLAY_STARTED",
		"APP_STATE_MRBT_PLAY_STOPPED",
		"APP_STATE_MEDIA_STARTED",
		"APP_STATE_MEDIA_IDLE",
		"APP_STATE_MEDIA_STOPPED",
		"APP_STATE_ENDPOINT_CONNECTED",
		"APP_STATE_FRAMING_LOST",
		"APP_STATE_UNKNOWN"
};

/* Macro Definitions */

#define MAX_STRING_SIZE  1000	/* Maximum length of string that to write on log file			*/
#define MAX_FILE_NAME			256
#define MAXCHAN						240		/* Maximum number of channels that can be opened - 2 E1 spans	*/
#define MAX_DEVNAME			 	 64		/* Maximum length of device name											*/
#define MAX_CALLS					  2		/* Maximum number of calls per channel */

/* Logging defines */
#define NON_GC_APIERR		0	/* Had error on non-GC API function										*/
#define EVENT						1	/* An event																		*/
#define GC_APICALL			2	/* GC API funtion call														*/
#define GC_APIERR				3	/* Had error on GC API function											*/
#define STATE						4	/* STATE message																*/
#define GC_RESULT_INFO	5	/* want GC result information												*/
#define MISC						6	/* None of the above															*/
#define MISC_WITH_NL		7	/* None of the above with new line codes								*/
#define DX							8	
#define M3G							9
#define M3GEV						10
#define MM							11
#define MMEV						12
#define DM							13
#define DMEV						14
#define APP_STATE				15


/* Inbound or Outbound Direction of device */

#define DIR_IN		1
#define DIR_OUT		2

/* Miscellaneous declarations */

#define YES					1
#define NO					0

// SKS
#define BOARD_DEVICE  999
#define ALL_DEVICES	  -1		/* This value is used to write the */
#define OUT_OF_RANGE	  -1		/* callindex is out of range */

/*
 * Technology Types
 */
#define ISDN					1	
#define SS7						2


#define MAX_QUEUE_SIZE 		 64
#define CALL_TYPE_VIDEO 		1
#define CALL_TYPE_AUDIO 		0

/* Global variables */
FILE					*boardLogFd;			
static char		logfile[] = "mrbt";		/* log filename without the .log	 */
static char		cfgfile[MAX_FILE_NAME];

static int		num_devices;		/* Number of devices loaded from cfg file */
static int		interrupted = NO;	/* Flag for user interrupted signals		 */
static int		reset_linedevs_left_to_complete = 0; 


/* Data structure which stores call information for each line device */
typedef struct {
	CRN					crn;							/* GlobalCall call handle */
	int					call_state;					/* state of first layer state machine */
	int					dropcall_active;			/* Flag for simultaneous disconnect check	*/
} CALL;

/* Data structure which stores all information for each line device */
static struct channel 
{
// SKS
	enum app_state_t app_state;
	int nPartnerIndex;
	char m3gDeviceName[MAX_DEVNAME];
	char mmDeviceName[MAX_DEVNAME];
	M3G_BOOL bVideoCall;
	int m3gControlH;
	int m3gAudioH;
	int m3gVideoH;
	int mmH;
	int remoteAudioLogicalChannel;
	int remoteVideoLogicalChannel;
	M3G_BOOL bPlayActive;
	M3G_BOOL bMediaStarted;
	M3G_BOOL bMediaIdle;
	int txPortCtr;
	int rxPortCtr;
	int localCapCtr;
	unsigned int olcEvtMask;
	int modifyMediaCtr;
	int stopMediaCtr;

	int dmevPortDisconnectCtr;
	int dmevPortConnectCtr;
	int dmevConnectCtr;
	int dmevDisconnectCtr;

	int openAudVidOlcCmplt;
	int respondAudVidOlcCmplt;

	M3G_BOOL	bStopTimerDigitSent;
	M3G_BOOL	bStartTimerDigitSent;

	M3G_BOOL bM3gAndDtiConnected;
	M3G_BOOL bM3gAndMmConnected;
	M3G_BOOL bM3gAndM3gConnected;
	M3G_BOOL bH245Started;

	M3G_BOOL	bMmTxAndM3gRxConnected;
  M3G_BOOL	bM3gTxAndMmRxConnected;
  M3G_BOOL	bM3gTxAndM3gRxConnected;


  M3G_SIMULTANEOUS_CAP_SET simultaneousCapabilityList;
	M3G_CAPS_LIST h223CapsList;
	M3G_CAPS_LIST audioCapsList;
	M3G_CAPS_LIST videoCapsList;
	
   DM_PORT_INFO               mmAudioTxPortInfo;
   DM_PORT_INFO               mmAudioRxPortInfo;
   DM_PORT_INFO               mmVideoTxPortInfo;
   DM_PORT_INFO               mmVideoRxPortInfo;
   DM_PORT_INFO               m3gAudioTxPortInfo;
   DM_PORT_INFO               m3gAudioRxPortInfo;
   DM_PORT_INFO               m3gVideoTxPortInfo;
   DM_PORT_INFO               m3gVideoRxPortInfo;
// SKS

// calling / called party information
	char				calledPartyNumber[GC_ADDRSIZE];
	char				originalCalledPartyNumber[GC_ADDRSIZE];
	char				callingPartyNumber[GC_ADDRSIZE];
	char				destination_num[GC_ADDRSIZE];

	unsigned char	handPhoneDCILen;
	unsigned char	handPhoneDCIValue[OCTET_STRING_SIZE];


// call related flags
	int 				callType;
	int 				bBusy;
	int 				bAcmSent;
	int 				bAnmSent;
	bool 				xcode;

	int 				bDisconnectActive;
// device handles
	int 				networkH;
	int					dxH;
	int					ipmH;
	LINEDEV			ldev;							/* GlobalCall line device handle							*/
	LINEDEV			mediah;						/* Media handle - only for H323                    */

	CALL				call[MAX_CALLS];

	M3G_E_VIDEO_TYPE cfgCoderType;

// device names
	char				netdevice[MAX_DEVNAME]; /* Network device name										*/
	char				protname[MAX_DEVNAME];	/* Protocol name												*/
	char				devname[MAX_DEVNAME];	/* Argument to gc_OpenEx() function						*/
 	char        ipmDeviceName[MAX_DEVNAME]; /* Voice device name                    */
  char        dxDeviceName[MAX_DEVNAME]; /* Voice device name                   */

  char        audioFileName[MAX_DEVNAME];
  char        videoFileName[MAX_DEVNAME];

	int					index;						/* Device index as loaded from cfg file				*/
	int					direction;					/* Inbound(WaitCall will be issued)
																or outbound(MakeCall will be issued)				*/
	int					blocked;						/* YES or NO													*/
	int					num_codecs;					/* The No. of codecs specified (VoIP device only) 	*/
	int					numb_calls;					/* The No of calls on the device							*/
	int					waitcall_active;			/* Flag for waitcall function for issuing only once*/
	int					makecall_active;
	int					resetlinedev_active;		/* Necessary since must ignore call related GC		*/
															/* events with gc_ResetLineDev() active				*/
	int					resetlinedev_required_after_unblocked; /* for ISDN									*/
															/* and some error conditions								*/
													
	int					makecall_timeout;			/* necessary since not all technologies support		*/
															/* async makecall timeout and they might be			*/
															/* different for different technologies				*/
	GC_MAKECALL_BLK	makecallblk;				/* Variable for MAKECALL block							*/
	FILE					*log_fp;						/* logfile Pointer											*/
	int					techtype;					/* technology type: E1ORT1_CAS, ISDN, ANALOG,		*/
} port[MAXCHAN];

static int		in_calls, out_calls;					/* Counters for Inbound calls and Outbound calls	*/
static int		gc_started = NO;						/* Flag is set if gc_Start() is Success				*/
time_t			start_time;								/* To note down the start time for the test			*/
/* Function prototype(s) */

// SKS

int _kbhit();
void  *createEventHandlerThread(void *context);
static pthread_t  gEventHandlerThreadID;
static void playAudioPrompt(int index);
static int getCallType(METAEVENT *metaevent,struct channel *pline);

static void stopPlayAudioPrompt(int index);

static void releaseCall(int index);

static void 	initiate_disconnect(int index);
static void 	sendAcceptCall(int index);
static void 	sendAnswerCall(int index);
static void		init_srl_mode(void);							/* inits SRL mode */
static void		load_config_file(void);						/* Loads Configuration File									*/
static void		printandlog(int index, int log_type,	/* Prints and logs												*/ 
													METAEVENT *metavent, char *msg_str, int callindex);
static void		makeCall(int index);				/* Function for gc_MakeCall()									*/ 
static void		exitdemo(int exitlevel);					/* Exit routine													*/
static void		initiate_exit(void);
static void		intr_hdlr(void);								/* For handling user interrupted signals					*/
static void		enable_alarm_notification(struct channel *pline);/* enable alarm notification                */
static void		open_isdn_channel(int index);	/* Technology specific initialization for ISDN			*/
static void		open_ss7_channel(int index);	/* Technology specific initialization for SS7			*/

static void		open_all_devices(void);						/* Function invokes Technology specific initilization 
																			functions to open all devices								*/
static void		open_device(int index);						/* Function invokes Technology specific initilization 
																			functions to open specific device						*/
static void		inbound_application(struct channel *pline); /* PLACE HOLDER FOR YOUR APPLICATION CODE			*/
static void		outbound_application(struct channel *pline); /* PLACE HOLDER FOR YOUR APPLICATION CODE			*/
static void		print_alarm_info(METAEVENTP metaeventp, struct channel *pline);
static void		process_event(void);							/* Function handles the GlobalCall Events					*/
static void		process_connected_event(struct channel *pline, METAEVENT *metaeventp);
static void		process_disconnected_event(struct channel *pline, METAEVENT *metaeventp);
static void		process_offered_event(METAEVENT *metaeventp, struct channel *pline);
static void		process_call_related_inbound_event(struct channel *pline, METAEVENT *metaeventp); /* Function handles the GlobalCall Events for inbound */
static void		process_call_related_outbound_event(struct channel *pline, METAEVENT *metaeventp); /* Function handles the GlobalCall Events for outbound */
static int		open_all_logfiles(void);					/* Function to open the logfiles								*/
static void		append_call_state_name(char *buffer, int index, int callindex);	/* append call state name to buffer	*/
static void		format_error_info(char *dest_buffer);		/* formats error information									*/
static void		format_result_info(int index, METAEVENT *metaeventp, /* formats event result information									*/
											char *dest_buffer);
static int		existCRN(CRN crn, struct channel *pline);	/* Retrieve CRN if available */
// SKS
static void  	process_non_gc_event(METAEVENT *metaevent);
// SKS
static int m3gBoardH;
int m3gBoardParmCtr;
char strCoderType[5][21]={"M3G_E_G7231","M3G_E_GSM_AMR_NB","M3G_E_H263","M3G_E_MPEG4"};

static char    dirStr[5][15]={"M3G_E_IDLE","M3G_E_TX","M3G_E_RX","M3G_E_TXRX"};
unsigned int eventsReqiredBefOlc = (M3GEV_MSD_ESTABLISHED+M3GEV_LOCAL_TCS_ACKD+M3GEV_REMOTE_TCS_RCVD);

static void disconnect_partner_3g_call(int index);
static void process_end_session_received(int index);
static void process_framing_lost(int index);
static void process_answered_event(int index);
static void process_3g_call_disconnect(int index);
static void process_dmev_connect(int index);
static void process_dmev_port_connect(int index);
static void process_dmev_disconnect(int index);
static void process_dmev_port_disconnect(int index, int devH);
static void process_h245_started(int index);
static void process_h245_stopped(int index);

static void process_modify_media_complete(int index,M3G_E_DIRECTION *direction);
static void process_stop_media_complete(int index);
static void process_mrbt_play_started(int index);
static void process_mrbt_play_stopped(int index);

static void printLocalCapability(int index);
static void cleanResourcesBeforeExit();
static void m3gEnableDisableEvents(int index,int handle);
static void resetVariables(int index);
static void stopPlay(int index);
static void   stopMedia(int index);
static void		playVideoPrompt(int index);
static int 		InitM3g(int index);
static void		stopM3g(int index);
static void		resetM3gDevice(int index,int handle);
static void		open_m3g_channel(int index);	/* Technology specific initialization for ISDN			*/
static void		open_mm_channel(int index);	/* Technology specific initialization for ISDN			*/
static void 	m3gSetParameter(int index,int handle);
static void 	getLocalCapability(int index,int handle);
static void 	storeLocalCapability(int index,int handle,M3G_CAPS_LIST *pDefaultCapsList);
static void 	setLocalCapability(int index);
static void 	connectM3gAndDti(int index);
static void 	disconnectM3gAndDti(int index);
static void 	getTxRxPortInfo(int index,int handle);
static void storeTxPortInfo(int index,METAEVENT *metaevent);
static void storeRxPortInfo(int index,METAEVENT *metaevent);
static void routeUnrouteM3gAndMmDevice(int index,int conType,int flag);
static void routeUnrouteM3gAndM3gDevice(int index,int conType, int flag);
static void printPortInfoList(int index,DM_PORT_INFO_LIST* pPortInfoList);
static void printPortInfo(int index,DM_PORT_INFO* pPortInfo,char str[5]);
static void startH245(int index);
static void stopH245(int index);
static void	sendH245UII(int index);
static void	sendH245VFU(int index);
static void startM3gTrace(int index,int handle);
static void stopM3gTrace(int index,int handle);
static void openM3gMmChannel();
static void getMatchedCapabilityAndOpenLC(int index);
static void modifyMedia(int index,M3G_E_DIRECTION direction);
static void getMatchedCapability(int index,int handle,M3G_CAPS_LIST *capList);
static void sendOpenLC(int index);
static void sendOLCAck(int index,M3G_REMOTE_OLC_REQ* remoteOlcReq);
static void initiate_exit_video_call();
static void routeDtiAndDx(int index);
static void unRouteDtiAndDx(int index);
static void   openMediaDevice(int index);
static void   getNetworkHandle(int index);
static void		set_calling_num(struct channel *pline, char *phone_num);
unsigned char AsciiOctetToHex(const char *pOctet);
static void getDxDeviceName(int index);
#endif
