/***************************************************************************
* C Source:	     mrbt.c
* Version: v1.1
* Description:   This is a demonstration program to demonstrate MRBT call flow, which 
*					  receive an incoming video call and make an outgoing video call, when B party
*						alerting it either answer the A-party call first or start playing MRBT without
*						without sending answer if switch permits. Once the B-party answers the call, it 
*						stop playing MRBT and connect A and B party audio/video media streams. Once any 
* 					of party hangs up the call, both legs will be cleared. 
*					  which executes the program continuously until it encounters signal ctrl+c.
* General Notes: 1. The tab setting used for this file is 2. 
*					  2. Pagination is done with Courier New, 10 point, 64 lines per page
*					  3. open_isdn_channel or open_ss7_channel may need to be modified 
*							 for user specific environment
*
*	%created_by:	sks %
*	%date_created: Wed July 22 11:11:09 2009 %
*
***************************************************************************/
/**********************************************************************
* DIALOGIC CONFIDENTIAL
* Copyright © 2007 Dialogic Corporation. All Rights Reserved.
*
*  All names, products, and services mentioned herein are the 
*  trademarks or registered trademarks of their respective 
*  organizations and are the sole property of their respective owners.
**********************************************************************/
#include "mrbt.h"

#ifndef lint
static char *_csrc = "@(#) %filespec: mrbt.c-28 %  (%full_filespec: mrbt.c-28:csrc:gc#1 %)";
#endif

/********************************************************************************
 *			NAME: main ()
 *DESCRIPTION: This is the entry point. 
 *					It loads the configuration file.
 *					Take cares the	 Event Handling process.
 *					Starts GC Library.
 *					Opens Configured Devices.
 *					Waits for user interrupted signal (ctrl+c).
 *					Whenever an event is received on any of the opened 
 *					devices, Depending upon the event received the proper 
 *					action will be taken.
 *		  INPUT: None
 *		RETURNS: 0 if success else -1
 ********************************************************************************/
int main(int argc, char *argv[])
{
	int						i;
	char					str[MAX_STRING_SIZE], str1[MAX_STRING_SIZE];
	GC_CCLIB_STATUSALL	cclib_status_all;

	/* Note down the start time for the test */
	time(&start_time);
	
	/* Give control to Event Handler */ 
#ifdef _WIN32
	signal(SIGINT, (void (__cdecl*)(int)) intr_hdlr);
	signal(SIGTERM, (void (__cdecl*)(int)) intr_hdlr);
#else
	signal(SIGHUP, (void (*)()) intr_hdlr);
	signal(SIGQUIT, (void (*)()) intr_hdlr);
	signal(SIGINT, (void (*)()) intr_hdlr);
	signal(SIGTERM, (void (*)()) intr_hdlr);
#endif

#ifdef MRBT_AFTER_ANM
	printf("\t\t\t\t********** JAI HO ***********\n");
#endif

  if ( argc >= 3 ) {
     	printf("Usage: ./mrbt <mrbt_config.cfg>\n");
			exit(-1);
  }

  if ( argc == 2 ) {
      strcpy(cfgfile,argv[1]);
      if ( strlen(cfgfile)<5 ) {
     			printf("Usage: ./mrbt <mrbt_config.cfg>\n");
					exit(-1);
      }
  }
	if(argc==1) {
		strcpy(cfgfile,"mrbt_config.cfg");
	}

	init_srl_mode();									/* set SRL mode to ASYNC, polled */

	/* Load Configuration file */
	load_config_file();

	/* Function to open all the logfiles */
	open_all_logfiles();
	if (num_devices == 0) {
		printf("No devices enabled - please modify mrbt_config.cfg to enable desired technology\n");
		exitdemo(1);
	}
	printandlog(ALL_DEVICES, MISC, NULL, "********** MRBT DEMO ***********\n", 0);
	
	/* Start GlobalCall */
	if (gc_Start(NULL) != GC_SUCCESS) { 
		sprintf(str, "gc_Start(startp = NULL) Failed");
		printandlog(ALL_DEVICES, GC_APIERR, NULL, str, 0);
		exitdemo(1);
	}
	sprintf(str, "gc_Start(startp = NULL) Success");
	printandlog(ALL_DEVICES, GC_APICALL, NULL, str, 0);

	/* Set the flag gc_started to YES */
	gc_started = YES;

	/* this code is useful for installation problem debugging */
	/* you may not wish to use this code in production systems */
	if (gc_CCLibStatusEx("GC_ALL_LIB", &cclib_status_all) != GC_SUCCESS) {
		sprintf(str, "gc_CCLibStatusEx(GC_ALL_LIB, &cclib_status_all) Failed");
		printandlog(ALL_DEVICES, GC_APIERR, NULL, str, 0);
		exitdemo(1);
	}	
	strcpy(str, " Call Control Library Status:\n");
	for (i = 0; i < GC_TOTAL_CCLIBS; i++) {
		switch (cclib_status_all.cclib_state[i].state) {
			case GC_CCLIB_CONFIGURED:
				sprintf(str1, "		  %s - configured\n", cclib_status_all.cclib_state[i].name);
				break;

			case GC_CCLIB_AVAILABLE:
				sprintf(str1, "		  %s - available\n", cclib_status_all.cclib_state[i].name);
				break;

			case GC_CCLIB_FAILED:
				sprintf(str1, "		  %s - is not available for use\n", cclib_status_all.cclib_state[i].name);
				break;

			default:
				sprintf(str1, "		  %s - unknown CCLIB status\n", cclib_status_all.cclib_state[i].name);
				break;


		}
		strcat(str, str1);
	}
	printandlog(ALL_DEVICES, MISC, NULL, str, 0);

// SKS create separate thread for handling events.
   if (pthread_create(&gEventHandlerThreadID, NULL,createEventHandlerThread, NULL)!=0 )
   {
      printf("pthread_create failure \n");
      exit(-1);
   }

	/* open all the GC devices */
	open_all_devices(); 

	// SKS
	if(InitM3g(BOARD_DEVICE)<0) {
      printf("Can't continue there is an error while initializing video devices \n");
			exit(-1);	
	}

	char ch;
  int isHit = false;

	while ( interrupted==NO )
  {
			if ( _kbhit() )
      {
         ch = getchar();
         isHit = true;
      } else {
         isHit = false;
			}
			if(isHit)
			{
				 	switch ( ch ) 
				 	{
						case 'q':
						case 'Q':
						{
							interrupted=YES;
							break;
						}
						default:
							break;
					}
			}
			fflush(stdout);
			usleep(1000);
	}	//end of while loop

   pthread_join(gEventHandlerThreadID,0);

	return 0;
} /* End of Main */

// Funcition Descriptions
unsigned char AsciiOctetToHex(const char *pOctet)
{
   unsigned char hexDigit = 0;
   unsigned char value = 0;
	unsigned int numDigits=0;

   for (numDigits = 0; numDigits < 2; numDigits++, pOctet++)
   {
      if (*pOctet >= '0' && *pOctet <= '9')
         hexDigit = *pOctet - '0';
      // lineBuffer was converted to upper case, but for flexibility in reuse:
      else if (*pOctet >= 'a' && *pOctet <= 'f')
         hexDigit = (*pOctet - 'a') + 10;
      else if (*pOctet >= 'A' && *pOctet <= 'F')
         hexDigit = (*pOctet - 'A') + 10;

      value =  (value << 4) + hexDigit;
   }

   return value;
}



/****************************************************************
*			NAME: void init_srl_mode(void)
* DESCRIPTION: Inits SR mode
*		RETURNS: None
*	  CAUTIONS: NA
****************************************************************/
static void init_srl_mode(void)
{
/* 
SRL MODEL TYPE - Single Threaded Async, event handler will be called in the Main thread 
*/
#ifdef _WIN32
	int	mode = SR_STASYNC | SR_POLLMODE; 
#else
	int	mode = SR_POLLMODE;
#endif
	

	/* Set SRL mode */
#ifdef _WIN32
	if (sr_setparm(SRL_DEVICE, SR_MODELTYPE, &mode) == -1) {
		printandlog(ALL_DEVICES, NON_GC_APIERR, NULL, "Unable to set to SR_STASYNC | SR_POLLMODE", 0);
		exitdemo(1);
	}
	printandlog(ALL_DEVICES, MISC, NULL, "SRL Model Set to SR_STASYNC | SR_POLLMODE", 0);
#else
	if (sr_setparm(SRL_DEVICE, SR_MODEID, &mode) == -1) {
		printandlog(ALL_DEVICES, NON_GC_APIERR, NULL, "Unable to set mode ID to SR_POLLMODE ", 0);
		exitdemo(1);
	}
	
	printandlog(ALL_DEVICES, MISC, NULL, "SRL mode ID set to SR_POLLMODE", 0);
#endif
}

/****************************************************************
*			NAME: open_all_logfiles(void)
* DESCRIPTION: Function to Open the Logfiles and put file pointers in file 
*					pointer field of port array. Later depending upon the index the
*					particular file can be accessed to dump the data.
*					If the network device name is present, then the log file
*					uses the network device name
*					else it uses the voice device name
*		RETURNS: returns 1 if successful
*	  CAUTIONS: Assumes device name includes network device name or a voice device name
****************************************************************/
static int open_all_logfiles(void)
{	
	char		temp_filename[MAX_DEVNAME], temp_str[MAX_STRING_SIZE];
	char		*i;
	int		index;

	for (index = 0; index < num_devices; index++) {
		/* build up temp_filename */
		strcpy(temp_filename, logfile);

		/* case 1: if dti name exists, use it */
		if (strcmp(port[index].netdevice, "NONE") != 0) {
			strcat(temp_filename,  "_");
			strcat(temp_filename, port[index].netdevice);
			temp_str[0]='\0';
		}
		/* case 2: if voice name exists, use it */
		else if (strcmp(port[index].dxDeviceName, "NONE") != 0) {
			i = (char *)strstr(port[index].dxDeviceName, "B");
			strcpy(temp_str, i);
			strcat(temp_filename,  "_");
			strcat(temp_filename, temp_str);
		} else {
			printf("Missing both DTI and Voice resource in config file device name (%s)\n",
					port[index].devname);
			exitdemo(1);
		}		
		strcat(temp_filename, ".log");

		if ((port[index].log_fp = fopen(temp_filename, "w")) == NULL) {
			printf("\nCannot open log file %s for write\n", temp_filename);
			perror("Reason is: ");
			exitdemo(1);
		}
		printf("%s successfully opened\n", temp_filename); 
		
	} /* End of for loop */

	boardLogFd=fopen("mrbt_board.log","w");
	return 1;
}
/****************************************************************
*   NAME: void *createEventHandlerThread(void *context)
* 	DESCRIPTION: event handler thread
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/

void *createEventHandlerThread(void *context)
{
	int ret;
	/* 
	-- Forever loop where the main work is done - wait for an event or user requested exit
	*/
	for (;;) {
		ret = sr_waitevt(50);					/* 1/2 second */
		if (ret != -1) {							/* i.e. not timeout */
			process_event();
		}
		if (interrupted == YES) {				/* flag set in intr_hdlr() */
			initiate_exit_video_call();
			if (reset_linedevs_left_to_complete == 0) {
				exitdemo(1);
			}
		}
	}
}

/****************************************************************
*   NAME: void stopPlayAudioPrompt(int index)
*   DESCRIPTION: stop playing audio prompt for audio call
*		INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/

static void stopPlayAudioPrompt(int index)
{
    if(dx_stopch(port[index].dxH,EV_SYNC)==-1) {
      printandlog(index,MISC,NULL,"dx_stopch() failed",0);
    }
    else {
      printandlog(index,MISC,NULL,"dx_stopch() success",0);
    }
}

/****************************************************************
*   NAME:  void playAudioPrompt(int index)
*   DESCRIPTION: start play audio prompt for audio call
*		INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void playAudioPrompt(int index)
{
	int devH=port[index].dxH;
	int fdPrompt;	

	DX_IOTT iott;
	DV_TPT tpt;

	fdPrompt=open("intro.vox",O_RDONLY);

	/* set up DX_IOTT */
	iott.io_type = IO_DEV|IO_EOT;
	iott.io_bufp = 0;
	iott.io_offset = 0;
	iott.io_length = -1; /* play till end of file */
	iott.io_fhandle=fdPrompt;

	/* set up DV_TPT */
	dx_clrtpt(&tpt,1);
	tpt.tp_type = IO_EOT; /* only entry in the table */
	tpt.tp_termno = DX_MAXDTMF; /* Maximum digits */
	tpt.tp_length = 4; /* terminate on four digits */
	tpt.tp_flags = TF_MAXDTMF; /* Use the default flags */

	/* clear previously entered digits */
	if (dx_clrdigbuf(devH) == -1) {
		/* process error */
	}

	/* Now play the file */
	if (dx_play(devH,&iott,&tpt,EV_ASYNC|PM_SR6) == -1) {
		printandlog(index,MISC,NULL,"dx_play() failed",0);
	}
	else {
		printandlog(index,MISC,NULL,"dx_play() success",0);
	}
}

/****************************************************************
*			NAME: inbound_application(struct channel *pline)
* DESCRIPTION: Place holder for your inbound application code in the connected state
*		  INPUT: pline - pointer to port data structure
*		RETURNS: NA
*	  CAUTIONS: none
****************************************************************/
static void inbound_application(struct channel *pline)
{

	int index;
	index=pline->index;
	if(port[index].bDisconnectActive==YES) {
        return;
	}
	if(port[index].callType==CALL_TYPE_VIDEO) {
		startH245(index);
	} else {
		playAudioPrompt(index);
	}
}
/****************************************************************
*			NAME: outbound_application(struct channel *pline)
* DESCRIPTION: Place holder for your outbound application code in the connected state
*		  INPUT: pline - pointer to port data structure
*		RETURNS: NA
*	  CAUTIONS: none
****************************************************************/
static void outbound_application(struct channel *pline)
{

  int index;
  index=pline->index;
	if(port[index].bDisconnectActive==YES) {
        return;
  }
   if(port[index].callType==CALL_TYPE_VIDEO) {
     	startH245(index);
   } else {
     	playAudioPrompt(index);
   }
}

/****************************************************************
*			NAME: process_event(void)
* DESCRIPTION: Function to handle the GlobalCall Events 
*		RETURNS: NA
*	  CAUTIONS: none
****************************************************************/
static void process_event(void)
{
	METAEVENT			metaevent;
	struct channel		*pline;
	char					str[MAX_STRING_SIZE];
	int					index;				/* Device index */
	int					evttype;
	GC_INFO				t_info;	  			/* Structure which stores information about GC related errors */ 
	int					callindex;			/* CAUTION: once initialized, do not change */
													/* the value in this subroutine! */
	int					temp_callindex;	/* use the temporary callindex instead */
	
	/* Populate the metaEvent structure */
	if(gc_GetMetaEvent(&metaevent) != GC_SUCCESS)
	{
		/* serious problem - should never fail */
		printandlog(ALL_DEVICES, GC_APIERR, NULL, "gc_GetMetaEvent() failed", 0);
		exitdemo(1);
	}

	/* process GlobalCall events */
	if ((metaevent.flags & GCME_GC_EVENT) == 0)	{
		process_non_gc_event(&metaevent);
		return;												/* RETURN POINT */
	}
	/**************************************/
	/* when here => processing a GC event */
	/**************************************/
	/* initialize pline to port[index],
	The pointer to this structure was stored in GC
	during gc_OpenEx() */
	pline = (struct channel *) metaevent.usrattr;
	
	/* Retrieve device index */
	index = pline->index;

	evttype = metaevent.evttype;



	/* skip checks if no crn for this event */

	if ((metaevent.crn != 0) && (pline->resetlinedev_active == NO)) {
		/* Assumption: Only GCEV_DETECTED/GCEV_OFFERED will require the CRN
	   	to be inserted into the array. */
		callindex = existCRN(metaevent.crn, pline);
		if (callindex == OUT_OF_RANGE) {
			/* CRN was not found, so insert CRN into the array.
			 * Should only occurs when GCEV_DETECTED or GCEV_OFFERED event is received.
			 * Note: GCEV_DETECTED is not enabled, but this code will support it if
			 * if you enable this event */
			if ((evttype == GCEV_DETECTED) || (evttype == GCEV_OFFERED)) {
				for (callindex = 0; callindex < MAX_CALLS; callindex++) {
					if (pline->call[callindex].crn == 0) {
						/* crn is not found, insert into crn array. */
						pline->call[callindex].crn = metaevent.crn;
						break;
					}
				}
			} else {
				/* Error case: should not happen */
				sprintf(str, " Received an event(0x%x) with an invalid CRN(0x%lx)", evttype, metaevent.crn);
				printandlog(index, MISC_WITH_NL, NULL, str, 0);
				return;
			}
		}
	} else {
		callindex = 0;						/* some printing routines use callindex */
	}
	/**********************************************************************/
	/* CAUTION: at this point callindex is initialized, do not change it! */
	/**********************************************************************/

	printandlog(index, MISC_WITH_NL, NULL, "********* Received a GC event *********", 0);
	printandlog(index, EVENT, NULL, GCEV_MSG(evttype), 0);
	printandlog(index, STATE, NULL, " is the current GC call state ", callindex);

	/* 1st handle the generic events */
	switch (evttype) 
	{
		case GCEV_OPENEX:
			printandlog(index, MISC, NULL, "GCEV_OPENEX received", 0);

			break;

		case GCEV_OPENEX_FAIL:
			printandlog(index, MISC, NULL, "GCEV_OPENEX_FAIL received", 0);
			printandlog(index, GC_RESULT_INFO, &metaevent, "Reason for GCEV_OPENEX_FAIL ", 0);
			exitdemo(1);
			break;
		case GCEV_ALARM:
			print_alarm_info(&metaevent, pline);
			break;

		case GCEV_FATALERROR:
			printandlog(index, GC_RESULT_INFO, &metaevent, "Reason for fatalerror ", 0);
			printandlog(index, STATE, NULL, " ", callindex);

			if (gc_ResultInfo(&metaevent, &t_info) < 0) {
				sprintf(str, "gc_ResultInfo() call failed");
				printandlog(index, GC_APIERR, NULL, str, 0);
				exitdemo(1);
			}

			if (t_info.gcValue == GCRV_RESETABLE_FATALERROR) {
				pline->resetlinedev_active = YES;					/* reset linedevice is on as a background process */
				/* do nothing else= wait till the GCEV_RESETLINEDEV completion event arrives */
			} else if (t_info.gcValue == GCRV_RECOVERABLE_FATALERROR) {
				/* do a close and open on recoverable, fatal errors */
				printandlog(index, MISC, NULL, " Recoverable fatal error occurred - closing device and re-opening it", 0);
				if (gc_Close(port[index].ldev) < 0) {
					printandlog(index, GC_APIERR, NULL, "gc_Close() failed", 0);
					exitdemo(1);
				}
				open_device(index);

			} else {
				printandlog(index, MISC, NULL, " Non-recoverable fatal error occurred", 0);
				exitdemo(1);
			}
			break;				

		case GCEV_BLOCKED:
			pline->blocked = YES;						/* Nothing to do when a blocked event occurs */
			break;
			
		case GCEV_UNBLOCKED:
			/* blocked/unblocked do not cause state transitions */
			pline->blocked = NO;
			if (pline->resetlinedev_required_after_unblocked == YES) {
				pline->resetlinedev_required_after_unblocked = NO;

				if (gc_ResetLineDev(port[index].ldev, EV_ASYNC) != GC_SUCCESS) {
					sprintf(str, "gc_ResetLineDev(linedev=%ld, mode=EV_ASYNC) Failed", port[index].ldev);
					printandlog(index, GC_APIERR, NULL, str, 0);
					exitdemo(1);
				}
				pline->resetlinedev_active = YES;
				sprintf(str, "gc_ResetLineDev(linedev=%ld, mode=EV_ASYNC) Success", port[index].ldev);
				printandlog(index, GC_APICALL, NULL, str, 0);
				/* nothing more can do until the completion event arrives */
				break;
			}

			if (pline->waitcall_active == YES) {
				/* waitcall already active - do not call it again */
				break;
			}
			if (pline->resetlinedev_active == YES) {
				/* The line device is being reset */
				break;
			}
			/* FALL THROUGH ON PURPOSE */

		case GCEV_RESETLINEDEV:
			if (evttype == GCEV_RESETLINEDEV) {
				/* Reset call related flags and variables */
				pline->waitcall_active = NO;
				pline->makecall_active = NO;
				pline->resetlinedev_active = NO;
				for (temp_callindex = 0; temp_callindex < MAX_CALLS; temp_callindex++) {
					/* reset internal variables to that of beginning */
					pline->call[temp_callindex].call_state = GCST_NULL;
					pline->call[temp_callindex].crn = 0;
					pline->call[temp_callindex].dropcall_active = NO;
				}
			}

			if ((evttype == GCEV_RESETLINEDEV) 
			&& (reset_linedevs_left_to_complete != 0)) {		/* tests if processing shutdown request */
				reset_linedevs_left_to_complete--;
				sprintf(str, "One less GCEV_RESETLINEDEV to wait for, %d left",
					reset_linedevs_left_to_complete);
				printandlog(ALL_DEVICES, MISC, NULL, str, 0);

			}

			/* must further process the event if unblocked */
			if ((interrupted == NO) && (pline->blocked == NO)) {
				if (pline->direction == DIR_IN) {
					/* Note: by definition, waitcall is not active at this point */
					if (pline->blocked == NO) {
						if (gc_WaitCall(pline->ldev, NULL, NULL, -1, EV_ASYNC) != GC_SUCCESS) {
							sprintf(str, "gc_WaitCall(linedev=%ld, crnp=NULL, waittime=0, mode=EV_ASYNC) Failed", pline->ldev);
							printandlog(index, GC_APIERR, NULL, str, 0);
							exitdemo(1);
						}
						pline->waitcall_active = YES;
						sprintf(str, "gc_WaitCall(linedev=%ld, crnp=NULL, waittime=0, mode=EV_ASYNC) Success", pline->ldev);
						printandlog(index, GC_APICALL, NULL, str, 0);
					}
				} else {
				}
			}
			break;

		case GCEV_TASKFAIL:
			printandlog(index, GC_RESULT_INFO, &metaevent, "Reason for taskfail ", 0);
			printandlog(index, STATE, NULL, " ", callindex);

			/* best can do is a reset linedevice */
			/* if reset lindevice is already active, then retry it */
			if (gc_ResetLineDev(port[index].ldev, EV_ASYNC) == GC_SUCCESS) {
				pline->call[callindex].dropcall_active = NO;		/* in case on */
				pline->makecall_active = NO;							/* in case on */
				pline->resetlinedev_active = YES;
				sprintf(str, "gc_ResetLineDev(linedev=%ld, mode=EV_ASYNC) Success", port[index].ldev);
				printandlog(index, GC_APICALL, NULL, str, 0);
			} else {
				sprintf(str, "gc_ResetLineDev(linedev=%ld, mode=EV_ASYNC) Failed", port[index].ldev);
				printandlog(index, GC_APIERR, NULL, str, 0);
				if (pline->blocked == NO) {							/* If no alarms are active */
					exitdemo(1);											/* Then should never get here */
				}
				initiate_disconnect(index);
				initiate_disconnect(port[index].nPartnerIndex);
			}
			break;

		case GCEV_DISCONNECTED:
		{
			printandlog(index, STATE, NULL, " GCEV_DISCONNECTED Received\n", callindex);
			if (pline->resetlinedev_active == NO) {
					if(port[index].callType==CALL_TYPE_AUDIO) {
              stopPlayAudioPrompt(index);
              unRouteDtiAndDx(index);
							process_disconnected_event(pline, &metaevent);	/* process the disconnected event */
					} else {		// else call type video
								process_3g_call_disconnect(index);
					}	
				}
			break;
		}
		default:
			if (pline->resetlinedev_active == NO) {	/* ignore all call related events with reset linedevice active */
				if (pline->direction == DIR_IN) {
					process_call_related_inbound_event(pline, &metaevent);
				} else {
					process_call_related_outbound_event(pline, &metaevent);
				}
			}
			break;
	}
	printandlog(index, STATE, NULL, " is the new GC call state after processing the event", callindex);
}

/****************************************************************
*   NAME:  void initiate_disconnect(int index)
*   DESCRIPTION: application initiated drop call 
*		INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void initiate_disconnect(int index)
{
  printandlog(index, MISC, NULL, "initiate_disconnect()",0);
  struct channel    *pline;
	int callindex=0;
	char					str[MAX_STRING_SIZE];


	if(port[index].call[callindex].call_state==GCST_NULL || port[index].call[callindex].call_state==GCST_DISCONNECTED)
	{
			return;
	}

// SKS assume glare condition never happens subjected 
//     incoming and outgoing cic are in two different group
  pline = &port[index];

        /* when here, resetlinedev failed and in the blocked state */
        /* most likely due to an alarm being active */
        if (pline->call[callindex].crn) {         /* is there an active call? */
          if (pline->call[callindex].dropcall_active == NO) {         /* drop and release the call if can */
            if (gc_DropCall(pline->call[callindex].crn, GC_NORMAL_CLEARING, EV_ASYNC) == GC_SUCCESS)
            {
              sprintf(str, "gc_DropCall(crn=0x%lx, cause=GC_NORMAL_CLEARING, mode=EV_ASYNC) Success", pline->call[callindex].crn);
              printandlog(pline->index, GC_APICALL, NULL, str, 0);
              pline->call[callindex].dropcall_active = YES;
            } else {
              sprintf(str, "gc_DropCall(crn=0x%lx, cause=GC_NORMAL_CLEARING, mode=EV_ASYNC) Failed", pline->call[callindex].crn);
              printandlog(pline->index, GC_APIERR, NULL, str, 0);
              printandlog(pline->index, STATE, NULL, " ", callindex);
            }
          }
					else {
              printandlog(pline->index, STATE, NULL, " There is one drop call already active\n",callindex);
					}
        }
				else {
              printandlog(pline->index, STATE, NULL, " There is no active call\n",callindex);
				}
        pline->resetlinedev_required_after_unblocked = YES;   /* try again later */
				pline->call[callindex].call_state = GCST_DISCONNECTED;
}
/****************************************************************
*   NAME: void releaseCall(int index)
*   DESCRIPTION: release the call upon receiving GCEV_DROPCALL
*		INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void releaseCall(int index)
{
  struct channel    *pline;
	int callindex=0;
	char					str[MAX_STRING_SIZE];

// SKS assume glare condition never happens subjected 
//     incoming and outgoing cic are in two different group
  pline = &port[index];
  pline->call[callindex].dropcall_active = NO;
	pline->makecall_active = NO;

  /* Call ReleaseCall */
  pline->call[callindex].call_state = GCST_IDLE;

  if (gc_ReleaseCallEx(pline->call[callindex].crn, EV_ASYNC) != GC_SUCCESS) {
      sprintf(str, "gc_ReleaseCallEx(crn=0x%lx, EV_ASYNC) Failed", pline->call[callindex].crn);
      printandlog(index, GC_APIERR, NULL, str, 0);
      exitdemo(1);
  }
  sprintf(str, "gc_ReleaseCallEx(crn=0x%lx, EV_ASYNC) Success", pline->call[callindex].crn);
  printandlog(index, GC_APICALL, NULL, str, 0);
	int prev_state=port[index].app_state;
	resetVariables(index);
	if(prev_state!=port[index].app_state) {
		sprintf(str,"Application states after event processing Prev_State=%s | current State=%s",appStatesStr[prev_state],appStatesStr[port[index].app_state]);
  	printandlog(index,M3GEV,NULL,str,0);
	}
  fflush(stdout);
}

/****************************************************************
*			NAME: void	print_alarm_info(METAEVENTP metaeventp, struct channel *pline)
* DESCRIPTION: Prings alarm information
*		 INPUTS: metaeventp - pointer to the alarm event
*              pline - pointer to the channel data structure
*		RETURNS: NA
*	  CAUTIONS: Assumes already known to be an alarm event
****************************************************************/
static void	print_alarm_info(METAEVENTP metaeventp, struct channel *pline)
{
	long					alarm_number;
	char					*alarm_name;
	unsigned long		alarm_source_objectID;
	char					*alarm_source_object_name;
	char					str[MAX_STRING_SIZE];

	if (gc_AlarmNumber(metaeventp, &alarm_number) != GC_SUCCESS)
	{
		sprintf(str, "gc_AlarmNumber(...) FAILED");
		printandlog(pline->index, GC_APIERR, NULL, str, 0);
		printandlog(pline->index, STATE, NULL, " ", 0);
		exitdemo(1);
	}
	if (gc_AlarmName(metaeventp, &alarm_name) != GC_SUCCESS)
	{
		sprintf(str, "gc_AlarmName(...) FAILED");
		printandlog(pline->index, GC_APIERR, NULL, str, 0);
		printandlog(pline->index, STATE, NULL, " ", 0);
		exitdemo(1);
	}

	if (gc_AlarmSourceObjectID(metaeventp, &alarm_source_objectID) != GC_SUCCESS)
	{
		sprintf(str, "gc_AlarmSourceObjectID(...) FAILED");
		printandlog(pline->index, GC_APIERR, NULL, str, 0);
		printandlog(pline->index, STATE, NULL, " ", 0);
		exitdemo(1);
	}


	if (gc_AlarmSourceObjectName(metaeventp, &alarm_source_object_name) != GC_SUCCESS)
	{
		sprintf(str, "gc_AlarmSourceObjectName(...) FAILED");
		printandlog(pline->index, GC_APIERR, NULL, str, 0);
		printandlog(pline->index, STATE, NULL, " ", 0);
		exitdemo(1);
	}

	sprintf(str, "Alarm %s (0x%lx) occurred on ASO %s (%d)",
				alarm_name, alarm_number, alarm_source_object_name, (int) alarm_source_objectID);

	printandlog(pline->index, MISC, NULL, str, 0);
}



/****************************************************************
*			NAME: process_disconnected_event(struct channel *pline, METAEVENT *metaeventp)
* DESCRIPTION: Function to process a disconnected event
*		 INPUTS: pline - pointer to entry in port table
*					metaeventp - pointer to the current metaevent
*		RETURNS: NA
*	  CAUTIONS: Assumes event logging already done
****************************************************************/
static void process_disconnected_event(struct channel *pline, METAEVENT *metaeventp)
{
	char		str[MAX_STRING_SIZE];
	int		callindex;
	int index=pline->index;

  printandlog(index, MISC, NULL, "process_disconnected_event()",0);
	/* Retrieve CRN */
	callindex = existCRN(metaeventp->crn, pline);
	if (callindex == OUT_OF_RANGE) {
		/* CRN not found, so log error and return from function. */
		sprintf(str, "CRN not found");
		printandlog(pline->index, MISC_WITH_NL, NULL, str, 0);
		return;
	}

	switch (pline->call[callindex].call_state)
	{
		case GCST_IDLE:
			/*************************************************************/
			/* this represents a simultaneous disconnect						 */
			/* do nothing as gc_DropCall() must have already been issued */
			/* to reach this point!													 */
			/*************************************************************/
			break;

		case GCST_NULL:
			/* In case have a disconnect before dialing is reached */
			if (pline->makecall_active == NO) {
				/* this represents an error */
				printandlog(pline->index, MISC, NULL, "Received GCEV_DISCONNECTED in NULL call state", 0); 

				/* however, there is a PT - 24402 - which indicates that the inbound side */
				/* may under alarm conditions receive a disconnected event */
				/* even though the call has not been offered */
				/* the other work-around is to enable the detected state, but */
				/* for purposes of showing a second work-around, we will "create" the call */
				/* and then drop and release it */
				pline->call[callindex].crn = metaeventp->crn;					/* "create the call */
																		/* the call state will be init in default */

				/* break; - when PT24402 is fixed, put this back in and pull the call "creation" */
			}
			/* fall through on purpose - need to do a dropcall and release call since there is a call active */
			/* may have reached this condition if there is a problem before dialing */
			/* or the inbound side has not enabled detection of events and the disconnected */
			/* event arrives (usually due to alarm conditions) without the app previously */
			/* being informed the call exists */

		default:
			initiate_disconnect(index);
			break;
	}
}

/****************************************************************
*			NAME: process_call_related_inbound_event(struct channel *pline, METAEVENT *metaeventp)
* DESCRIPTION: Function to process inbound GlobalCall event
*		 INPUTS: pline - pointer to entry in port table
*					metaeventp - pointer to the metaevent structure
*		RETURNS: NA
*	  CAUTIONS: Tightly coupled with process_event() for pre-processing
*					Assumes event logging already done
*					Disconnected event already handled
****************************************************************/
static void process_call_related_inbound_event(struct channel *pline, METAEVENT *metaeventp)
{
	int		evttype, index;
	char		str[MAX_STRING_SIZE];
	int		unexpected_event = 0;					/*assume not till proven otherwise */
																/* May be overridden for GCEV_UNBLOCKED */
	int		callindex;

	evttype = metaeventp->evttype;
	index = pline->index;
  printandlog(index, MISC, NULL, "process_call_related_inbound_event()",0);

	/* Retrieve CRN */
	callindex = existCRN(metaeventp->crn, pline);
	if (callindex == OUT_OF_RANGE) {
		/* CRN not found, so log error and return from function. */
		sprintf(str, "CRN not found");
		printandlog(pline->index, MISC_WITH_NL, NULL, str, 0);
		return;
	}
// SKS
		sprintf(str, "SKS Rx Event Type = 0x%x",evttype);
      printandlog(index, MISC_WITH_NL, NULL, str, 0);

	/* Implement the state machine */
	switch (pline->call[callindex].call_state)
	{
		/************************************/
		/* first come the call setup states */
		/************************************/
		case GCST_NULL:
		{
			switch (evttype)
			{
				case GCEV_OFFERED:
					sprintf(str, "GCEV_OFFERED received in GCST_NULL state on device=%s",port[index].netdevice);
					printandlog(pline->index, MISC_WITH_NL, NULL, str, 0);
					process_offered_event(metaeventp, pline);
					break;
				case GCEV_CALLINFO:
					sprintf(str, "GCEV_CALLINFO received in GCST_NULL state");
					printandlog(pline->index, MISC_WITH_NL, NULL, str, 0);
					break;
				default:
					unexpected_event = 1;
					break;
			}
			break;
			/* end of case GCST_NULL */
		}

		case GCST_OFFERED:
		{
				switch(evttype) 
				{
		   			case GCEV_CALLINFO:
						{
								sprintf(str, "GCEV_CALLINFO received in GCST_OFFERED state");
								printandlog(pline->index, MISC_WITH_NL, NULL, str, 0);
								break;
						}
						case GCEV_ACCEPT:
						{
								pline->call[callindex].call_state = GCST_ACCEPTED;

								if(port[index].callType==CALL_TYPE_AUDIO) {
										sendAnswerCall(index);
								} else {
										#ifndef MRBT_AFTER_ANM
												inbound_application(pline);
										#endif
								}
								break;
						}
						case GCEV_ANSWERED:
						{
							pline->call[callindex].call_state = GCST_CONNECTED;
							pline->numb_calls++;
							in_calls++;
							if(port[index].callType==CALL_TYPE_VIDEO) {
									process_answered_event(index);
							} else {
         	 			inbound_application(pline);
        			}
							break;
						}
						case GCEV_DROPCALL:
						{
								releaseCall(index);
								break;
						}
						default:
							unexpected_event = 1;
				}
			break;
		}		// GCST_OFFERED

		case GCST_ACCEPTED:
		{
				switch(evttype)
				{
							case GCEV_ANSWERED: 
							{
								pline->call[callindex].call_state = GCST_CONNECTED;
								pline->numb_calls++;
								in_calls++;
								if(port[index].callType==CALL_TYPE_VIDEO) {
										process_answered_event(index);
								} else {
										inbound_application(pline);
								}
								break;
							} 
							case GCEV_DROPCALL: 
							{
								break;
							}
							default:
								unexpected_event = 1;
			}
			break;
		}	// GCST_ACCEPTED
    case GCST_CONNECTED:
    {
      /*************************************************/
      /* Depending upon your application, you may or   */
      /* may not have application specific event     */
      /* handling here for when in the connected state */
      /* You could also have your app specific code  */
      /* in process_event()                  */
      /*************************************************/
      if (evttype != GCEV_DROPCALL) {     /* recall the outbound side issues a gc_DropCall() */
        unexpected_event = 1;         /* for the purposes of this demo */
        break;
      }
      /* FALL THROUGH ON PURPOSE since this is one of the 2 ways of reaching Idle */
      /* the other being a disconnected event */
    }


		/*************************************/
		/* now come the call teardown states */
		/*************************************/
		case GCST_DISCONNECTED:
		{
			/* If the GCEV_OFFERED event is observed in this state,
			   then add the code that in located in GCST_IDLE state
				to resolved the issue. */
				if (evttype == GCEV_DROPCALL) {
					releaseCall(index);
				} else if(evttype==GCEV_UNLISTEN) {
				} else {
					unexpected_event = 1;
				}
			break;
		}

		case GCST_IDLE:
		{
			/* In most cases, only GCEV_RELEASECALL will be seen in this state,
			   but GCEV_OFFERED can be received after gc_ReleaseCallEx() and
			   before GCEV_RELEASECALL. */
			if (evttype == GCEV_RELEASECALL) {
				pline->call[callindex].call_state = GCST_NULL;
				pline->call[callindex].crn = 0;
			} else if (evttype == GCEV_OFFERED) {
				process_offered_event(metaeventp, pline);
			} else {
				unexpected_event = 1;
			}
			break;
		}

		default:
		{
			/* Should never get here */
			printandlog(index, MISC, NULL, "Invalid state in process_call_related_inbound_event()", 0); 
			printandlog(index, STATE, NULL, " ", callindex);
			break;
		}
	}

	if (unexpected_event && (evttype != GCEV_UNBLOCKED)) {
		printandlog(index, GC_RESULT_INFO, metaeventp, "Event was unexpected in the current state", 0);
		printandlog(index, STATE, NULL, " ", callindex);
	}
}

// SKS get type(audio/video) of incoming call
/****************************************************************
*   NAME: int getCallType(METAEVENT *metaevent,struct channel *pline)
*   DESCRIPTION: upon GCEV_OFFERED this function will determine 
									whether incomig call is an audio or a video call
*		INPUT: METAEVENT, struct channel
*   RETURNS: CALL_TYPE_AUDIO | CALL_TYPE_VIDEO
*   CAUTIONS: NA
****************************************************************/
static int getCallType(METAEVENT *metaevent,struct channel *pline)
{
	int callType=CALL_TYPE_AUDIO;
	int 				nCtr;
	S7_IE_BLK      ie_blk;
	int 				ieType,ieLen;
	int 				x;
	int 				index=pline->index;
	int 				nAllIELen=0;
	char 				str[128];
	
	if(port[index].techtype==ISDN) {
		if (gc_GetSigInfo(pline->ldev, (char *)&ie_blk, U_IES, metaevent) != GC_SUCCESS)  { 
			// process error
			printandlog(pline->index, MISC, NULL, "gc_GetSigInfo() call failed", 0);
		}
	} else if(port[index].techtype==SS7) {
		if (gc_GetSigInfo(pline->ldev,(char *)&ie_blk, 0, metaevent) != GC_SUCCESS)  {
         // process error
         printandlog(pline->index, MISC, NULL, "gc_GetSigInfo() call failed", 0);
      }
	}
	nAllIELen=(int)ie_blk.length;
//	printf("All IE Length Decimal=%d | Hex=0x%x\n",nAllIELen,ie_blk.length);
	for (nCtr=0;nCtr<(nAllIELen-1);)  {
			ieType=(int)(unsigned char)ie_blk.data[nCtr];
			ieLen=(int)(unsigned char)ie_blk.data[nCtr+1];
//			sprintf(str,"IE Type=0x%x| IE Len=0x%x ==> |",(unsigned char)ie_blk.data[nCtr],(unsigned char)ie_blk.data[nCtr+1]);
			sprintf(str,"IE Type=0x%x| IE Len=0x%x ==> |",ieType ,ieLen);
			nCtr+=2;
			if(ieType==4 || ieType==124 || ieType==29 || ieType==3) {
					int val1, val2, val3;	
					val1=(int)(unsigned char)ie_blk.data[nCtr];
					val2=(int)(unsigned char)ie_blk.data[nCtr+1];
					val3=(int)(unsigned char)ie_blk.data[nCtr+2];
					if(val1==136 && val2==144 && val3==166) {
							callType=CALL_TYPE_VIDEO;
							return CALL_TYPE_VIDEO;
					}
					if(ieType==3) {
						val1=(int)(unsigned char)ie_blk.data[nCtr+2];
               	val2=(int)(unsigned char)ie_blk.data[nCtr+3];
               	val3=(int)(unsigned char)ie_blk.data[nCtr+4];
						if(val1==136 && val2==144 && val3==166) {
							callType=CALL_TYPE_VIDEO;
							return CALL_TYPE_VIDEO;
						}
					} 
				}
				for(x=0;x<ieLen;x++,nCtr++) {
					sprintf(str,"%s0x%x|",str, (unsigned char)ie_blk.data[nCtr]);
				}
				sprintf(str,"%s",str);
//				printandlog(pline->index, MISC, NULL, str,0);
	}
	return callType;
}

/****************************************************************
*			NAME: process_offered_event(struct channel *pline)
* DESCRIPTION: Function to process an offered event while in the NULL state
*		 INPUTS: pline - pointer to entry in port table
*		RETURNS: NA
*	  CAUTIONS: Assumes event logging already done
****************************************************************/
static void process_offered_event(METAEVENT *metaeventp, struct channel *pline)
{
	char		str[MAX_STRING_SIZE];
	int			callindex;
	int 		index;
	index=pline->index;
	port[index].bBusy=YES;

  printandlog(index, MISC, NULL, "process_offered_event()",0);
	/* Retrieve CRN */
	callindex = existCRN(metaeventp->crn, pline);
	if (callindex == OUT_OF_RANGE) {
		printandlog(index, MISC, NULL, "metaeventp->crn unexpectedly not found for a GCEV_OFFERED event", 0);
		exitdemo(1);
	}
	/* Save the CRN in port[index] and call Acceptcall */
	pline->call[callindex].call_state = GCST_OFFERED;
	pline->call[callindex].crn = metaeventp->crn;

	// SKS MRBT to drop the incoming call coming on outgoing channel index
	if(index>(num_devices/2)) {
			printandlog(index, MISC_WITH_NL, NULL, "Dropping call because it landed on outgoing index", callindex);
			initiate_disconnect(index);
			return;
	}

		/* first, get called party number */
		if (gc_GetCallInfo(pline->call[callindex].crn, DESTINATION_ADDRESS, pline->calledPartyNumber) == GC_SUCCESS) {
			sprintf(str, "gc_GetCallInfo(crn=0x%lx) Success - called party = %s", pline->call[callindex].crn, pline->calledPartyNumber);
		} else {
			sprintf(str, "gc_GetCallInfo(crn=0x%lx) Failure - called party not available ", pline->call[callindex].crn);
		}
		printandlog(pline->index, GC_APICALL, NULL, str, 0);

	/* next get the calling party here */
		if (gc_GetCallInfo(pline->call[callindex].crn, ORIGINATION_ADDRESS, pline->callingPartyNumber) == GC_SUCCESS) {
			sprintf(str, "gc_GetCallInfo(crn=0x%lx) Success - calling party = %s", pline->call[callindex].crn, pline->callingPartyNumber);
		} else {
			sprintf(str, "gc_GetCallInfo(crn=0x%lx) Failure - calling party not available", pline->call[callindex].crn);
		}
		printandlog(pline->index, GC_APICALL, NULL, str, 0);

   /* next get the original called party number here */
    if (gc_GetCallInfo(pline->call[callindex].crn, REDIRECTING_NUMBER,  pline->originalCalledPartyNumber) == GC_SUCCESS) {
      sprintf(str, "gc_GetCallInfo(crn=0x%lx) Success - originalCalledPartyNumber = %s", pline->call[callindex].crn, pline->originalCalledPartyNumber);
    } else {
      sprintf(str, "gc_GetCallInfo(crn=0x%lx) Failure - originalCalledPartyNumber not available", pline->call[callindex].crn);
    }
    printandlog(pline->index, GC_APICALL, NULL, str, 0);
		if(strlen(pline->callingPartyNumber)>=8)
		{
			int partnerIndex=port[pline->index].nPartnerIndex;
			if(port[index].techtype==SS7) {
				set_calling_num(pline,pline->callingPartyNumber);
			} else if(port[index].techtype==ISDN) { 
				strcpy(port[partnerIndex].makecallblk.gclib->origination.address, pline->callingPartyNumber);
			}
		}

// SKS
	 port[index].callType=getCallType(metaeventp,pline);
    if(port[index].callType==CALL_TYPE_VIDEO) {
        printandlog(pline->index, MISC, NULL, "TDM Video Call",0);
    } else {
        printandlog(pline->index, MISC, NULL, "TDM Audio Call",0);
    }

	if(port[index].callType==CALL_TYPE_VIDEO) {
		connectM3gAndDti(pline->index);
		// SKS place an outgoing call to B-party
	 	makeCall(port[index].nPartnerIndex);	
	} else if(port[index].callType==CALL_TYPE_AUDIO) {
		sendAcceptCall(index);
		routeDtiAndDx(index);
	}
}

/****************************************************************
*   NAME: 	void sendAcceptCall(int index)
*   DESCRIPTION: this function will send ACM to called party
*		INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void sendAcceptCall(int index)
{
 	struct channel    *pline;
  int callindex=0;
  char          str[MAX_STRING_SIZE];

	if(port[index].bAcmSent==YES) {
			return;
	} else if(port[index].bAcmSent==NO) {
		port[index].bAcmSent=YES;
	}


// SKS assume glare condition never happens subjected
//     incoming and outgoing cic are in two different group
  pline = &port[index];

  /* Note: if # of rings is == 0 then the outbound side */
  /* may not get the alerting event, depending upon the protocol */
  /* if the number of rings is 1, there is an occasional problem in */
  /* in the outbound side's ability to recognize the tone */
  if (gc_AcceptCall(pline->call[callindex].crn, 3, EV_ASYNC) != GC_SUCCESS) {
    sprintf(str, "gc_AcceptCall(crn=0x%lx, # of rings=3, mode=EV_ASYNC) Failed", pline->call[callindex].crn);
    printandlog(pline->index, GC_APIERR, NULL, str, 0);
    exitdemo(1);
  }
  sprintf(str, "gc_AcceptCall(crn=0x%lx, mode=EV_ASYNC) Success", pline->call[callindex].crn);
  printandlog(pline->index, GC_APICALL, NULL, str, 0);

}

/****************************************************************
*   NAME: 			void sendAnswerCall(int index)
*   DESCRIPTION: this function will send ANM to called party
*		INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void sendAnswerCall(int index)
{
 	struct channel    *pline;
  int callindex=0;
  char          str[MAX_STRING_SIZE];

	if(port[index].bAnmSent==YES) {
			return;
	}
	port[index].bAnmSent=YES;
// SKS assume glare condition never happens subjected
//     incoming and outgoing cic are in two different group
  pline = &port[index];

				/* Note: if both answercall and acceptcall use 0 rings  */
				/* then the outbound side may not get an alerting event */
				/* this behavior is technology/protocol dependent		  */
	if (gc_AnswerCall(pline->call[callindex].crn, 0, EV_ASYNC) != GC_SUCCESS) {
			sprintf(str, "gc_AnswerCall(crn=0x%lx, # of rings=0, mode=EV_ASYNC) Failed", pline->call[callindex].crn);
			printandlog(index, GC_APIERR, NULL, str, 0);
			exitdemo(1);
	}
	sprintf(str, "gc_AnswerCall(crn=0x%lx, mode=EV_ASYNC) Success", pline->call[callindex].crn);
	printandlog(index, GC_APICALL, NULL, str, 0);
}
/****************************************************************
*			NAME: process_call_related_outbound_event(struct channel *pline, METAEVENT *metaeventp)
* DESCRIPTION: Function to process outbound GlobalCall event
*		 INPUTS: pline - pointer to entry in port table
*					metaeventp - pointer to the metaevent structure
*		RETURNS: NA
*	  CAUTIONS: Tightly coupled with rocess_event() for pre-processing
*					Assumes event logging already done
*					Disconnected event already handled
****************************************************************/
static void process_call_related_outbound_event(struct channel *pline, METAEVENT *metaeventp)
{
	int		evttype, index;
	int		unexpected_event = 0;					/* till proven otherwise */
	char		str[MAX_STRING_SIZE];
	int		callindex;

	evttype = metaeventp->evttype;
	index = pline->index;

  printandlog(index, MISC, NULL, "process_call_related_outbound_event()",0);
	/* Retrieve CRN */
	callindex = existCRN(metaeventp->crn, pline);
	if (callindex == OUT_OF_RANGE) {
		/* CRN not found, so log error and return from function. */
		sprintf(str, "CRN not found");
		printandlog(pline->index, MISC_WITH_NL, NULL, str, 0);
		return;
	}
	sprintf(str,"Rx Event Type 0x%x",evttype);	
	printandlog(index, MISC_WITH_NL, NULL, str, 0);

	int partnerIndex;
	partnerIndex=port[index].nPartnerIndex;
	switch (pline->call[callindex].call_state)
	{
		/************************************/
		/* first come the call setup states */
		/************************************/
		case GCST_NULL:
		{
			switch (evttype)
			{
				case GCEV_ALERTING:						/* just change the state to alerting - no other action is necessary */
					pline->call[callindex].call_state = GCST_ALERTING;
					if(port[index].callType==CALL_TYPE_VIDEO) {
						connectM3gAndDti(index);
						#ifndef MRBT_AFTER_ANM
							// send ACM to A party
							sendAcceptCall(partnerIndex);
						#else
							// send ANM to A party if MRBT_AFTER_ANM is true
  						sendAnswerCall(partnerIndex);
						#endif
					}
					break;

				case GCEV_PROCEEDING:					/* just change the state to proceeding - no other action is necessary */
					pline->call[callindex].call_state = GCST_PROCEEDING;
						if(port[index].callType==CALL_TYPE_VIDEO) {
							connectM3gAndDti(pline->index);
						}
					break;

				case GCEV_CONNECTED:
					if(port[index].callType==CALL_TYPE_VIDEO) {
						connectM3gAndDti(pline->index);
						#ifndef MRBT_AFTER_ANM
							// B party answered the call let's answer A party call as well
  						sendAnswerCall(partnerIndex);
						#else 
							process_answered_event(partnerIndex);
						#endif
					}
					process_connected_event(pline, metaeventp);
					break;
				default:
					unexpected_event = 1;
					break;
			}	// end of evttype switch .... case in NULL state
			break;
		}

		case GCST_ALERTING:
		{
			if (evttype == GCEV_CONNECTED) {
				if(port[index].callType==CALL_TYPE_VIDEO) {
					#ifndef MRBT_AFTER_ANM
							// B party answered the call let's answer A party call as well
  						sendAnswerCall(partnerIndex);
					#else 
							process_answered_event(partnerIndex);
					#endif
				}
				process_connected_event(pline, metaeventp);
			} else {
				unexpected_event = 1;
			}
			break;
		}

    case GCST_PROCEEDING:
    {
				switch (evttype)
				{
					case GCEV_ALERTING:						/* just change the state to alerting - no other action is necessary */
					{
						if(port[index].callType==CALL_TYPE_VIDEO) 
						{
       				#ifndef MRBT_AFTER_ANM
            		// send ACM to A party
            		sendAcceptCall(partnerIndex);
        			#else
            		// send ANM to A party if MRBT_AFTER_ANM is true
            		sendAnswerCall(partnerIndex);
        			#endif
						}
						pline->call[callindex].call_state = GCST_ALERTING;
						break;
					}
					case GCEV_CONNECTED:
					{
						if(port[index].callType==CALL_TYPE_VIDEO) {
       				#ifndef MRBT_AFTER_ANM
									// B party answered the call let's answer A party call as well
  								sendAnswerCall(partnerIndex);
							#else 
  								sendAnswerCall(partnerIndex);
							#endif
						}
						process_connected_event(pline, metaeventp);
						break;
					}
				default:
					unexpected_event = 1;
					break;
			}
			break;
		}

		/*************************************/
		/* now come the call teardown states */
		/*************************************/
		case GCST_CONNECTED:
		{
			/*************************************************/
			/* Depending upon your application, you may or	 */
			/* may not have application specific event		 */
			/* handling here for when in the connected state */
			/* You could also have your app specific code	 */
			/* in process_event()									 */
			/*************************************************/
			if (evttype != GCEV_DROPCALL) {			/* recall the outbound side issues a gc_DropCall() */
				unexpected_event = 1;					/* for the purposes of this demo */
				break;
			}
			/* FALL THROUGH ON PURPOSE since this is one of the 2 ways of reaching Idle */
			/* the other being a disconnected event */
				break;
		}

		case GCST_DISCONNECTED:
		{
			if (evttype == GCEV_DROPCALL) {
				releaseCall(index);
				break;
			} else {
				unexpected_event = 1;
			}
			break;
		}

		case GCST_IDLE:
		{
			if (evttype == GCEV_RELEASECALL) {
				pline->call[callindex].call_state = GCST_NULL;
				pline->call[callindex].crn = 0;
				pline->makecall_active = NO;					/* makecall is no longer active */
			} else {
				unexpected_event = 1;
			}
			break;
		}

		default:
		{
			/* Should never get here */
			printandlog(index, MISC, NULL, "Invalid state in process_call_related_outbound_event()", 0); 
			break;
		}
	}

	if (unexpected_event && (evttype != GCEV_UNBLOCKED)) {
		printandlog(index, GC_RESULT_INFO, metaeventp, "Event was unexpected in the current state", 0);
		printandlog(index, EVENT, NULL, GCEV_MSG(evttype), 0);
		printandlog(index, STATE, NULL, " ", callindex);
	}
}

/****************************************************************
*			NAME: process_connected_event(struct channel *pline, METAEVENT *metaeventp)
* DESCRIPTION: Function to process a connected event
*		 INPUTS: pline - pointer to entry in port table
*		RETURNS: NA
*	  CAUTIONS: Assumes event logging already done
*					Recall that this is only done for the outbound side
*					as the GCEV_CONNECTED event is an outbound side event only
****************************************************************/
static void process_connected_event(struct channel *pline, METAEVENT *metaeventp)
{
	char		str[MAX_STRING_SIZE];
	char		value;										/* for connect type */
	int		rc;
	int		callindex;

  printandlog(pline->index, MISC, NULL, "process_connected_event()",0);
	/* Retrieve CRN */
	callindex = existCRN(metaeventp->crn, pline);
	if (callindex == OUT_OF_RANGE) {
		/* CRN not found, so log error and return from function. */
		sprintf(str, "CRN not found");
		printandlog(pline->index, MISC_WITH_NL, NULL, str, 0);
		return;
	}

		/* print connect type */
		rc = gc_GetCallInfo(pline->call[callindex].crn, CONNECT_TYPE, &value);
		if (rc != GC_SUCCESS) {
			if (rc == EGC_UNSUPPORTED) {
				strcpy(str, "call connected - gc_GetCallInfo(CONNECT_TYPE) not supported");
			} else {
				strcpy(str, "call connected - gc_GetCallInfo(CONNECT_TYPE) error\n");
				sprintf(str, "gc_GetCallInfo(crn=0x%lx, CONNECT_TYPE, &value) failed", pline->call[callindex].crn);
				printandlog(pline->index, GC_APIERR, NULL, str, 0);
			}
		} else {
			switch (value)
			{
				case GCCT_CAD :
					strcpy(str, "call connected - cadence break");
					break;

				case GCCT_LPC :
					strcpy(str, "call connected - loop current drop");
					break;

				case GCCT_PVD :
					strcpy(str, "call connected - voice detection");
					break;

				case GCCT_PAMD :
					strcpy(str, "call connected - answering machine");
					break;

				case GCCT_FAX :
					strcpy(str, "call connected - FAX machine");
					break;

				case GCCT_NA :
					strcpy(str, "call connected - call progress not applicable");
					break;
	
				default :
					strcpy(str, "call connected - unknown connect type");
					break;
			}
		}
		printandlog(pline->index, MISC, NULL, str, 0); 

	pline->call[callindex].call_state = GCST_CONNECTED;
	pline->numb_calls++;							/* Increment the total calls on this device */
	out_calls++;									/* Increment the total outbound calls on all devices */
	outbound_application(pline);				/* call your application specific code here */
}

/****************************************************************
*			NAME: enable_alarm_notification(struct channel *pline)
* DESCRIPTION: Enables all alarms notification for pline
*					Also fills in pline->mediah
*		  INPUT: pline - pointer to channel data structure
*		RETURNS: None - exits if error
*	  CAUTIONS: Does not sanity checking as to whether or not the technology supports
*				   alarms - assumes caller has done that already
****************************************************************/
static void	enable_alarm_notification(struct channel *pline)
{
	char		str[MAX_STRING_SIZE];
	int		alarm_ldev;						/* linedevice that alarms come on */

	alarm_ldev = pline->ldev;				/* until proven otherwise */

	if (gc_SetAlarmNotifyAll(alarm_ldev, ALARM_SOURCE_ID_NETWORK_ID, ALARM_NOTIFY) != GC_SUCCESS) {
		sprintf(str, "gc_SetAlarmNotifyAll(linedev=%ld, ALARM_SOURCE_ID_NETWORK_ID, \
ALARM_NOTIFY) Failed", pline->ldev);
		printandlog(pline->index, GC_APIERR, NULL, str, 0);
		printandlog(pline->index, MISC, NULL, "You may ignore this error - \
most likely the underlying technology does not support alarms yet", 0);
	} else {
		sprintf(str, "gc_SetAlarmNotifyAll(linedev=%ld, ALARM_SOURCE_ID_NETWORK_ID, ALARM_NOTIFY) PASSED", pline->ldev);
		printandlog(pline->index, MISC, NULL, str, 0);
	}
}


/****************************************************************
*			NAME: open_all_devices(void)
* DESCRIPTION: Routine calls the specific technology function, which
*					opens all the line devices. 
*		  INPUT: None
*		RETURNS: None - exits if error
*	  CAUTIONS: none
****************************************************************/
static void open_all_devices(void)
{
	int	index;

	for (index = 0; index < num_devices; index++) { 
		open_device(index);
      //sr_setparm(port[index].ldev, SR_USERCONTEXT, (void *)&port[index].index);
	}
}
/****************************************************************
*			NAME: open_devices(int index)
* DESCRIPTION: Routine calls the specific technology function, which
*					opens all specific line device. 
*		  INPUT: index - index into port data structure
*		RETURNS: None - exits if error
*	  CAUTIONS: Since multiple paths here, will init run-time fields of port data structure
****************************************************************/
static void open_device(int index)
{
	int		callindex;

	/* reset run-time variables that might now be wrong */
	port[index].ldev = 0;

	/* reset the call related information */
	for (callindex = 0; callindex < MAX_CALLS; callindex++) {
		port[index].call[callindex].crn = 0;
		port[index].call[callindex].call_state = GCST_NULL;
		port[index].call[callindex].dropcall_active = NO;	
	}
	port[index].callType=CALL_TYPE_VIDEO;
	port[index].index = index;
	port[index].blocked = YES;
	port[index].waitcall_active = NO;	
	port[index].makecall_active = NO;	
	port[index].resetlinedev_active = NO;	
	port[index].resetlinedev_required_after_unblocked = NO;	

	if ((strcmp(port[index].protname, "isdn") == 0) ||
		(strcmp(port[index].protname, "ISDN") == 0)) {
		open_isdn_channel(index);
	} else if ((strstr(port[index].protname, "ss7") != NULL) ||
		(strstr(port[index].protname, "SS7") != NULL)) {
		open_ss7_channel(index);
	} 
  // SKS get the network handle here
  getNetworkHandle(index);
	if(port[index].direction==DIR_IN) {
  	// SKS DX
  		openMediaDevice(index);
	}
}

/***************************************************************************
 *			NAME: void set_calling_num(struct channel *pline, char *phone_num)
 *DESCRIPTION: Sets the calling number for the outbound direction
 *		  INPUT: pline - pointer to entry in port table
 *					phone_num - pointer to the phone number
 *		RETURNS: None.
  ***************************************************************************/
static void set_calling_num(struct channel *pline, char *phone_num)
{
	char		str[MAX_STRING_SIZE];

	if (pline->direction == DIR_OUT) {
		if (gc_SetCallingNum(pline->ldev, phone_num) < 0) {
			sprintf(str, "gc_SetCallingNum(linedev=%ld, phone_num = %s) Failed", pline->ldev, phone_num);
			printandlog(pline->index, GC_APIERR, NULL, str, 0);
			exitdemo(1);
		}
		sprintf(str, "gc_SetCallingNum(linedev=%ld, phone_num = %s) Success\n", pline->ldev, phone_num);
		printandlog(pline->index, MISC, NULL, str, 0);
	}
}    

/****************************************************************
*   NAME: void getNetworkHandle(int index)
*   DESCRIPTION: get the network handle from line device
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/

void getNetworkHandle(int index)
{
  char str[MAX_STRING_SIZE];
  if ( gc_GetResourceH(port[index].ldev, &port[index].networkH,GC_NETWORKDEVICE) != GC_SUCCESS ) {
     sprintf(str,"gc_GetResourceH(%s) failed",port[index].netdevice);
    // Handle Error
  } else {
     sprintf(str,"gc_GetResourceH(%s) successful",port[index].netdevice);
  }
  printandlog(index,GC_APICALL,NULL,str,0);
  sr_setparm(port[index].networkH, SR_USERCONTEXT, (void *)&port[index].index);
}
/****************************************************************
*   NAME: 	void openMediaDevice(int index)
*   DESCRIPTION:	open dx device and get the handle to play audio
									prompt
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void openMediaDevice(int index)
{
  char str[MAX_STRING_SIZE];
  port[index].dxH=dx_open(port[index].dxDeviceName,0);
  if(port[index].dxH>0) {
      sprintf(str,"dx_open(%s) successfull",port[index].dxDeviceName);
      sr_setparm(port[index].dxH, SR_USERCONTEXT, (void *)&port[index].index);
  }
  else {
      sprintf(str,"dx_open(%s) failed",port[index].dxDeviceName);
  }
  printandlog(index,DX,NULL,str,0);
}

/****************************************************************
*   NAME:		void routeDtiAndDx(int index) 
*   DESCRIPTION: route dti and dx device, required in audio call
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void routeDtiAndDx(int index)
{
  char str[MAX_STRING_SIZE];
   long         ts = 0;
   SC_TSINFO    scts;

#if 0
	 // SKS uncomment if full duplex connection is required 
   scts.sc_numts = 1;
   scts.sc_tsarrayp = &ts;
   if ( gc_GetXmitSlot(port[index].ldev, &scts) < 0 ) {
      sprintf(str,"gc_GetXmitSlot(%s) failed",port[index].netdevice);
      printandlog(index,DX,NULL,str,0);
      exitdemo(1);
   }
   if ( dx_listen(port[index].dxH, &scts) < 0 ) {
      sprintf(str,"dx_listen(%s) failed",port[index].dxDeviceName);
      printandlog(index,DX,NULL,str,0);
      exitdemo(1);
   }
   ts = 0;
#endif
   scts.sc_numts = 1;
   scts.sc_tsarrayp = &ts;
   if ( dx_getxmitslot(port[index].dxH, &scts) < 0 ) {
      sprintf(str,"dx_getxmitslot(%s) failed",port[index].dxDeviceName);
      printandlog(index,DX,NULL,str,0);
      exitdemo(1);
   }
   if ( gc_Listen(port[index].ldev, &scts, EV_SYNC) < 0 ) {
      sprintf(str,"gc_Listen(%s) failed",port[index].netdevice);
      printandlog(index,DX,NULL,str,0);
      exitdemo(1);
   }
	 else {
      sprintf(str,"gc_Listen(%s) success",port[index].netdevice);
      printandlog(index,DX,NULL,str,0);
	 }
}
/****************************************************************
*   NAME: 	void unRouteDtiAndDx(int index)
*   DESCRIPTION: unroute dti and dx device if route earlier for audio call
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void unRouteDtiAndDx(int index)
{
  char str[MAX_STRING_SIZE];
#if 0
	 // SKS uncomment if full duplex connection need to broke 
   if ( dx_unlisten(port[index].dxH) < 0 ) {
      sprintf(str,"dx_unlisten(%s) failed",port[index].dxDeviceName);
      printandlog(index,DX,NULL,str,0);
      exitdemo(1);
   }
#endif
   if ( gc_UnListen(port[index].ldev,EV_SYNC) < 0 ) {
      sprintf(str,"gc_UnListen(%s) failed",port[index].netdevice);
      printandlog(index,DX,NULL,str,0);
      exitdemo(1);
   }
	 else {
      sprintf(str,"gc_UnListen(%s) success",port[index].netdevice);
      printandlog(index,DX,NULL,str,0);
	 }
}

/****************************************************************
*			NAME: makeCall(index)
* DESCRIPTION: Routine calls the gc_Makecall() function if not blocked and there is not a call active
*		  INPUT: index into port[] for this channel
*		RETURNS: exits with exitdemo() if not successful
*	  CAUTIONS: none
****************************************************************/
static void makeCall(int index)
{
	char					str[MAX_STRING_SIZE];
	struct channel		*pline;
	GC_MAKECALL_BLK	*makecallblkp;				/* Variable for MAKECALL block */		
	int		callindex;
	char					*numberstr;

	pline = &port[index];
	if ( (pline->blocked == YES) || (pline->resetlinedev_active == YES)
	|| (interrupted == YES) ) {
		return;										/* cannot make a call until unblocked and completion of reset line device */
	}

	/* Find empty place in array to store CRN. */
	for (callindex = 0; callindex < MAX_CALLS; callindex++) {
		if (pline->call[callindex].crn == 0) {
			break;
		}
	}

	if (pline->makecall_active == YES) {
		printandlog(index, MISC, NULL, "Trying to make a call while makecall is already active\n", 0);
		return;
	}
	else {
			sprintf(str,"Trying to make a call on index=%d | channel=%d\n",index,index+1);
			printandlog(index, MISC, NULL, str, 0);
	}

	GC_IE_BLK   ie;
	IE_BLK      ieblk;

	ie.gclib = NULL;    // GlobalCall specific portion
	ie.cclib = &ieblk;  // CCLIB specific portion

	if(port[index].techtype==ISDN) {

		ieblk.length  = 10;	// increase length if using both LLC and BC

		ieblk.data[0] = (char)0x04;	// octet 1 IE type i.e. 0x04 = BC / 0x7c = LLC
		ieblk.data[1] = (char)0x03;	// octet 2 length of the IE
		ieblk.data[2] = (char)0x88;   	// octet 3 0x80 = speech / 0x88=unrestricted data
		ieblk.data[3] = (char)0x90;   	// octet 4 0x90 = 64 kbps packet mode
		ieblk.data[4] = (char)0xA6;   	// octet 5 0xA2 = uLaw / 0xA6 = video

	// if not able to place call with LLC setting, then try using BC

		ieblk.data[5] = (char)0x7c;	// octet 1 IE type i.e. 0x04 = BC / 0x7c = LLC
		ieblk.data[6] = (char)0x03;	// octet 2 length of the IE
		ieblk.data[7] = (char)0x88;   	// octet 3 0x80 = speech  / 0x88=unrestricted data
		ieblk.data[8] = (char)0x90;   	// octet 4 0x90 = 64 kbps packet mode
		ieblk.data[9] = (char)0xA6;   	// octet 5 0xA2 = uLaw / 0xA6 = video

	} else if(port[index].techtype==SS7) {

		ieblk.length  = 12;	// increase length if using both LLC and BC

      ieblk.data[0] = (char)0x03;   // octet 1 IE type i.e. 0x04 = BC / 0x7c = LLC
      ieblk.data[1] = (char)0x05;   // octet 2 length of the IE
      ieblk.data[2] = (char)0x7C;      // octet 4 0x90 = 64 kbps packet mode
      ieblk.data[3] = (char)0x03;      // octet 5 0xA2 = uLaw / 0xA6 = video
      ieblk.data[4] = (char)0x88;      // octet 3 0x80 = speech / 0x88=unrestricted data
      ieblk.data[5] = (char)0x90;      // octet 4 0x90 = 64 kbps packet mode
      ieblk.data[6] = (char)0xA6;      // octet 5 0xA2 = uLaw / 0xA6 = video

   // if not able to place call with LLC setting, then try using BC

      ieblk.data[7] = (char)0x1D;   // octet 1 IE type i.e. 0x04 = BC / 0x7c = LLC
      ieblk.data[8] = (char)0x03;   // octet 2 length of the IE
      ieblk.data[9] = (char)0x88;      // octet 3 0x80 = speech  / 0x88=unrestricted data
      ieblk.data[10] = (char)0x90;      // octet 4 0x90 = 64 kbps packet mode
      ieblk.data[11] = (char)0xA6;      // octet 5 0xA2 = uLaw / 0xA6 = video

	}

	if (gc_SetInfoElem(pline->ldev, &ie) != GC_SUCCESS) {
		printandlog(index, GC_APICALL, NULL, "can't make video call gc_SetInfoElem() for the channel failed", 0);
		return;
	} else {
		printandlog(index, GC_APICALL, NULL, "gc_SetInfoElem() for the channel success, Will make an outgoing Video call", 0);
	}
	port[index].callType=CALL_TYPE_VIDEO;

	//		override with original called party number
	if(strlen(pline->originalCalledPartyNumber)>=8) {
				numberstr = pline->originalCalledPartyNumber;
	} else {
				numberstr = pline->destination_num;
	}
	/* if the makecall block has been initialized, use it */
	/* else use NULL for the makecall block pointer */
	makecallblkp = (pline->makecallblk.gclib != NULL) ? &pline->makecallblk : NULL;


	if (gc_MakeCall(pline->ldev, &pline->call[callindex].crn, numberstr, makecallblkp, pline->makecall_timeout, EV_ASYNC) == GC_SUCCESS) {
		pline->makecall_active = YES;					/* indicate there is a call active */
		sprintf(str, "gc_MakeCall(linedev=%ld, numberstr=%s, mode=EV_ASYNC) Success", pline->ldev, pline->destination_num);
		printandlog(index, GC_APICALL, NULL, str, 0);
	} else {
		sprintf(str, "gc_MakeCall(linedev=%ld, numberstr=%s, mode=EV_ASYNC) Failed", pline->ldev, pline->destination_num);
		printandlog(index, GC_APIERR, NULL, str, 0);
		if (pline->techtype == ISDN) {
			sprintf(str, "Please verify setup in open_isdn_channel() is correct");
			printandlog(index, MISC, NULL, str, 0);
		}
	}
} /* End of Function */

/****************************************************************
*   NAME: void process_non_gc_event(METAEVENT *metaevent)
*   DESCRIPTION: to handle media related event like
									DX, M3G Device Management etc
*   INPUT: METAEVENT
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_non_gc_event(METAEVENT *metaevent)
{
	char str[MAX_STRING_SIZE];
	int index;
	int devH = (int)metaevent->evtdev;
  sr_getparm(devH, SR_USERCONTEXT, &index);
	int prev_state;
	if(index!=BOARD_DEVICE) {
		prev_state=port[index].app_state;
	} 
	sprintf(str,"SKS Rx Evt evttype=0x%lx|%ld on %d",metaevent->evttype,metaevent->evttype,index);
//	printandlog(index,MISC,NULL,str,0);
	switch(metaevent->evttype)
	{
		case M3GEV_REMOTE_VENDORID_RCVD:
			{
				M3G_VENDORID_INFO *vendorID=(M3G_VENDORID_INFO*)metaevent->evtdatap;
				sprintf(str,"M3GEV_REMOTE_VENDORID_RCVD evt on %d Vendor Prod=%s|Ver=%s",devH,vendorID->productNumber.octet,vendorID->versionNumber.octet);
				printandlog(index,M3GEV,NULL,str,0);
				break;
			}
		case M3GEV_H245_UII_RCVD:
			{
				M3G_H245_UII *h245Uii=(M3G_H245_UII*)metaevent->evtdatap;
				sprintf(str,"Rx M3GEV_H245_UII_RCVD evt on handle=%d NoOfDigit=%d Digit=%s",devH,h245Uii->numDigits,h245Uii->digitBuffer);
				printandlog(index,M3GEV,NULL,str,0);
				break;
			}
		case M3GEV_H245_MISC_CMD_RCVD:
			{
				sprintf(str,"M3GEV_H245_MISC_CMD_RCVD Event Received on handle=%d",devH);
				printandlog(index,M3GEV,NULL,str,0);
				break;
			}
		case M3GEV_OPEN_CMPLT:
			{
				sprintf(str,"M3GEV_OPEN_CMPLT Event Received on handle=%d index=%d",devH,index);
				printandlog(index,M3GEV,NULL,str,0);
#if 0
				if(metaevent->evtdev==m3gBoardH)  {
					resetM3gDevice(BOARD_DEVICE,m3gBoardH);
				} else {
					getLocalCapability(index,devH);
				}
#endif
            getLocalCapability(index,devH);
				break;
			}
		case M3GEV_OPEN_FAIL:
			sprintf(str,"M3GEV_OPEN_FAIL Event Received on Handle=%d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_RESET_CMPLT:
			{
				sprintf(str,"M3GEV_RESET_CMPLT Event Received on device Handle(%d)",devH);
				printandlog(index,M3GEV,NULL,str,0);
				if(metaevent->evtdev==m3gBoardH) {
					startM3gTrace(BOARD_DEVICE,m3gBoardH);
					m3gSetParameter(BOARD_DEVICE,m3gBoardH);
					m3gEnableDisableEvents(BOARD_DEVICE,m3gBoardH);
				}	
				break;
			}
		case M3GEV_RESET_FAIL:
			sprintf(str,"M3GEV_RESET_FAIL Event Received on Handle=%d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_SET_PARM_CMPLT:
			{
         	m3gBoardParmCtr++;
				sprintf(str,"M3GEV_SET_PARM_CMPLT Event Received on Handle=%d",devH);
				printandlog(index,M3GEV,NULL,str,0);
         	if(m3gBoardParmCtr==4 && (metaevent->evtdev==m3gBoardH) ) {
							openM3gMmChannel();
         	}
				break;
			}
		case M3GEV_SET_PARM_FAIL:
			sprintf(str,"M3GEV_SET_PARM_FAIL Event Received on Handle=%d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_ENABLE_EVENTS_CMPLT:
			sprintf(str,"M3GEV_ENABLE_EVENTS_CMPLT Event Received on Handle=%d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_ENABLE_EVENTS_FAIL:
			sprintf(str,"M3GEV_ENABLE_EVENTS_FAIL Event Received on Handle=%d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_DISABLE_EVENTS_CMPLT:
			sprintf(str,"M3GEV_DISABLE_EVENTS_CMPLT Event Received on Handle=%d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_DISABLE_EVENTS_FAIL:
			sprintf(str,"M3GEV_DISABLE_EVENTS_FAIL Event Received on Handle=%d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
			
		case M3GEV_GET_LOCAL_CAPS_CMPLT:
			{
				port[index].localCapCtr++;
				sprintf(str,"M3GEV_GET_LOCAL_CAPS_CMPLT_%d Evt Rcvd on Handle=%d", port[index].localCapCtr,devH);
				printandlog(index,M3GEV,NULL,str,0);
				storeLocalCapability(index,devH,(M3G_CAPS_LIST*)metaevent->evtdatap);
				if(port[index].localCapCtr==3) {
			 		setLocalCapability(index);
				}
				break;
			}
		case M3GEV_GET_LOCAL_CAPS_FAIL:
			sprintf(str,"M3GEV_GET_LOCAL_CAPS_FAIL Event Received on Handle=%d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_SET_TCS_CMPLT:
			sprintf(str,"M3GEV_SET_TCS_CMPLT Event Received on Handle=%d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			getTxRxPortInfo(index,port[index].m3gAudioH);
			getTxRxPortInfo(index,port[index].m3gVideoH);
			getTxRxPortInfo(index,port[index].mmH);
			break;
		case M3GEV_SET_TCS_FAIL:
			sprintf(str,"M3GEV_SET_TCS_FAIL Event Received on Handle=%d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case DMEV_CONNECT:
		{
			port[index].dmevConnectCtr++;
			sprintf(str,"DMEV_CONNECT Event Received on %d",devH);
			printandlog(index,DMEV,NULL,str,0);
			process_dmev_connect(index);
			break;
		}
		case DMEV_CONNECT_FAIL:
			sprintf(str,"DMEV_CONNECT_FAIL Event Received on %d",devH);
			printandlog(index,DMEV,NULL,str,0);
			break;
		case DMEV_DISCONNECT:
			port[index].dmevDisconnectCtr++;
			sprintf(str,"DMEV_DISCONNECT Event Received on %d",devH);
			printandlog(index,DMEV,NULL,str,0);
			process_dmev_disconnect(index);
			break;
		case DMEV_DISCONNECT_FAIL:
			sprintf(str,"DMEV_DISCONNECT_FAIL Event Received on %d",devH);
			printandlog(index,DMEV,NULL,str,0);
			break;
		case DMEV_PORT_CONNECT:
		{
			port[index].dmevPortConnectCtr++;
			sprintf(str,"DMEV_PORT_CONNECT Event Received on %d totalCount=%d",devH,port[index].dmevPortConnectCtr);
			printandlog(index,DMEV,NULL,str,0);
      	process_dmev_port_connect(index);
			break;
		}
		case DMEV_PORT_CONNECT_FAIL:
			sprintf(str,"DMEV_PORT_CONNECT_FAIL Event Received on %d",devH);
			printandlog(index,DMEV,NULL,str,0);
			break;
		case DMEV_PORT_DISCONNECT:
			port[index].dmevPortDisconnectCtr++;
			sprintf(str,"DMEV_PORT_DISCONNECT Event Received on %d totalCount=%d | connectCount=%d",
										devH,port[index].dmevPortDisconnectCtr,port[index].dmevPortConnectCtr);
			printandlog(index,DMEV,NULL,str,0);
				process_dmev_port_disconnect(index,devH);
			break;
		case DMEV_PORT_DISCONNECT_FAIL:
			sprintf(str,"DMEV_PORT_DISCONNECT_FAIL Event Received on %d",devH);
			printandlog(index,DMEV,NULL,str,0);
			break;
		case DMEV_GET_TX_PORT_INFO:
			{
				port[index].txPortCtr++;
				sprintf(str,"DMEV_GET_TX_PORT_INFO Event Received on %d",devH);
				printandlog(index,DMEV,NULL,str,0);
				storeTxPortInfo(index,metaevent);
				break;
			}
		case DMEV_GET_RX_PORT_INFO:
			{
				port[index].rxPortCtr++;
				sprintf(str,"DMEV_GET_RX_PORT_INFO Event Received on %d",devH);
				printandlog(index,DMEV,NULL,str,0);
        storeRxPortInfo(index,metaevent);
				break;
			}
		case MMEV_OPEN:
			sprintf(str,"MMEV_OPEN Event Received on %d",devH);
			printandlog(index,MMEV,NULL,str,0);
			break;
		case MMEV_OPEN_FAIL:
			sprintf(str,"MMEV_OPEN_FAIL Event Received on %d",devH);
			printandlog(index,MMEV,NULL,str,0);
			break;
		case MMEV_PLAY_ACK:
			{
				sprintf(str,"MMEV_PLAY_ACK Event Received on %d",devH);
				printandlog(index,MMEV,NULL,str,0);
				process_mrbt_play_started(index);
				break;
			}
		case MMEV_PLAY_ACK_FAIL:
			{
				sprintf(str,"MMEV_PLAY_ACK_FAIL Event Received on %d",devH);
				printandlog(index,MMEV,NULL,str,0);
				break;
			}
		case MMEV_PLAY:
			{
				MM_PLAY_RECORD_CMPLT *mmPlayCmplt=(MM_PLAY_RECORD_CMPLT*)metaevent->evtdatap;
				sprintf(str,"MMEV_PLAY Event Received on %d Reason=%d",devH,mmPlayCmplt->details[0].Reason);
				printandlog(index,MMEV,NULL,str,0);
				port[index].bPlayActive=M3G_FALSE;
				if(mmPlayCmplt->details[0].Reason==EMM_TR_EOF) {
						playVideoPrompt(index);				
				} else {
						process_mrbt_play_stopped(index);
				}
				break;
			}
		case MMEV_PLAY_FAIL:
			{
				sprintf(str,"MMEV_PLAY_FAIL Event Received on %d",devH);
				printandlog(index,MMEV,NULL,str,0);
				break;
			}
		case MMEV_STOP_ACK:
			sprintf(str,"MMEV_STOP_ACK Event Received on %d",devH);
			printandlog(index,MMEV,NULL,str,0);
			break;
		case MMEV_STOP_ACK_FAIL:
			sprintf(str,"MMEV_STOP_ACK_FAIL Event Received on %d",devH);
			printandlog(index,MMEV,NULL,str,0);
			break;
		case M3GEV_FRAMING_ESTABLISHED:
				sprintf(str,"M3GEV_FRAMING_ESTABLISHED on %d",devH);
				printandlog(index,M3GEV,NULL,str,0);
				break;
		case M3GEV_MSD_ESTABLISHED:
			{
				port[index].olcEvtMask+=M3GEV_MSD_ESTABLISHED;
				sprintf(str,"M3GEV_MSD_ESTABLISHED on %d (evtReqBefOlc=0x%x|olcMask=0x%x)",devH,eventsReqiredBefOlc,port[index].olcEvtMask);
				printandlog(index,M3GEV,NULL,str,0);
				if(port[index].olcEvtMask==eventsReqiredBefOlc) {
					getMatchedCapabilityAndOpenLC(index);
				}
				break;
			}
		case M3GEV_MSD_FAILED:
			sprintf(str,"M3GEV_MSD_FAILED Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_REMOTE_TCS_RCVD:
			{
				port[index].olcEvtMask+=M3GEV_REMOTE_TCS_RCVD;
				sprintf(str,"M3GEV_REMOTE_TCS_RCVD on %d (evtReqBefOlc=0x%x|olcMask=0x%x)",devH,eventsReqiredBefOlc,port[index].olcEvtMask);
				printandlog(index,M3GEV,NULL,str,0);
				if(port[index].olcEvtMask==eventsReqiredBefOlc) {
					getMatchedCapabilityAndOpenLC(index);
				}
				break;
			}
		case M3GEV_LOCAL_TCS_ACKD:
			{
				port[index].olcEvtMask+=M3GEV_LOCAL_TCS_ACKD;
				sprintf(str,"M3GEV_LOCAL_TCS_ACKD on %d (evtReqBefOlc=0x%x|olcMask=0x%x)",devH,eventsReqiredBefOlc,port[index].olcEvtMask);
				printandlog(index,M3GEV,NULL,str,0);
				if(port[index].olcEvtMask==eventsReqiredBefOlc) {
					getMatchedCapabilityAndOpenLC(index);
				}
				break;
			}
		case M3GEV_START_H245_CMPLT:
		{
			sprintf(str,"M3GEV_START_H245_CMPLT Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			process_h245_started(index);
			break;
		}
		case M3GEV_START_H245_FAIL:
			sprintf(str,"M3GEV_START_H245_FAIL Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_STOP_H245_CMPLT:
			sprintf(str,"M3GEV_STOP_H245_CMPLT Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			process_h245_stopped(index);
			break;
		case M3GEV_OPEN_LC_CMPLT:
			sprintf(str,"M3GEV_OPEN_LC_CMPLT Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			M3G_REMOTE_OLCACK_RESP *remoteOlcAckResp=(M3G_REMOTE_OLCACK_RESP*)metaevent->evtdatap;
			if(remoteOlcAckResp->capabilityType==M3G_E_AUDIO_CAPABILITY) {
				port[index].openAudVidOlcCmplt+=AUDIO_LCN;
			}
			else if(remoteOlcAckResp->capabilityType==M3G_E_VIDEO_CAPABILITY) {
				port[index].openAudVidOlcCmplt+=VIDEO_LCN;
			}
// SKS added to call m3g_ModifyMedia for inbound call here. 
//		 call modifyMedia function for outgoing leg here and also in state APP_STATE_M3G_M3G_CONNECTED as well

   if(port[index].openAudVidOlcCmplt==AUDIO_VIDEO_LCN && port[index].respondAudVidOlcCmplt==AUDIO_VIDEO_LCN && port[index].direction==DIR_IN) {
        //  added to call modify media only if bi-directional logical channel established.
        modifyMedia(index,M3G_E_TX);
        modifyMedia(index,M3G_E_RX);
      }
#if 0

			if(port[index].openAudVidOlcCmplt==AUDIO_VIDEO_LCN && port[index].direction==DIR_IN) {
				modifyMedia(index,M3G_E_TX);
			}
#endif
			if( port[index].openAudVidOlcCmplt==AUDIO_VIDEO_LCN 
						&& (port[index].app_state==APP_STATE_M3G_M3G_CONNECTED || port[index].app_state==APP_STATE_ENDPOINT_CONNECTED) 
									&& port[index].direction==DIR_OUT) {
					modifyMedia(index,M3G_E_TX);
			}
			break;
		case M3GEV_OPEN_LC_FAIL:
			sprintf(str,"M3GEV_OPEN_LC_FAIL Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_REMOTE_OLC_RCVD:
			{
				M3G_REMOTE_OLC_REQ* pOLCReq = (M3G_REMOTE_OLC_REQ*)metaevent->evtdatap;
				sprintf(str,"M3GEV_REMOTE_OLC_RCVD Event Received on %d",devH);
				printandlog(index,M3GEV,NULL,str,0);
				if(pOLCReq->capabilityType==M3G_E_VIDEO_CAPABILITY && port[index].cfgCoderType==M3G_E_MPEG4) {	
					port[index].handPhoneDCILen=pOLCReq->mediaCapability.videoCapability.options.mpeg4.decoderConfigLength;
					memcpy(port[index].handPhoneDCIValue,pOLCReq->mediaCapability.videoCapability.options.mpeg4.decoderConfigInfo,port[index].handPhoneDCILen);
				}
				sendOLCAck(index,(M3G_REMOTE_OLC_REQ*)metaevent->evtdatap);
				break;
			}
		case M3GEV_RESPOND_TO_LC_CMPLT:
			{
				M3G_LOGICAL_CHANNEL_NUMBER *logicalChannel=(M3G_LOGICAL_CHANNEL_NUMBER*)metaevent->evtdatap;
				sprintf(str,"M3GEV_RESPOND_TO_LC_CMPLT Event Received on %d lcn=%d",devH,*logicalChannel);
				printandlog(index,M3GEV,NULL,str,0);
				if(*logicalChannel==port[index].remoteAudioLogicalChannel) {
					port[index].respondAudVidOlcCmplt+=AUDIO_LCN;
				}
				else if(*logicalChannel==port[index].remoteVideoLogicalChannel) {
					port[index].respondAudVidOlcCmplt+=VIDEO_LCN;
				}
// SKS added to call m3g_ModifyMedia for inbound call here. 
//		 call modifyMedia function for outgoing leg here and also in state APP_STATE_M3G_M3G_CONNECTED as well

   if(port[index].respondAudVidOlcCmplt==AUDIO_VIDEO_LCN && port[index].openAudVidOlcCmplt==AUDIO_VIDEO_LCN && port[index].direction==DIR_IN) {
          //  added to call modify media only if bi-directional logical channel established.
          modifyMedia(index,M3G_E_TX);
          modifyMedia(index,M3G_E_RX);
        }

#if 0
				if(port[index].respondAudVidOlcCmplt==AUDIO_VIDEO_LCN && port[index].direction==DIR_IN) {
					modifyMedia(index,M3G_E_RX);
				}
#endif
				if(port[index].respondAudVidOlcCmplt==AUDIO_VIDEO_LCN 
								&& (port[index].app_state==APP_STATE_M3G_M3G_CONNECTED || port[index].app_state==APP_STATE_ENDPOINT_CONNECTED)
										&& port[index].direction==DIR_OUT) {
					modifyMedia(index,M3G_E_RX);
				}
				break;
			}
		case M3GEV_RESPOND_TO_LC_FAIL:
			sprintf(str,"M3GEV_RESPOND_TO_LC_FAIL Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_CLOSE_LC_CMPLT:
			sprintf(str,"M3GEV_CLOSE_LC_CMPLT Event Received on %d",devH);
			printandlog(index,M3G,NULL,str,0);
			break;
		case M3GEV_CLOSE_LC_FAIL:
			sprintf(str,"M3GEV_CLOSE_LC_FAIL Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_REMOTE_CLOSE_LC_RCVD:
			sprintf(str,"M3GEV_REMOTE_CLOSE_LC_RCVD Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_START_MEDIA_CMPLT:
			sprintf(str,"M3GEV_START_MEDIA_CMPLT Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_START_MEDIA_FAIL:
			sprintf(str,"M3GEV_START_MEDIA_FAIL Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_MODIFY_MEDIA_CMPLT:
		{
				M3G_E_DIRECTION *direction = (M3G_E_DIRECTION*)metaevent->evtdatap;
				if(*direction==M3G_E_TXRX || *direction==M3G_E_TX || *direction==M3G_E_IDLE) {
					port[index].modifyMediaCtr+=1;
				}
				sprintf(str,"M3GEV_MODIFY_MEDIA_CMPLT Evt on %d dir=%s|devName=%s",devH,dirStr[*direction],ATDV_NAMEP(metaevent->evtdev));
				printandlog(index,M3GEV,NULL,str,0);
				process_modify_media_complete(index,direction);
				break;
		}
		case M3GEV_MODIFY_MEDIA_FAIL:
			sprintf(str,"M3GEV_MODIFY_MEDIA_FAIL Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_STOP_MEDIA_CMPLT:
			port[index].stopMediaCtr+=1;
			sprintf(str,"M3GEV_STOP_MEDIA_CMPLT Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			process_stop_media_complete(index);
			break;
		case M3GEV_STOP_MEDIA_FAIL:
			sprintf(str,"M3GEV_STOP_MEDIA_FAIL Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_H245_MSD_EVT:
			sprintf(str,"M3GEV_H245_MSD_EVT Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_H245_MES_EVT:
			sprintf(str,"M3GEV_H245_MES_EVT Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_ENDSESSION_SENT:
			sprintf(str,"M3GEV_ENDSESSION_SENT Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			break;
		case M3GEV_ENDSESSION_RCVD:
			{
				sprintf(str,"M3GEV_ENDSESSION_RCVD Event Received on %d",devH);
				printandlog(index,M3GEV,NULL,str,0);
				process_end_session_received(index);
				break;
			}
		case M3GEV_FRAMING_LOST:
			{
				sprintf(str,"M3GEV_FRAMING_LOST Event Received on %d",devH);
				printandlog(index,M3GEV,NULL,str,0);
				process_framing_lost(index);
				break;
			}
		case M3GEV_START_TRACE_CMPLT:
			sprintf(str,"M3GEV_START_TRACE_CMPLT Event Received on %d",devH);
			printandlog(index,M3GEV,NULL,str,0);
			printandlog(BOARD_DEVICE,M3GEV,NULL,str,0);
			break;
		case M3GEV_START_TRACE_FAIL:
			{
				M3G_E_ERROR_TYPE	*pError=(M3G_E_ERROR_TYPE *)metaevent->evtdatap;
				sprintf(str,"M3GEV_START_TRACE_FAIL Event Received on %d with err =%d",devH,(int)*pError);
				printandlog(index,M3GEV,NULL,str,0);
			}
			break;
		case TDX_PLAY: 
		{
				sprintf(str,"TDX_PLAY Event Received on %d",devH);
				long termReason;
				termReason=ATDX_TERMMSK(devH);
				if(termReason & TM_USRSTOP) {
					sprintf(str,"%s Reason=TM_USRSTOP",str);		
				} else {
					sprintf(str,"%s Reason=TM_EOD",str);
					initiate_disconnect(index);
				}
				printandlog(index,DX,NULL,str,0);
				break;
		}
	}	// end of switch .... case statement

	if(index != BOARD_DEVICE) {
		if(prev_state!=port[index].app_state && port[index].callType==CALL_TYPE_VIDEO) {
			sprintf(str,"App state after event processing PrevSt=%s | CurrSt=%s",appStatesStr[prev_state],appStatesStr[port[index].app_state]);
  		printandlog(index,APP_STATE,NULL,str,0);
		}
	}
}

/****************************************************************
*   NAME: void process_end_session_received(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_end_session_received(int index)
{
  printandlog(index, MISC, NULL, "process_end_session_received()",0);
	process_framing_lost(index);
}

/****************************************************************
*   NAME:  void process_framing_lost(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_framing_lost(int index)
{
  printandlog(index, MISC, NULL, "process_framing_lost()",0);
	int prev_state=port[index].app_state;
	port[index].app_state=APP_STATE_FRAMING_LOST;

	port[index].bDisconnectActive=YES;

  switch(prev_state)
  {
        case APP_STATE_ENDPOINT_CONNECTED:
        {
            stopMedia(index);
            break;
        }
        case APP_STATE_M3G_M3G_CONNECTED:
        {
            if(port[index].bMediaStarted==M3G_TRUE || port[index].bMediaIdle==M3G_TRUE) {
              stopMedia(index);
            }
            if (port[index].bMediaStarted==M3G_FALSE && port[index].bMediaIdle==M3G_FALSE) {
              stopH245(index);
            }
            break;
        }
        case APP_STATE_MM_M3G_DISCONNECTED:
            stopMedia(index);
            break;
        case APP_STATE_MEDIA_IDLE:
        case APP_STATE_MEDIA_STARTED:
        case APP_STATE_MRBT_PLAY_STOPPED:
        {
            stopMedia(index);
            break;
        }
        case APP_STATE_MRBT_PLAY_STARTED:
        {
            stopPlay(index);
            break;
        }
        case APP_STATE_MM_M3G_CONNECTED:
        {
            stopH245(index);
            break;
        }
        case APP_STATE_H245_STARTED:
        {
            stopH245(index);
            break;
        }
        case APP_STATE_DTI_M3G_CONNECTED:
        {
            disconnectM3gAndDti(index);
            break;
        }
        case APP_STATE_INITIAL:
        {
          initiate_disconnect(index);
          break;
        }
		}
  	printandlog(index, MISC, NULL, "calling disconnect_partner_3g_call() to disconnect partner leg", 0);
 	 	disconnect_partner_3g_call(port[index].nPartnerIndex);
}

/****************************************************************
*   NAME: void process_modify_media_complete(int index, M3G_E_DIRECTION *direction)
*   DESCRIPTION:
*   INPUT: index and direction 
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_modify_media_complete(int index, M3G_E_DIRECTION *direction)
{
  printandlog(index, MISC, NULL, "process_modify_media_complete()",0);
	if(port[index].modifyMediaCtr<2) {
			return;
	}
	port[index].modifyMediaCtr=0;
	switch(port[index].app_state)
	{
				case APP_STATE_MM_M3G_CONNECTED:
				{
						if(port[index].direction==DIR_IN && (*direction==M3G_E_TXRX || *direction==M3G_E_TX) ) {
								port[index].app_state=APP_STATE_MEDIA_STARTED;
								sendH245UII(index);
								#ifdef MRBT_AFTER_ANM
									if(port[index].bAnmSent==YES) {
										playVideoPrompt(index);
									}
								#else
									if(port[index].bAnmSent==NO) {
										playVideoPrompt(index);
									}
								#endif
						}
						break;
				}
				case APP_STATE_MEDIA_STARTED:
				{
					if(*direction==M3G_E_IDLE) {
						port[index].app_state=APP_STATE_MEDIA_IDLE;
            port[index].bMediaIdle=M3G_TRUE;
            port[index].bMediaStarted=M3G_FALSE;
						routeUnrouteM3gAndMmDevice(index,FULL_DUP,UNROUTE);
					}
					break;
				}
				case APP_STATE_MRBT_PLAY_STOPPED:
				{
					if(*direction==M3G_E_IDLE) {
						port[index].app_state=APP_STATE_MEDIA_IDLE;
						port[index].bMediaStarted=M3G_FALSE;
						port[index].bMediaIdle=M3G_TRUE;
						routeUnrouteM3gAndMmDevice(index,FULL_DUP,UNROUTE);
					}
					break;
				}
				case APP_STATE_M3G_M3G_CONNECTED:
				{
						if(port[index].bMediaIdle==M3G_TRUE) {
								port[index].bMediaIdle=M3G_FALSE;
						}
						port[index].app_state=APP_STATE_ENDPOINT_CONNECTED;
						sendH245VFU(index);
						if(port[index].direction==DIR_IN) {
									sendH245UII(index);
									sendH245UII(index);
						}
						break;
				}
				default:
						break;
	}
}
/****************************************************************
*   NAME: void process_stop_media_complete(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_stop_media_complete(int index)
{
  printandlog(index, MISC, NULL, "process_stop_media_complete()",0);
		if(port[index].stopMediaCtr==2) {
				port[index].app_state=APP_STATE_MEDIA_STOPPED;
				port[index].stopMediaCtr=0;
				stopH245(index);
		}
}
/****************************************************************
*   NAME: void process_mrbt_play_started(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_mrbt_play_started(int index)
{
  printandlog(index, MISC, NULL, "process_mrbt_play_started()",0);
	port[index].app_state=APP_STATE_MRBT_PLAY_STARTED;
}
/****************************************************************
*   NAME: void process_mrbt_play_stopped(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_mrbt_play_stopped(int index)
{
  printandlog(index, MISC, NULL, "process_mrbt_play_stopped()",0);
	switch(port[index].app_state)
	{
		case APP_STATE_MRBT_PLAY_STARTED:
  			port[index].app_state=APP_STATE_MRBT_PLAY_STOPPED;
				if(port[index].bDisconnectActive==NO) {
					modifyMedia(index,M3G_E_IDLE);
				} else if(port[index].bDisconnectActive==YES) {
					stopMedia(index);
				} else {
  					printandlog(index, MISC, NULL, "process_mrbt_play_stopped()::MMEV_PLAY received in invalid state",0);
				}
				break;
    case APP_STATE_FRAMING_LOST:
  			port[index].app_state=APP_STATE_MRBT_PLAY_STOPPED;
        process_3g_call_disconnect(index);
        break;

		default:
			break;
	}
}

/****************************************************************
*   NAME: 	void process_h245_stopped(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_h245_stopped(int index)
{
  	printandlog(index, MISC, NULL, "process_h245_stopped()",0);
		int prev_state=port[index].app_state;
		port[index].app_state=APP_STATE_H245_STOPPED;

		switch(prev_state)
  	{
				case APP_STATE_H245_STARTED:
						disconnectM3gAndDti(index);
						break;
				case APP_STATE_MM_M3G_CONNECTED:
          	routeUnrouteM3gAndMmDevice(index, FULL_DUP, UNROUTE);
						break;
				case APP_STATE_FRAMING_LOST:
				{
						if(port[index].bM3gAndM3gConnected==M3G_TRUE) {
          			routeUnrouteM3gAndM3gDevice(index, HALF_DUP, UNROUTE);
						} else if(port[index].bM3gAndMmConnected==M3G_TRUE) {
          			routeUnrouteM3gAndMmDevice(index, FULL_DUP, UNROUTE);
						} else if(port[index].bM3gAndDtiConnected==M3G_TRUE) {
								disconnectM3gAndDti(index);
						}
						break;
				}
				case APP_STATE_MEDIA_STOPPED:
				{
						if(port[index].direction==DIR_IN && port[index].bM3gAndMmConnected==M3G_TRUE) {
          			routeUnrouteM3gAndMmDevice(index, FULL_DUP, UNROUTE);
						}
						if(port[index].bM3gAndM3gConnected==M3G_TRUE) {
							routeUnrouteM3gAndM3gDevice(index,HALF_DUP,UNROUTE);
						} else if(port[index].bM3gAndMmConnected==M3G_FALSE 
												&& port[index].bM3gAndM3gConnected==M3G_FALSE 
														&& port[index].bM3gAndDtiConnected==M3G_TRUE) {
        			disconnectM3gAndDti(index);
						}
						break;
				}
				case APP_STATE_M3G_M3G_CONNECTED:
						if(port[index].bM3gAndM3gConnected==M3G_TRUE) {
                routeUnrouteM3gAndM3gDevice(index, HALF_DUP, UNROUTE);
            } 
						break;
		}
}

/****************************************************************
*   NAME: void process_h245_started(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_h245_started(int index)
{
  printandlog(index, MISC, NULL, "process_h245_started()",0);
	port[index].app_state=APP_STATE_H245_STARTED;
	if(port[index].direction==DIR_IN) {
		if(port[index].bAnmSent==NO) {
    	if(port[index].txPortCtr<1 && port[index].rxPortCtr>1) {
        	// connect audio and video of mm tx to m3g rx port
          routeUnrouteM3gAndMmDevice(index, FULL_DUP, ROUTE);
    	}
    	else {
        	printandlog(index, MISC, NULL, "Error: Not all the ports information retrieved successfully check logs carefully",0);
    	}
		} else if(port[index].bAnmSent==YES) { 

			if(port[index].txPortCtr>1 && port[index].rxPortCtr>1) {
				#ifndef MRBT_AFTER_ANM
        	// connect audio and video of m3g tx to m3g rx port
          routeUnrouteM3gAndM3gDevice(index, FULL_DUP, ROUTE);
				#else 
        	// connect audio and video of mm tx to m3g rx port
          routeUnrouteM3gAndMmDevice(index, FULL_DUP, ROUTE);
				#endif
      }
      else {
        	printandlog(index, MISC, NULL, "Error: Not all the ports information retrieved successfully check logs carefully",0);
      }

		}
	}		// incoming call
}

/****************************************************************
*   NAME: 	void process_dmev_connect(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_dmev_connect(int index)
{
  printandlog(index, MISC, NULL, "process_dmev_connect()",0);
		if(port[index].dmevConnectCtr!=2) {
				return;
		}
	port[index].dmevConnectCtr=0;
	port[index].app_state=APP_STATE_DTI_M3G_CONNECTED;
}
/****************************************************************
*   NAME: void process_dmev_disconnect(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_dmev_disconnect(int index)
{
  printandlog(index, MISC, NULL, "process_dmev_disconnect()",0);
		if(port[index].dmevDisconnectCtr!=2)
		{
				return;
		}
		port[index].bM3gAndDtiConnected=M3G_FALSE;
		port[index].dmevDisconnectCtr=0;
		port[index].app_state=APP_STATE_DTI_M3G_DISCONNECTED;
		initiate_disconnect(index);
}
/****************************************************************
*   NAME: void process_dmev_port_connect(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/

static void process_dmev_port_connect(int index)
{
  	printandlog(index, MISC, NULL, "process_dmev_port_connect()",0);
		if(port[index].dmevPortConnectCtr<HALF_DUP) {
				return;
		}
		if( (port[index].app_state==APP_STATE_H245_STARTED) 
						&& (port[index].direction==DIR_IN) 
								&& (port[index].dmevPortConnectCtr!=FULL_DUP) 
												&&  (port[index].bAnmSent==NO) ) {
				return;
		} 
		port[index].dmevPortConnectCtr=0;
		switch(port[index].app_state) 
		{
			case APP_STATE_H245_STARTED:
			{
				if(port[index].direction==DIR_IN) {
					if(port[index].bAnmSent==YES) {
						#ifndef MRBT_AFTER_ANM
        				port[index].app_state=APP_STATE_M3G_M3G_CONNECTED;
						#else
								port[index].app_state=APP_STATE_MM_M3G_CONNECTED;
						#endif
					} else {
						port[index].app_state=APP_STATE_MM_M3G_CONNECTED;
					} 
				} else if(port[index].direction==DIR_OUT) {
        			port[index].app_state=APP_STATE_M3G_M3G_CONNECTED;
					if(port[index].openAudVidOlcCmplt==AUDIO_VIDEO_LCN) {
						modifyMedia(index,M3G_E_TX);
					}
					if(port[index].respondAudVidOlcCmplt==AUDIO_VIDEO_LCN) {
						modifyMedia(index,M3G_E_TXRX);
					}
				}
				break;
			}
			case APP_STATE_MM_M3G_DISCONNECTED:
      {
        	port[index].app_state=APP_STATE_M3G_M3G_CONNECTED;
					if(port[index].openAudVidOlcCmplt==AUDIO_VIDEO_LCN) {
						modifyMedia(index,M3G_E_TX);
					}
					if(port[index].respondAudVidOlcCmplt==AUDIO_VIDEO_LCN) {
						modifyMedia(index,M3G_E_TXRX);
					}
					break;
			}
			default:
				break;
		}
}
/****************************************************************
*   NAME: 	void process_dmev_port_disconnect(int index,int devH)
*   DESCRIPTION:
*   INPUT: index and device Handle
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_dmev_port_disconnect(int index,int devH)
{
  printandlog(index, MISC, NULL, "process_dmev_port_disconnect()",0);
		if(port[index].dmevPortDisconnectCtr<HALF_DUP) {
				return;
		}
		if( (port[index].app_state==APP_STATE_MM_M3G_CONNECTED || port[index].app_state==APP_STATE_MEDIA_IDLE) 
							&& (port[index].direction==DIR_IN) && (port[index].dmevPortDisconnectCtr!=FULL_DUP) ) {
				return;
		}
	port[index].dmevPortDisconnectCtr=0;
	int prev_state=port[index].app_state;
	switch(prev_state)
  {
//			case APP_STATE_MEDIA_STARTED:
//			case APP_STATE_MRBT_PLAY_STOPPED:
			case APP_STATE_MM_M3G_CONNECTED:	
			{
					port[index].app_state=APP_STATE_MM_M3G_DISCONNECTED;
        	port[index].bM3gAndMmConnected=M3G_FALSE;
        	routeUnrouteM3gAndM3gDevice(index,FULL_DUP,ROUTE);
					break;
			}
			case APP_STATE_MEDIA_IDLE:
			{
				port[index].app_state=APP_STATE_MM_M3G_DISCONNECTED;
				port[index].bM3gAndMmConnected=M3G_FALSE;
 				routeUnrouteM3gAndM3gDevice(index,FULL_DUP,ROUTE);
				break;
			}
			case APP_STATE_H245_STOPPED:
			{
				if(port[index].bM3gAndMmConnected==M3G_TRUE) {
					port[index].app_state=APP_STATE_MM_M3G_DISCONNECTED;
					port[index].bM3gAndMmConnected=M3G_FALSE;
				} else if(port[index].bM3gAndM3gConnected==M3G_TRUE) {
					port[index].app_state=APP_STATE_M3G_M3G_DISCONNECTED;
					port[index].bM3gAndM3gConnected=M3G_FALSE;
				}
				disconnectM3gAndDti(index);
				break;
			}
			case APP_STATE_FRAMING_LOST:
			{
						port[index].app_state=APP_STATE_MM_M3G_DISCONNECTED;
						port[index].bM3gAndMmConnected=M3G_FALSE;
						port[index].bM3gAndM3gConnected=M3G_FALSE;
						stopH245(index);
						break;
			}
			default:
				break;
	}
}
/****************************************************************
*   NAME: void process_answered_event(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_answered_event(int index)
{
  printandlog(index, MISC, NULL, "process_answered_event()",0);
		switch(port[index].app_state)
		{
			case APP_STATE_DTI_M3G_CONNECTED:
			{
					struct channel    *pline=&port[index];
					inbound_application(pline);
					break;
			}
			case APP_STATE_MEDIA_STARTED:
			{
					sendH245UII(index);
					modifyMedia(index,M3G_E_IDLE);
					break;
			}
			case APP_STATE_MRBT_PLAY_STARTED:
					// SKS on answer send UII digit and stop current play
					sendH245UII(index);
					stopPlay(index);
					break;
			case APP_STATE_MM_M3G_CONNECTED:
						routeUnrouteM3gAndMmDevice(index,FULL_DUP,UNROUTE);
						break;
			case APP_STATE_H245_STARTED:
						routeUnrouteM3gAndM3gDevice(index,FULL_DUP,ROUTE);
						break;
			default:
					break;
		}
}
/****************************************************************
*   NAME: 	void process_3g_call_disconnect(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void process_3g_call_disconnect(int index)
{
 	printandlog(index, MISC, NULL, "process_3g_call_disconnect()", 0);
  int partnerIndex;
	int prev_state=port[index].app_state;
  partnerIndex=port[index].nPartnerIndex;
	
	if(port[index].bDisconnectActive==YES) {
				return;
	}

	port[index].bDisconnectActive=YES;

	switch(prev_state)
	{
       	case APP_STATE_ENDPOINT_CONNECTED: 
				{
          	stopMedia(index);
						break;
				}
       	case APP_STATE_M3G_M3G_CONNECTED:
				{
						if(port[index].bMediaStarted==M3G_TRUE || port[index].bMediaIdle==M3G_TRUE) {
          		stopMedia(index);
						}
						if (port[index].bMediaStarted==M3G_FALSE && port[index].bMediaIdle==M3G_FALSE) {
							stopH245(index);
						}
						break;
				}
       	case APP_STATE_MM_M3G_DISCONNECTED:
          	stopMedia(index);
						break;
       	case APP_STATE_MEDIA_IDLE:
       	case APP_STATE_MEDIA_STARTED:
       	case APP_STATE_MRBT_PLAY_STOPPED:
				{
          	stopMedia(index);
						break;
		   	}
       	case APP_STATE_MRBT_PLAY_STARTED:
				{
						stopPlay(index);
						break;
				}
       	case APP_STATE_MM_M3G_CONNECTED:
				{
						stopH245(index);
						break;
				}
       	case APP_STATE_H245_STARTED:
				{
						stopH245(index);
						break;
				}
       	case APP_STATE_DTI_M3G_CONNECTED:
				{
            disconnectM3gAndDti(index);
						break;
				}
				case APP_STATE_INITIAL:
				{
					initiate_disconnect(index);
					break;
				}

			// no need to handle these states are only expected when disconnect is already active
       case APP_STATE_DTI_M3G_DISCONNECTED:
						break;
       case APP_STATE_M3G_M3G_DISCONNECTED: 
						break;
       case APP_STATE_H245_STOPPED:
						break;
       case APP_STATE_MEDIA_STOPPED:
						break;
       case APP_STATE_FRAMING_LOST:
						break;
			 default:
						break;
	}
 	printandlog(index, MISC, NULL, "calling disconnect_partner_3g_call() to disconnect partner leg", 0);
  disconnect_partner_3g_call(partnerIndex);
}
/****************************************************************
*   NAME: 	void disconnect_partner_3g_call(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void disconnect_partner_3g_call(int index)
{
	printandlog(index, MISC, NULL, "disconnect_partner_3g_call()", 0);
	if(port[index].bDisconnectActive==YES) {
				return;
	}
	int prev_state=port[index].app_state;
	port[index].bDisconnectActive=YES;
	switch(prev_state)
	{
        case APP_STATE_ENDPOINT_CONNECTED:
        {
            stopMedia(index);
            break;
        }
        case APP_STATE_M3G_M3G_CONNECTED:
        {
                  if(port[index].bMediaStarted==M3G_TRUE || port[index].bMediaIdle==M3G_TRUE) {
               stopMedia(index);
                  }
                  if (port[index].bMediaStarted==M3G_FALSE && port[index].bMediaIdle==M3G_FALSE) {
                     stopH245(index);
                  }
            break;
        }
        case APP_STATE_MM_M3G_DISCONNECTED:
            stopMedia(index);
            break;
        case APP_STATE_MEDIA_IDLE:
        {
            stopMedia(index);
            break;
        }
        case APP_STATE_MEDIA_STARTED:
        {
            stopMedia(index);
            break;
        }
        case APP_STATE_MRBT_PLAY_STOPPED:
        {
            stopMedia(index);
            break;
        }
        case APP_STATE_MRBT_PLAY_STARTED:
        {
            stopPlay(index);
            break;
        }
        case APP_STATE_MM_M3G_CONNECTED:
        {
            stopH245(index);
            break;
        }
        case APP_STATE_H245_STARTED:
        {
            stopH245(index);
            break;
        }
        case APP_STATE_DTI_M3G_CONNECTED:
        {
            disconnectM3gAndDti(index);
            break;
        }
        case APP_STATE_INITIAL:
        {
          initiate_disconnect(index);
          break;
        }

      // no need to handle these states are only expected when disconnect is already active
       case APP_STATE_DTI_M3G_DISCONNECTED:
            break;
       case APP_STATE_M3G_M3G_DISCONNECTED:
            break;
       case APP_STATE_H245_STOPPED:
            break;
       case APP_STATE_MEDIA_STOPPED:
            break;
       case APP_STATE_FRAMING_LOST:
            break;
       default:
            break;
	}
}

/****************************************************************
*   NAME: 	void cleanResourcesBeforeExit()
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void cleanResourcesBeforeExit()
{
	int i;
	for(i=0;i<num_devices;i++) {
		if(port[i].direction==DIR_IN) {
			if(dx_close(port[i].dxH)==-1) {
	 			printandlog(i,DX,NULL,"dx_close() failed",0);
			} else {
	 			printandlog(i,DX,NULL,"dx_close() success",0);
			}
		}
		if(m3g_Close(port[i].m3gControlH)==M3G_SUCCESS) {
	 		printandlog(i,M3G,NULL,"m3g_Close() success for Control device",0);
		} else {
	 		printandlog(i,M3G,NULL,"m3g_Close() failed for Control device",0);
		}
		if(m3g_Close(port[i].m3gAudioH)==M3G_SUCCESS) {
	 		printandlog(i,M3G,NULL,"m3g_Close() success for Audio device",0);
		} else {
	 		printandlog(i,M3G,NULL,"m3g_Close() failed for Audio device",0);
		}
		if(m3g_Close(port[i].m3gVideoH)==M3G_SUCCESS) {
	 		printandlog(i,M3G,NULL,"m3g_Close() success for Video device",0);
		} else {
	 		printandlog(i,M3G,NULL,"m3g_Close() failed for Video device",0);
		}
		if(mm_Close(port[i].mmH,NULL)==M3G_SUCCESS) {
	 		printandlog(i,MM,NULL,"mm_Close() success for mm device",0);
		} else {
	 		printandlog(i,MM,NULL,"mm_Close() failed for mm device",0);
		}
	}
	stopM3gTrace(BOARD_DEVICE,m3gBoardH);
	stopM3g(BOARD_DEVICE);
	initiate_exit();
}
/****************************************************************
*   NAME: 	 void initiate_exit_video_call()
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void initiate_exit_video_call()
{
	static int exit_active = NO;
   if (exit_active == YES) {
      return;
   }
	exit_active=YES;
	printandlog(ALL_DEVICES, MISC, NULL, "********** Exit initiated (VIDEO) ***********\n", 0);
  cleanResourcesBeforeExit();
}
/****************************************************************
*   NAME: 	void sendH245VFU(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void sendH245VFU(int index)
{
	 	M3G_H245_MISC_CMD h245MiscCmd;
   	INIT_M3G_H245_MISC_CMD(&h245MiscCmd);

   	h245MiscCmd.logicalChannelNumber = port[index].remoteVideoLogicalChannel;
   	h245MiscCmd.h245MiscCmdType = M3G_E_FAST_UPDATE_PICTURE;
   	h245MiscCmd.h245MiscCmdParams.noParams = NULL;

   	if (m3g_SendH245MiscCmd(port[index].m3gControlH, &h245MiscCmd)==M3G_SUCCESS) {
	 		printandlog(index,M3G,NULL,"m3g_SendH245MiscCmd(VFU) Video Fast Update success",0);
		} else {
	 		printandlog(index,M3G,NULL,"m3g_SendH245MiscCmd(VFU) Video Fast Update failed",0);
		}
}
/****************************************************************
*   NAME: void sendH245UII(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void sendH245UII(int index)
{
	if(port[index].bDisconnectActive==YES) {
      return;
  }

	M3G_H245_UII h245UIIBuf;
	char digBuf;
	h245UIIBuf.version = M3G_LIBRARY_VERSION;
	h245UIIBuf.numDigits = 1;
	switch(port[index].app_state)
	{
		case APP_STATE_ENDPOINT_CONNECTED:
		case APP_STATE_MEDIA_STARTED:
		{
				if(port[index].bStopTimerDigitSent==M3G_FALSE) {
					// send digit 0
					digBuf='0';
					port[index].bStopTimerDigitSent=M3G_TRUE;
				} else if(port[index].bStartTimerDigitSent==M3G_FALSE) {
					digBuf='1';
					port[index].bStartTimerDigitSent=M3G_TRUE;
				} else {
					return;
				}
				break;
		}
		case APP_STATE_MRBT_PLAY_STARTED:
		{
					// send digit 1
					digBuf='1';
					port[index].bStartTimerDigitSent=M3G_TRUE;
					break;
		}
		default:
					break;
	}
	h245UIIBuf.digitBuffer[0]=digBuf;
	if (m3g_SendH245UII(port[index].m3gControlH, &h245UIIBuf)==M3G_SUCCESS) {
			printandlog(index,M3G,NULL,"m3g_SendH245UII() successful",0);
	}
	else {
			printandlog(index,M3G,NULL,"m3g_SendH245UII() successful",0);
	}
}
/****************************************************************
*   NAME: 	void m3gEnableDisableEvents(int index,int handle)
*   DESCRIPTION:
*   INPUT: index and device handle
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void m3gEnableDisableEvents(int index,int handle)
{
	unsigned int enableEvtBitMask=M3G_H245_UII_EVT_TYP|M3G_MES_EVTS_EVT_TYP|M3G_REMOTE_VENDORID_EVT_TYP|M3G_VERBOSE_MSD_EVT_TYP;
	unsigned int disableEvtBitMask=M3G_H245_FASTUPDATE_EVT_TYP|M3G_H245_TEMP_SPAT_TRDFF_EVT_TYP|M3G_H245_VIDEO_FREEZE_EVT_TYP;

	if(m3g_EnableEvents(handle, enableEvtBitMask)!=M3G_SUCCESS) {
			printandlog(index,M3G,NULL,"m3g_EnableEvents() failed",0);
	}
	else {
			printandlog(index,M3G,NULL,"m3g_EnableEvents() successful",0);
	}
	if(m3g_DisableEvents(handle, disableEvtBitMask)!=M3G_SUCCESS) {
			printandlog(index,M3G,NULL,"m3g_DisableEvents() failed",0);
	}
	else {
			printandlog(index,M3G,NULL,"m3g_DisableEvents() successful",0);
	}
	
}

/****************************************************************
*   NAME:	void resetVariables(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/

static void resetVariables(int index)
{
	printandlog(index,M3G,NULL,"resetVariables() function called",0);
	port[index].remoteAudioLogicalChannel=0;
  	port[index].remoteVideoLogicalChannel=0;

  	port[index].bPlayActive=M3G_FALSE;
	port[index].bMediaStarted=M3G_FALSE;
	port[index].bH245Started=M3G_FALSE;
	port[index].bM3gAndDtiConnected=M3G_FALSE;
	port[index].bM3gAndMmConnected=M3G_FALSE;
	port[index].bM3gAndM3gConnected=M3G_FALSE;

	port[index].app_state=APP_STATE_INITIAL;
  port[index].bStopTimerDigitSent=M3G_FALSE;
  port[index].bStartTimerDigitSent=M3G_FALSE;


	port[index].originalCalledPartyNumber[0]='\0';
	port[index].calledPartyNumber[0]='\0';
	port[index].callingPartyNumber[0]='\0';

//  port[index].bMRBTPlayRequired=M3G_TRUE;

  port[index].bMmTxAndM3gRxConnected=M3G_FALSE;
  port[index].bM3gTxAndMmRxConnected=M3G_FALSE;
  port[index].bM3gTxAndM3gRxConnected=M3G_FALSE;


 	port[index].bAcmSent=NO;
 	port[index].bAnmSent=NO;

	port[index].bDisconnectActive=NO;

  port[index].localCapCtr=0;
	port[index].olcEvtMask=0;
	port[index].modifyMediaCtr=0;
	port[index].stopMediaCtr=0;

  port[index].dmevPortDisconnectCtr=0;
  port[index].dmevPortConnectCtr=0;
  port[index].dmevConnectCtr=0;
  port[index].dmevDisconnectCtr=0;


	port[index].openAudVidOlcCmplt=0;
	port[index].respondAudVidOlcCmplt=0;

	memset(&port[index].h223CapsList, 0, sizeof(M3G_CAPS_LIST));
	memset(&port[index].audioCapsList, 0, sizeof(M3G_CAPS_LIST));
	memset(&port[index].videoCapsList, 0, sizeof(M3G_CAPS_LIST));
}
/****************************************************************
*   NAME: void stopPlay(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void stopPlay(int index)
{

	if(port[index].bPlayActive==M3G_FALSE) {
			return;
	}

	 MM_STOP mmStopInfo[2];
   MM_STOP_DETAILS mmStopDetails;
	
//  	port[index].bMRBTPlayRequired=M3G_FALSE;
//	  port[index].bPlayActive=M3G_FALSE;

    memset(&mmStopDetails, 0, sizeof(MM_STOP_DETAILS));

    mmStopInfo[0].unVersion = 0;
    mmStopInfo[0].ItemChain = EMM_ITEM_CONT;
    mmStopInfo[0].ItemType = EMM_STOP_VIDEO_PLAY;
    mmStopInfo[0].details = mmStopDetails;
    mmStopInfo[0].next = &mmStopInfo[1];

    mmStopInfo[1].unVersion = 0;
    mmStopInfo[1].ItemChain = EMM_ITEM_EOT;
    mmStopInfo[1].ItemType = EMM_STOP_AUDIO_PLAY;
    mmStopInfo[1].details = mmStopDetails;
    mmStopInfo[1].next = NULL;

    if (mm_Stop(port[index].mmH, mmStopInfo, &index) < 0) {
			printandlog(index,MM,NULL,"mm_Stop() for Play failed",0);
    }
    else  {
			printandlog(index,MM,NULL,"mm_Stop() for Play success",0);
    }
}
/****************************************************************
*   NAME:	void playVideoPrompt(int index)
*   DESCRIPTION: play mrbt prompt
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void playVideoPrompt(int index)
{
  if(port[index].bDisconnectActive==YES) {
      return;
  }
  if( (port[index].bPlayActive==M3G_TRUE) || (port[index].direction==DIR_OUT)) {
      return;
  }
  port[index].bPlayActive=M3G_TRUE;


   char str[MAX_STRING_SIZE];


   MM_AUDIO_CODEC audioCodecType;

   MM_PLAY_RECORD_LIST m_playRecordList[2];
   MM_PLAY_INFO        m_playInfo;

   MM_MEDIA_ITEM_LIST m_audioMediaList;
   MM_MEDIA_ITEM_LIST m_videoMediaList;

   INIT_MM_AUDIO_CODEC(&audioCodecType);

   audioCodecType.unSampleRate        = MM_DRT_8KHZ;
   audioCodecType.unBitsPerSample        = 16;
   audioCodecType.unCoding            = MM_DATA_FORMAT_AMR_NB_12_20K;

   INIT_MM_MEDIA_ITEM_LIST(&m_audioMediaList);
   INIT_MM_MEDIA_AUDIO(&m_audioMediaList.item.audio);
   m_audioMediaList.ItemChain             = EMM_ITEM_EOT;
   m_audioMediaList.item.audio.codec         = audioCodecType;
   m_audioMediaList.item.audio.unMode = 0; //MM_MODE_AUD_BEEPINITIATED;
   m_audioMediaList.item.audio.unOffset = 0;
   m_audioMediaList.item.audio.szFileName       = port[index].audioFileName;
   m_audioMediaList.item.audio.eFileFormat      = EMM_AUD_FILEFORMAT_PROPRIETARY;

   INIT_MM_MEDIA_ITEM_LIST(&m_videoMediaList);
   INIT_MM_MEDIA_VIDEO(&m_videoMediaList.item.video);
   m_videoMediaList.ItemChain = EMM_ITEM_EOT;
   m_videoMediaList.item.video.szFileName       = port[index].videoFileName;
   m_videoMediaList.item.video.unMode = 0;
   m_videoMediaList.item.video.eFileFormat      = EMM_FILE_FORMAT_PROPRIETARY;

	if(port[index].cfgCoderType==M3G_E_MPEG4) {
      m_videoMediaList.item.video.codec.Coding               = VIDEO_CODING_MP4V_ES;
      m_videoMediaList.item.video.codec.Profile              = VIDEO_PROFILE_LEVEL_SP0_MPEG4;
	} else {
   	m_videoMediaList.item.video.codec.Coding               = VIDEO_CODING_H263;
   	m_videoMediaList.item.video.codec.Profile              = VIDEO_PROFILE_0_H263;
	}
   m_videoMediaList.item.video.codec.ImageWidth           = VIDEO_IMAGE_WIDTH_176;
   m_videoMediaList.item.video.codec.ImageHeight          = VIDEO_IMAGE_HEIGHT_144;
   m_videoMediaList.item.video.codec.FramesPerSec         = VIDEO_FRAMESPERSEC_15;

   INIT_MM_PLAY_RECORD_LIST(&m_playRecordList[0]);
   m_playRecordList[0].ItemType        = EMM_MEDIA_TYPE_AUDIO;
   m_playRecordList[0].list            = &m_audioMediaList;
   m_playRecordList[0].ItemChain          = EMM_ITEM_CONT;

   INIT_MM_PLAY_RECORD_LIST(&m_playRecordList[1]);
   m_playRecordList[1].ItemType        = EMM_MEDIA_TYPE_VIDEO;
   m_playRecordList[1].list            = &m_videoMediaList;
   m_playRecordList[1].ItemChain          = EMM_ITEM_EOT;

   INIT_MM_PLAY_RECORD_INFO(&m_playInfo);
   m_playInfo.eFileFormat           = EMM_FILE_FORMAT_PROPRIETARY;
   m_playInfo.list                     = m_playRecordList;

   sprintf(str,"Starting Play on videoFile=%s|audioFile=%s",m_videoMediaList.item.video.szFileName,m_audioMediaList.item.audio.szFileName);
   printandlog(index,MM,NULL,str,0);

   if ( mm_Play(port[index].mmH, &m_playInfo, NULL, NULL) < 0 )
   {
      sprintf(str,"mm_Play() failed on %s", port[index].mmDeviceName);
   }
   else {
      sprintf(str,"mm_Play() success on %s", port[index].mmDeviceName);
   }
   printandlog(index,MM,NULL,str,0);
}

/****************************************************************
*   NAME: void modifyMedia(int index,M3G_E_DIRECTION direction)
*   DESCRIPTION:
*   INPUT: index and direction
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void modifyMedia(int index,M3G_E_DIRECTION direction)
{
	if(port[index].bDisconnectActive==YES) {
      return;
  }
	char str[MAX_STRING_SIZE];
	if(port[index].bMediaStarted==M3G_FALSE) {
		port[index].bMediaStarted=M3G_TRUE;
	} else if(direction!=M3G_E_IDLE) {
		direction=M3G_E_TXRX;
	}

	if (m3g_ModifyMedia(port[index].m3gAudioH, direction)==M3G_SUCCESS) {
         sprintf(str,"m3g_ModifyMedia(%d) for audio success dir=%s",port[index].m3gAudioH,dirStr[direction]);
   }
 	else {
         sprintf(str,"m3g_ModifyMedia(%d) audio failure dir=%s",port[index].m3gAudioH,dirStr[direction]);
   }
   printandlog(index,M3G,NULL,str,0);

	if (m3g_ModifyMedia(port[index].m3gVideoH, direction)==M3G_SUCCESS) {
         sprintf(str,"m3g_ModifyMedia(%d) video success dir=%s",port[index].m3gVideoH,dirStr[direction]);
   }
 	else {
         sprintf(str,"m3g_ModifyMedia(%d) video failure dir=%s",port[index].m3gVideoH,dirStr[direction]);
   }
   printandlog(index,M3G,NULL,str,0);
}

/****************************************************************
*   NAME:	void sendOLCAck(int index,M3G_REMOTE_OLC_REQ* remoteOlcReq)
*   DESCRIPTION:
*   INPUT: index and pointer to M3G_REMOTE_OLC_REQ
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void sendOLCAck(int index,M3G_REMOTE_OLC_REQ* remoteOlcReq)
{
	char str[MAX_STRING_SIZE];
	if(remoteOlcReq->capabilityType==M3G_E_AUDIO_CAPABILITY) {
		port[index].remoteAudioLogicalChannel=remoteOlcReq->logicalChannelNumber;
	}
	else {
		port[index].remoteVideoLogicalChannel=remoteOlcReq->logicalChannelNumber;
	}
	if (m3g_RespondToOLC(port[index].m3gControlH, remoteOlcReq->logicalChannelNumber, M3G_E_OLCACK)==M3G_SUCCESS ) {
      sprintf(str,"m3g_RespondToLC(%d) success for type %d lcn=%d",port[index].m3gControlH,remoteOlcReq->capabilityType,remoteOlcReq->logicalChannelNumber);
   }
   else {
      sprintf(str,"m3g_RespondToLC(%d) failure for type %d lcn=%d",port[index].m3gControlH,remoteOlcReq->capabilityType,remoteOlcReq->logicalChannelNumber);
   }
	
   printandlog(index,MISC,NULL,str,0);
}
/****************************************************************
*   NAME:	 void sendOpenLC(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void sendOpenLC(int index)
{
	char str[MAX_STRING_SIZE];
	int i=0;
	// since we have only one audio/video into local capability list
	M3G_H223_LC_PARAMS h223OLCParams;
	
	int handle=port[index].m3gControlH;

	M3G_AUDIO_CAPABILITY* pAudioCap = NULL;
   M3G_VIDEO_CAPABILITY* pVideoCap = NULL;
	
	for(i=0;i<port[index].videoCapsList.numCaps;i++) {
			if(port[index].videoCapsList.capability[i].videoCapability.direction==M3G_E_TX) {
				pVideoCap= &port[index].videoCapsList.capability[i].videoCapability;
			}
	}
	for(i=0;i<port[index].audioCapsList.numCaps;i++) {
		if(port[index].audioCapsList.capability[i].audioCapability.direction==M3G_E_TX) {
			pAudioCap= &port[index].audioCapsList.capability[i].audioCapability;
		}
	}

// send audio OLC

   INIT_M3G_H223_LC_PARAMS(&h223OLCParams);
   h223OLCParams.adaptationLayerType=M3G_E_AL2_WITHOUT_SEQ_NUMS;	//M3G_E_AL2_WITHOUT_SEQ_NUMS
   h223OLCParams.segmentable = M3G_FALSE;									// audio is always not segmented

	//SKS rest default values will be initialzed by INIT_M3G_H223_LC_PARAMS() function
   if(m3g_OpenLC(handle, &h223OLCParams, M3G_E_AUDIO_CAPABILITY,(M3G_CAPABILITY*)pAudioCap) == M3G_SUCCESS) {
         sprintf(str,"m3g_OpenLC(%d) success for audio",handle);
   } else {
         sprintf(str,"m3g_OpenLC(%d) failed for audio",handle);
   }
   printandlog(index,M3G,NULL,str,0);

 	memset(&h223OLCParams, 0, sizeof(M3G_H223_LC_PARAMS));

// send video OLC

   INIT_M3G_H223_LC_PARAMS(&h223OLCParams);

   //h223OLCParams.adaptationLayerType=M3G_E_AL2_WITH_SEQ_NUMS;	//M3G_E_AL2_WITH_SEQ_NUMS
   h223OLCParams.adaptationLayerType=M3G_E_AL2_WITHOUT_SEQ_NUMS;	//M3G_E_AL2_WITH_SEQ_NUMS
   h223OLCParams.segmentable = M3G_TRUE;				// video is always segmented

	if(port[index].cfgCoderType==M3G_E_H263) {
		pVideoCap->options.h263.qcifMPI=2;
		pVideoCap->options.h263.sqcifMPI=0;
	} else {
// SKS	TODO for MPEG4

		if( (port[index].xcode==true) || (port[index].direction==DIR_IN) )  {
			unsigned int i=0;
			unsigned char strDCI[MAX_FILE_NAME];
			unsigned char dciOctetStr[OCTET_STRING_SIZE];
			strcpy(strDCI,DLGC_XCODER_QCIF_DCI_STR);
      	unsigned char length=(char)strlen(strDCI)/2;
      	// convert xcoder DCI from ASCII to binary:
      	for (i = 0; i <length; i++)
      	{
         	dciOctetStr[i] = AsciiOctetToHex(&strDCI[2*i]);
      	}
      	memcpy(pVideoCap->options.mpeg4.decoderConfigInfo,dciOctetStr,length);

			pVideoCap->options.mpeg4.profileAndLevel=M3G_MPEG4_SIMPLE_PROFILE_LEVEL0;
			pVideoCap->options.mpeg4.decoderConfigLength=length;
		} else {
				if (port[index].direction==DIR_OUT) {
					int nPartnerIndex=port[index].nPartnerIndex;
         		memcpy(pVideoCap->options.mpeg4.decoderConfigInfo,port[nPartnerIndex].handPhoneDCIValue,port[nPartnerIndex].handPhoneDCILen);
         		pVideoCap->options.mpeg4.profileAndLevel=M3G_MPEG4_SIMPLE_PROFILE_LEVEL0;
         		pVideoCap->options.mpeg4.decoderConfigLength=port[nPartnerIndex].handPhoneDCILen;
				}
		}
	}
	//SKS rest default values will be initialzed by INIT_M3G_H223_LC_PARAMS() function

   if(m3g_OpenLC(handle, &h223OLCParams, M3G_E_VIDEO_CAPABILITY,(M3G_CAPABILITY*)pVideoCap) == M3G_SUCCESS) {
         sprintf(str,"m3g_OpenLC(%d) success for video",handle);
   } else {
         sprintf(str,"m3g_OpenLC(%d) failed for video",handle);
   }
	printandlog(index,M3G,NULL,str,0);
}

/****************************************************************
*   NAME:	getMatchedCapability(int index,int handle, M3G_CAPS_LIST *matchedCapsList)
*   DESCRIPTION:
*   INPUT: index, device handle and pointer to M3G_CAPS_LIST
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
void getMatchedCapability(int index,int handle, M3G_CAPS_LIST *matchedCapsList)
{
	char str[MAX_STRING_SIZE];
   if (m3g_GetMatchedCaps(handle, matchedCapsList)==M3G_SUCCESS) {
      sprintf(str,"m3g_GetMatchedCaps() success for handle=%d",handle);
   } else {
      sprintf(str,"m3g_GetMatchedCaps() failed for handle=%d",handle);
   }
   printandlog(index,M3G,NULL,str,0);
}
/****************************************************************
*   NAME:		void getMatchedCapabilityAndOpenLC(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void getMatchedCapabilityAndOpenLC(int index)
{

//	getMatchedCapability(port[index].m3gControlH,&port[index].h223CapsList);
	getMatchedCapability(index,port[index].m3gVideoH,&port[index].videoCapsList);
	getMatchedCapability(index,port[index].m3gAudioH,&port[index].audioCapsList);

	sendOpenLC(index);
}
/****************************************************************
*   NAME:	void resetM3gDevice(int index,int handle)
*   DESCRIPTION:
*   INPUT: index and handle
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void resetM3gDevice(int index,int handle)
{
	char str[MAX_STRING_SIZE];
	if (m3g_Reset(handle) !=M3G_SUCCESS) {
		sprintf(str,"m3g_Reset(%d) failed",handle);
   }
   else {
		sprintf(str,"m3g_Reset(%d) successfull",handle);
   }
   printandlog(index,M3G,NULL,str,0);
}
/****************************************************************
*   NAME:	void openM3gMmChannel()
*   DESCRIPTION:
*   INPUT: NONE
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void openM3gMmChannel()
{
	int index;
	for(index=0;index<num_devices;index++) {
		port[index].txPortCtr=0;
		port[index].rxPortCtr=0;
		resetVariables(index);
		open_m3g_channel(index);
		open_mm_channel(index);
	}
}
/****************************************************************
*   NAME:			void stopM3gTrace(int index,int handle)
*   DESCRIPTION:
*   INPUT: index and device handle
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void stopM3gTrace(int index,int handle)
{
	if (m3g_StopTrace(handle)!=M3G_SUCCESS) {
      printandlog(index,M3G,NULL,"m3g_StopTrace failure",0);
   }
	else
      printandlog(index,M3G,NULL,"m3g_StopTrace successfull",0);
}
/****************************************************************
*   NAME:		void startM3gTrace(int index,int handle)
*   DESCRIPTION:
*   INPUT: index and device handle
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void startM3gTrace(int index,int handle)
{
	M3G_TRACE_INFO trcInfo;
	INIT_M3G_TRACE_INFO(&trcInfo);
	char strFileName[MAX_FILE_NAME];

	sprintf(strFileName,"../log/mrbt_m3g_trc_%d.txt",rand());

	trcInfo.logfile = strFileName;
	trcInfo.bitmask =M3G_TRACE_H245|M3G_TRACE_H223|M3G_TRACE_AUDIO|M3G_TRACE_VIDEO|M3G_TRACE_INTERNALS;
	if (m3g_StartTrace(handle, &trcInfo)!=M3G_SUCCESS) {
		printandlog(index,M3G,NULL,"m3g_StartTrace failed",0);
	}
	else {
		printandlog(index,M3G,NULL,"m3g_StartTrace successfull",0);
	}
}
/****************************************************************
*   NAME:	void stopMedia(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void stopMedia(int index)
{
	char str[MAX_STRING_SIZE];
	
	if(port[index].bMediaStarted==M3G_FALSE) {
			return;
	}
  port[index].bMediaStarted=M3G_FALSE;

	if (m3g_StopMedia(port[index].m3gVideoH)==M3G_SUCCESS) {
     	sprintf(str,"m3g_StopMedia() success for m3g Video");
  } else {
     	sprintf(str,"m3g_StopMedia() failed for m3g Video");
  }
	printandlog(index,M3G,NULL,str,0);

	if (m3g_StopMedia(port[index].m3gAudioH)==M3G_SUCCESS) {
     	sprintf(str,"m3g_StopMedia() success for m3g Audio");
  } else {
     	sprintf(str,"m3g_StopMedia() failed for m3g Audio");
  }
	printandlog(index,M3G,NULL,str,0);
}
/****************************************************************
*   NAME:		void stopH245(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void stopH245(int index)
{
	char str[MAX_STRING_SIZE];

  if(port[index].bH245Started==M3G_FALSE) {
         return;
  }
  port[index].bH245Started=M3G_FALSE;

	int handle = port[index].m3gControlH;
	if (m3g_StopH245(handle)==M3G_SUCCESS ) {
      sprintf(str,"m3g_StopH245(%d) successfull",handle);
  } else {
      sprintf(str,"m3g_StopH245(%d) failure",handle);
  }
	printandlog(index,M3G,NULL,str,0);
}
/****************************************************************
*   NAME:			void startH245(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void startH245(int index)
{
	char str[MAX_STRING_SIZE];

	if(port[index].bDisconnectActive==YES) {
			return;
	}

   if(port[index].bH245Started==M3G_TRUE) {
         return;
   }
   port[index].bH245Started=M3G_TRUE;


	M3G_H223_SESSION h223Session;
   INIT_M3G_H223_SESSION(&h223Session);

	int handle=port[index].m3gControlH;

	h223Session.defaultH223MuxLevel=M3G_E_H223_MUX_LEVEL2;
	h223Session.maxALSDUSize=(unsigned int)1024;
 	h223Session.isWNSRPEnabled=M3G_TRUE;
	h223Session.isMONAEnabled=M3G_TRUE;
	h223Session.isMultipleMsgsPerPdu=M3G_TRUE;

	if (m3g_StartH245(handle,&h223Session)!=M3G_SUCCESS ) {
      sprintf(str,"m3g_StartH245() failure handle=%d",handle);
   } else {
      sprintf(str,"m3g_StartH245() successfull handle=%d",handle);
   }
	printandlog(index,M3G,NULL,str,0);
}
/****************************************************************
*   NAME:		void printPortInfoList(int index,DM_PORT_INFO_LIST* pPortInfoList)
*   DESCRIPTION:
*   INPUT: index and pointer to DM_PORT_INFO_LIST
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void printPortInfoList(int index,DM_PORT_INFO_LIST* pPortInfoList)
{
	char temp[512];
   char tmpBuff[128] = {0};
   char tokenBuff[10] = {0};
	int listCtr;
	int ctr;
   sprintf(temp,"DM_PORT_INFO_LIST(%d):----> ",pPortInfoList->unCount);
	printandlog(index,DM,NULL,temp,0);

   for (listCtr=0; listCtr< pPortInfoList->unCount; listCtr++ ) {
     	for (ctr=0; ctr<16; ctr++ ) {
         sprintf(tokenBuff, "0x%.2x ", (int)pPortInfoList->port_info[listCtr].device_ID[ctr]);
         strcat(tmpBuff,tokenBuff);
     	}
     	sprintf(temp,"DM_PORT_INFO#%d devID=%s|portID=%s|mediaType=%s",listCtr,tmpBuff,(char *)(pPortInfoList->port_info[listCtr].port_ID),
																							portMediaTypeStr[pPortInfoList->port_info[listCtr].port_media_type]);
		printandlog(index,DM,NULL,temp,0);
   }
}
/****************************************************************
*   NAME:			void printPortInfo(int index,DM_PORT_INFO* pPortInfo, char str[5])
*   DESCRIPTION:
*   INPUT: index, pointer to DM_PORT_INFO and mediaType (Audio/Video)
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void printPortInfo(int index,DM_PORT_INFO* pPortInfo, char str[5])
{
   char temp[512];
   char tmpBuff[128] = {0};
   char tokenBuff[10] = {0};
   int ctr;
   for (ctr=0; ctr<16; ctr++ ) {
         sprintf(tokenBuff, "0x%.2x,",(int)pPortInfo->device_ID[ctr]);
         strcat(tmpBuff,tokenBuff);
   }
   sprintf(temp,"DM_PORT_INFO[%s]: devID=%s|portID=%s|mediaType=%s",str,tmpBuff,(char *)(pPortInfo->port_ID),portMediaTypeStr[pPortInfo->port_media_type]);
   printandlog(index,DM,NULL,temp,0);
}

/****************************************************************
*   NAME:			void routeUnrouteM3gAndMmDevice(int index,int conType, int flag)
*   DESCRIPTION:
*   INPUT: index, conType(HALF_DUP | FULL_DUP) and flag(route|unroute)
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void routeUnrouteM3gAndMmDevice(int index,int conType, int flag)
{

	if(port[index].bDisconnectActive==YES && flag==ROUTE) {
      return;
  }

  char str[MAX_STRING_SIZE];
  if( (flag==UNROUTE && port[index].bM3gAndMmConnected==M3G_FALSE) || (flag==ROUTE && port[index].bM3gAndMmConnected==M3G_TRUE)) {
         return;
  }

  if(flag==ROUTE) {
     port[index].bM3gAndMmConnected=M3G_TRUE;
  } else if(flag==UNROUTE) {
      port[index].bM3gAndMmConnected=M3G_FALSE;
  }


	DM_PORT_CONNECT_INFO_LIST portConnectInfoList={0};

  INIT_DM_PORT_CONNECT_INFO_LIST(&portConnectInfoList);
	portConnectInfoList.unCount = 1;

  INIT_DM_PORT_CONNECT_INFO(&portConnectInfoList.port_connect_info[0]);

	if(port[index].xcode==true) {
 		portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_ON;
	} else {
 		portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_NATIVE;
	}

	INIT_DM_PORT_INFO(&portConnectInfoList.port_connect_info[0].port_info_tx);
	INIT_DM_PORT_INFO(&portConnectInfoList.port_connect_info[0].port_info_rx);

	if(flag==ROUTE) {
			printPortInfo(index,&port[index].m3gAudioRxPortInfo,"M3G");
			printPortInfo(index,&port[index].mmAudioTxPortInfo,"MM ");
			printPortInfo(index,&port[index].m3gVideoRxPortInfo,"M3G");
			printPortInfo(index,&port[index].mmVideoTxPortInfo,"MM ");
	}
	
   	memcpy(&(portConnectInfoList.port_connect_info[0].port_info_tx), &port[index].mmAudioTxPortInfo, sizeof(DM_PORT_INFO));
   	memcpy(&(portConnectInfoList.port_connect_info[0].port_info_rx), &port[index].m3gAudioRxPortInfo, sizeof(DM_PORT_INFO));

		//printPortInfo(index,&portConnectInfoList->port_connect_info[0]);
		if(flag==ROUTE) {
   		if( dev_PortConnect(port[index].mmH, &portConnectInfoList, &index)==DEV_SUCCESS ) {
     			sprintf(str,"dev_PortConnect() success mmTxAud(%s) - m3gRxAud(%s)",port[index].mmDeviceName ,port[index].m3gDeviceName);
   		} else {
     			sprintf(str,"dev_PortConnect() failed mmTxAud(%s) - m3gRxAud(%s)",port[index].mmDeviceName ,port[index].m3gDeviceName);
   		}
		}
		else if(flag==UNROUTE) {
   		if(dev_PortDisconnect(port[index].mmH, &portConnectInfoList, &index)==DEV_SUCCESS ) {
     			sprintf(str,"dev_PortDisconnect() success mmTxAud(%s) - m3gRxAud(%s)",port[index].mmDeviceName ,port[index].m3gDeviceName);
   		} else {
     			sprintf(str,"dev_PortDisconnect() failed mmTxAud(%s) - m3gRxAud(%s)",port[index].mmDeviceName ,port[index].m3gDeviceName);
   		}
		}
		printandlog(index,DM,NULL,str,0);

  	INIT_DM_PORT_CONNECT_INFO_LIST(&portConnectInfoList);

		portConnectInfoList.unCount = 1;
  	INIT_DM_PORT_CONNECT_INFO(&portConnectInfoList.port_connect_info[0]);
	
		if(port[index].xcode==true) {
      	portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_ON;
   	} else {
 			portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_NATIVE;
		}

   	memcpy(&(portConnectInfoList.port_connect_info[0].port_info_tx), &port[index].mmVideoTxPortInfo, sizeof(DM_PORT_INFO));
   	memcpy(&(portConnectInfoList.port_connect_info[0].port_info_rx), &port[index].m3gVideoRxPortInfo, sizeof(DM_PORT_INFO));

   	// This call connects the Tx of MMdevice to Rx of 3G device (Video)
		if(flag==ROUTE) {
   		if (dev_PortConnect(port[index].mmH, &portConnectInfoList, &index)==DEV_SUCCESS ) {
    		sprintf(str, "dev_PortConnect() success mmTxVid(%s) - m3gRxVid(%s)",port[index].mmDeviceName ,port[index].m3gDeviceName);
  	 	} else {
    		sprintf(str, "dev_PortConnect() failed mmTxVid(%s) - m3gRxVid(%s)",port[index].mmDeviceName ,port[index].m3gDeviceName);
   		}
		}
		else if(flag==UNROUTE) {
      if(dev_PortDisconnect(port[index].mmH, &portConnectInfoList, &index)==DEV_SUCCESS ) {
         sprintf(str,"dev_PortDisconnect() success mmTxVid(%s) - m3gRxVid(%s)",port[index].mmDeviceName ,port[index].m3gDeviceName);
      } else {
         sprintf(str,"dev_PortDisconnect() failed mmTxVid(%s) - m3gRxVid(%s)",port[index].mmDeviceName ,port[index].m3gDeviceName);
      }
   	}
		printandlog(index,DM,NULL,str,0);
	if(conType==HALF_DUP)
	{
		return;			// SKS return since in MRBT system always Half-Duplex connection is required.
	}

// m3g tx to mm rx
	if(flag==ROUTE) {
			printPortInfo(index,&port[index].m3gAudioTxPortInfo,"M3G");
			printPortInfo(index,&port[index].mmAudioRxPortInfo,"MM ");
			printPortInfo(index,&port[index].m3gVideoTxPortInfo,"M3G");
			printPortInfo(index,&port[index].mmVideoRxPortInfo,"MM ");
	}
   INIT_DM_PORT_CONNECT_INFO_LIST(&portConnectInfoList);

   portConnectInfoList.unCount = 1;
   INIT_DM_PORT_CONNECT_INFO(&portConnectInfoList.port_connect_info[0]);


		if(port[index].xcode==true) {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_ON;
      } else {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_NATIVE;
      }

   memcpy(&(portConnectInfoList.port_connect_info[0].port_info_tx), &port[index].m3gAudioTxPortInfo, sizeof(DM_PORT_INFO));
   memcpy(&(portConnectInfoList.port_connect_info[0].port_info_rx), &port[index].mmAudioRxPortInfo, sizeof(DM_PORT_INFO));

	if(flag==ROUTE) {
   	if ( dev_PortConnect(port[index].m3gAudioH, &portConnectInfoList, &index)==DEV_SUCCESS ) {
      	sprintf(str, "dev_PortConnect() success m3gTxAud(%s) - mmRxAud(%s)",port[index].m3gDeviceName,port[index].mmDeviceName);
   	} else {
      	sprintf(str, "dev_PortConnect() failed for m3gTxAud(%s) - mmRxAud(%s)",port[index].m3gDeviceName,port[index].mmDeviceName);
   	}
	}
	else if(flag==UNROUTE) {
	  	if (dev_PortDisconnect(port[index].m3gAudioH, &portConnectInfoList, &index) ==DEV_SUCCESS) {
         sprintf(str, "dev_PortDisconnect() success m3gTxAud(%s) - mmRxAud(%s)",port[index].m3gDeviceName,port[index].mmDeviceName);
      } else {
         sprintf(str, "dev_PortDisconnect() failed for m3gTxAud(%s) - mmRxAud(%s)",port[index].m3gDeviceName,port[index].mmDeviceName);
      }
	}
	printandlog(index,DM,NULL,str,0);

   INIT_DM_PORT_CONNECT_INFO_LIST(&portConnectInfoList);

   portConnectInfoList.unCount = 1;
   INIT_DM_PORT_CONNECT_INFO(&portConnectInfoList.port_connect_info[0]);


		if(port[index].xcode==true) {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_ON;
      } else {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_NATIVE;
      }

   memcpy(&(portConnectInfoList.port_connect_info[0].port_info_tx), &port[index].m3gVideoTxPortInfo, sizeof(DM_PORT_INFO));
   memcpy(&(portConnectInfoList.port_connect_info[0].port_info_rx), &port[index].mmVideoRxPortInfo, sizeof(DM_PORT_INFO));

   // This call connects the Tx of MMdevice to Rx of 3G device (Video)
	if(flag==ROUTE) {
   	if ( dev_PortConnect(port[index].m3gVideoH, &portConnectInfoList, &index)==DEV_SUCCESS ) {
      	sprintf(str, "dev_PortConnect() success m3gTxVid(%s) - mmRxVid(%s)",port[index].m3gDeviceName,port[index].mmDeviceName);
   	} else {
      	sprintf(str, "dev_PortConnect() failed for m3gTxVid(%s) - mmRxVid(%s)",port[index].m3gDeviceName,port[index].mmDeviceName);
   	}
	}
	else if(flag==UNROUTE) {
		  if ( dev_PortDisconnect(port[index].m3gVideoH, &portConnectInfoList, &index) ==DEV_SUCCESS) {
      		sprintf(str, "dev_PortDisconnect() success m3gTxVid(%s) - mmRxVid(%s)",port[index].m3gDeviceName,port[index].mmDeviceName);
   		} else {
      		sprintf(str, "dev_PortDisconnect() failed for m3gTxVid(%s) - mmRxVid(%s)",port[index].m3gDeviceName,port[index].mmDeviceName);
   		}
	}
	printandlog(index,DM,NULL,str,0);
}

/****************************************************************
*   NAME:		void routeUnrouteM3gAndM3gDevice(int index,int conType, int flag)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void routeUnrouteM3gAndM3gDevice(int index,int conType, int flag)
{
	if(port[index].bDisconnectActive==YES  && flag==ROUTE) {
      return;
  }

  char str[MAX_STRING_SIZE];
  if( (flag==UNROUTE && port[index].bM3gAndM3gConnected==M3G_FALSE) || (flag==ROUTE && port[index].bM3gAndM3gConnected==M3G_TRUE)) {
         return;
  }

  if(flag==ROUTE) {
     port[index].bM3gAndM3gConnected=M3G_TRUE;
  } else if(flag==UNROUTE) {
      port[index].bM3gAndM3gConnected=M3G_FALSE;
  }
	int partnerIndex=port[index].nPartnerIndex;


  DM_PORT_CONNECT_INFO_LIST portConnectInfoList={0};

  INIT_DM_PORT_CONNECT_INFO_LIST(&portConnectInfoList);
  portConnectInfoList.unCount = 1;

  INIT_DM_PORT_CONNECT_INFO(&portConnectInfoList.port_connect_info[0]);


		if(port[index].xcode==true) {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_ON;
      } else {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_NATIVE;
      }

  INIT_DM_PORT_INFO(&portConnectInfoList.port_connect_info[0].port_info_tx);
  INIT_DM_PORT_INFO(&portConnectInfoList.port_connect_info[0].port_info_rx);

  if(flag==ROUTE) {
      printPortInfo(index,&port[index].m3gAudioTxPortInfo,"M3G");
      printPortInfo(index,&port[index].m3gVideoTxPortInfo,"M3G");
      printPortInfo(index,&port[partnerIndex].m3gAudioRxPortInfo,"M3G");
      printPortInfo(index,&port[partnerIndex].m3gVideoRxPortInfo,"M3G");
  }

    memcpy(&(portConnectInfoList.port_connect_info[0].port_info_tx), &port[index].m3gAudioTxPortInfo, sizeof(DM_PORT_INFO));
    memcpy(&(portConnectInfoList.port_connect_info[0].port_info_rx), &port[partnerIndex].m3gAudioRxPortInfo, sizeof(DM_PORT_INFO));

    //printPortInfo(index,&portConnectInfoList->port_connect_info[0]);
    if(flag==ROUTE) {
      if( dev_PortConnect(port[index].m3gAudioH, &portConnectInfoList, &index)==DEV_SUCCESS ) {
          sprintf(str,"dev_PortConnect() success m3gTxAud(%s) -> m3gRxAud(%s)",port[index].m3gDeviceName ,port[partnerIndex].m3gDeviceName);
      } else {
          sprintf(str,"dev_PortConnect() failed m3gTxAud(%s) -> m3gRxAud(%s)",port[index].m3gDeviceName ,port[partnerIndex].m3gDeviceName);
      }
    }
    else if(flag==UNROUTE) {
      if(dev_PortDisconnect(port[index].m3gAudioH, &portConnectInfoList, &index)==DEV_SUCCESS ) {
          sprintf(str,"dev_PortDisconnect() success m3gTxAud(%s) -> m3gRxAud(%s)",port[index].m3gDeviceName ,port[partnerIndex].m3gDeviceName);
      } else {
          sprintf(str,"dev_PortDisconnect() failed m3gTxAud(%s) -> m3gRxAud(%s)",port[index].m3gDeviceName ,port[partnerIndex].m3gDeviceName);
      }
    }
    printandlog(index,DM,NULL,str,0);

    INIT_DM_PORT_CONNECT_INFO_LIST(&portConnectInfoList);

    portConnectInfoList.unCount = 1;
    INIT_DM_PORT_CONNECT_INFO(&portConnectInfoList.port_connect_info[0]);

		if(port[index].xcode==true) {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_ON;
      } else {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_NATIVE;
      }

    memcpy(&(portConnectInfoList.port_connect_info[0].port_info_tx), &port[index].m3gVideoTxPortInfo, sizeof(DM_PORT_INFO));
    memcpy(&(portConnectInfoList.port_connect_info[0].port_info_rx), &port[partnerIndex].m3gVideoRxPortInfo, sizeof(DM_PORT_INFO));

    // This call connects the Tx of M3g_A device to Rx of M3g_B device (Video)
    if(flag==ROUTE) {
      if (dev_PortConnect(port[index].m3gVideoH, &portConnectInfoList, &index)==DEV_SUCCESS ) {
        sprintf(str, "dev_PortConnect() success m3gTxVid(%s) -> m3gRxVid(%s)",port[index].m3gDeviceName ,port[partnerIndex].m3gDeviceName);
      } else {
        sprintf(str, "dev_PortConnect() failed m3gTxVid(%s) -> m3gRxVid(%s)",port[index].m3gDeviceName ,port[partnerIndex].m3gDeviceName);
      }
    }
    else if(flag==UNROUTE) {
      if(dev_PortDisconnect(port[index].m3gVideoH, &portConnectInfoList, &index)==DEV_SUCCESS ) {
         sprintf(str,"dev_PortDisconnect() success m3gTxVid(%s) -> m3gRxVid(%s)",port[index].m3gDeviceName ,port[partnerIndex].m3gDeviceName);
      } else {
         sprintf(str,"dev_PortDisconnect() failed m3gTxVid(%s) -> m3gRxVid(%s)",port[index].m3gDeviceName ,port[partnerIndex].m3gDeviceName);
      }
    }
    printandlog(index,DM,NULL,str,0);
  if(conType==HALF_DUP)
  {
    return;
  }

// m3g_b tx to m3g_a rx
if(flag==ROUTE) {
     port[partnerIndex].bM3gAndM3gConnected=M3G_TRUE;
  } else if(flag==UNROUTE) {
      port[partnerIndex].bM3gAndM3gConnected=M3G_FALSE;
  }

  if(flag==ROUTE) {
      printPortInfo(partnerIndex,&port[partnerIndex].m3gAudioTxPortInfo,"M3G");
      printPortInfo(partnerIndex,&port[partnerIndex].m3gVideoTxPortInfo,"M3G");
      printPortInfo(partnerIndex,&port[index].m3gAudioRxPortInfo,"M3G");
      printPortInfo(partnerIndex,&port[index].m3gVideoRxPortInfo,"M3G");
  }
   INIT_DM_PORT_CONNECT_INFO_LIST(&portConnectInfoList);

   portConnectInfoList.unCount = 1;
   INIT_DM_PORT_CONNECT_INFO(&portConnectInfoList.port_connect_info[0]);


		if(port[index].xcode==true) {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_ON;
      } else {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_NATIVE;
      }

   memcpy(&(portConnectInfoList.port_connect_info[0].port_info_tx), &port[partnerIndex].m3gAudioTxPortInfo, sizeof(DM_PORT_INFO));
   memcpy(&(portConnectInfoList.port_connect_info[0].port_info_rx), &port[index].m3gAudioRxPortInfo, sizeof(DM_PORT_INFO));

  if(flag==ROUTE) {
    if ( dev_PortConnect(port[partnerIndex].m3gAudioH, &portConnectInfoList, &partnerIndex)==DEV_SUCCESS ) {
        sprintf(str, "dev_PortConnect() success m3gTxAud(%s) -> m3gRxAud(%s)",port[partnerIndex].m3gDeviceName,port[index].m3gDeviceName);
    } else {
        sprintf(str, "dev_PortConnect() failed for m3gTxAud(%s) -> m3gRxAud(%s)",port[partnerIndex].m3gDeviceName,port[index].m3gDeviceName);
    }
  }
  else if(flag==UNROUTE) {
      if (dev_PortDisconnect(port[partnerIndex].m3gAudioH, &portConnectInfoList, &partnerIndex) ==DEV_SUCCESS) {
         sprintf(str, "dev_PortDisconnect() success m3gTxAud(%s) -> m3gRxAud(%s)",port[partnerIndex].m3gDeviceName,port[index].m3gDeviceName);
      } else {
         sprintf(str, "dev_PortDisconnect() failed for m3gTxAud(%s) -> m3gRxAud(%s)",port[partnerIndex].m3gDeviceName,port[index].m3gDeviceName);
      }
  }
  printandlog(partnerIndex,DM,NULL,str,0);

   INIT_DM_PORT_CONNECT_INFO_LIST(&portConnectInfoList);

   portConnectInfoList.unCount = 1;
   INIT_DM_PORT_CONNECT_INFO(&portConnectInfoList.port_connect_info[0]);


		if(port[index].xcode==true) {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_ON;
      } else {
         portConnectInfoList.port_connect_info[0].unFlags = DMFL_TRANSCODE_NATIVE;
      }

   memcpy(&(portConnectInfoList.port_connect_info[0].port_info_tx), &port[partnerIndex].m3gVideoTxPortInfo, sizeof(DM_PORT_INFO));
   memcpy(&(portConnectInfoList.port_connect_info[0].port_info_rx), &port[index].m3gVideoRxPortInfo, sizeof(DM_PORT_INFO));

   // This call connects the Tx of MMdevice to Rx of 3G device (Video)
  if(flag==ROUTE) {
    if ( dev_PortConnect(port[partnerIndex].m3gVideoH, &portConnectInfoList, &partnerIndex)==DEV_SUCCESS ) {
        sprintf(str, "dev_PortConnect() success m3gTxVid(%s) - m3gRxVid(%s)",port[partnerIndex].m3gDeviceName,port[index].m3gDeviceName);
    } else {
        sprintf(str, "dev_PortConnect() failed for m3gTxVid(%s) - m3gRxVid(%s)",port[partnerIndex].m3gDeviceName,port[index].m3gDeviceName);
    }
  }
  else if(flag==UNROUTE) {
      if ( dev_PortDisconnect(port[partnerIndex].m3gVideoH, &portConnectInfoList, &partnerIndex) ==DEV_SUCCESS) {
          sprintf(str, "dev_PortDisconnect() success m3gTxVid(%s) -> m3gRxVid(%s)",port[partnerIndex].m3gDeviceName,port[index].m3gDeviceName);
      } else {
          sprintf(str, "dev_PortDisconnect() failed for m3gTxVid(%s) -> m3gRxVid(%s)",port[partnerIndex].m3gDeviceName,port[index].m3gDeviceName);
      }
  }
  printandlog(partnerIndex,DM,NULL,str,0);
}

/****************************************************************
*   NAME:		void storeRxPortInfo(int index,METAEVENT *metaevent)
*   DESCRIPTION:
*   INPUT: index and METAEVENT
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void storeRxPortInfo(int index,METAEVENT *metaevent)
{
  	char str[MAX_STRING_SIZE];
	DM_PORT_INFO_LIST* pRxPortInfoList = (DM_PORT_INFO_LIST*)metaevent->evtdatap;
	unsigned int i;
  	if(metaevent->evtdev==port[index].m3gAudioH) {
       sprintf(str,"Storing Rx audio port info for m3g device %s",port[index].m3gDeviceName);
       memcpy(&port[index].m3gAudioRxPortInfo, &pRxPortInfoList->port_info[0], sizeof(DM_PORT_INFO));
  	}
  	else if(metaevent->evtdev==port[index].m3gVideoH) {
       sprintf(str,"Storing Rx Video port info for m3g device %s",port[index].m3gDeviceName);
       memcpy(&port[index].m3gVideoRxPortInfo, &pRxPortInfoList->port_info[0], sizeof(DM_PORT_INFO));
  	}
  	else if(metaevent->evtdev==port[index].mmH) {
		for ( i=0; i< pRxPortInfoList->unCount; i++ ) {
			switch ( pRxPortInfoList->port_info[i].port_media_type ) {
         	case DM_PORT_MEDIA_TYPE_AUDIO:
               sprintf(str,"Storing Rx audio port info for mm device %s",port[index].mmDeviceName);
               memcpy(&port[index].mmAudioRxPortInfo, &pRxPortInfoList->port_info[i], sizeof(DM_PORT_INFO));
            	break;
         	case DM_PORT_MEDIA_TYPE_VIDEO:
               sprintf(str,"Storing Rx Video port info for mm device %s",port[index].mmDeviceName);
               memcpy(&port[index].mmVideoRxPortInfo, &pRxPortInfoList->port_info[i], sizeof(DM_PORT_INFO));
            	break;
					default:
							break;
			}
		}
  	}
	printandlog(index,DM,NULL,str,0);
}
/****************************************************************
*   NAME:		void storeTxPortInfo(int index,METAEVENT *metaevent)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void storeTxPortInfo(int index,METAEVENT *metaevent)
{
  	char str[MAX_STRING_SIZE];
	DM_PORT_INFO_LIST	*pTxPortInfoList = (DM_PORT_INFO_LIST*)metaevent->evtdatap;
	unsigned int i;
  	if(metaevent->evtdev==port[index].m3gAudioH) {
       sprintf(str,"Storing Tx audio port info for m3g device %s",port[index].m3gDeviceName);
       memcpy(&port[index].m3gAudioTxPortInfo, &pTxPortInfoList->port_info[0], sizeof(DM_PORT_INFO));
  	}
  	else if(metaevent->evtdev==port[index].m3gVideoH) {
       sprintf(str,"Storing Tx Video port info for m3g device %s",port[index].m3gDeviceName);
       memcpy(&port[index].m3gVideoTxPortInfo, &pTxPortInfoList->port_info[0], sizeof(DM_PORT_INFO));
  	}
  	else if(metaevent->evtdev==port[index].mmH) {
		for ( i=0; i< pTxPortInfoList->unCount; i++) {
			switch ( pTxPortInfoList->port_info[i].port_media_type ) {
         	case DM_PORT_MEDIA_TYPE_AUDIO:
               sprintf(str,"Storing Tx audio port info for mm device %s",port[index].mmDeviceName);
               memcpy(&port[index].mmAudioTxPortInfo, &pTxPortInfoList->port_info[i], sizeof(DM_PORT_INFO));
            	break;
         	case DM_PORT_MEDIA_TYPE_VIDEO:
               sprintf(str,"Storing Tx Video port info for mm device %s",port[index].mmDeviceName);
               memcpy(&port[index].mmVideoTxPortInfo, &pTxPortInfoList->port_info[i], sizeof(DM_PORT_INFO));
            	break;
					default:
							break;
			}
		}
  	}
	printandlog(index,DM,NULL,str,0);
}
/****************************************************************
*   NAME:		void getTxRxPortInfo(int index,int handle)
*   DESCRIPTION:
*   INPUT: index and device handle
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/

static void getTxRxPortInfo(int index,int handle)
{
  char str[MAX_STRING_SIZE];
   // retrieve Tx and Rx PStream ports:
   if (dev_GetTransmitPortInfo(handle, &index)!=DEV_SUCCESS ) {
		sprintf(str,"dev_GetTransmitPortInfo(%d) failure",handle);
   } else {
		sprintf(str,"dev_GetTransmitPortInfo(%d) successful",handle);
   }
	printandlog(index,DM,NULL,str,0);

   if (dev_GetReceivePortInfo(handle, &index)!=DEV_SUCCESS ) {
		sprintf(str,"dev_GetReceivePortInfo(%d) failure",handle);
   } else {
		sprintf(str,"dev_GetReceivePortInfo(%d) successful",handle);
   }
	printandlog(index,DM,NULL,str,0);
}
/****************************************************************
*   NAME:		void connectM3gAndDti(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void connectM3gAndDti(int index)
{
  char str[MAX_STRING_SIZE];
	if(port[index].bDisconnectActive==YES) {
      return;
  }
   
	if(port[index].bM3gAndDtiConnected==M3G_TRUE) {
		return;
	}

  port[index].bM3gAndDtiConnected=M3G_TRUE;

  if (dev_Connect(port[index].m3gControlH, port[index].networkH, DM_FULLDUP, EV_ASYNC)!=DEV_SUCCESS ) {
  		sprintf(str,"dev_Connect() failed for 3G-DTI");
  } else {
     	sprintf(str,"dev_Connect() successful for 3G-DTI");
  }
	printandlog(index,DM,NULL,str,0);
}
/****************************************************************
*   NAME:		void disconnectM3gAndDti(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void disconnectM3gAndDti(int index)
{
  char str[MAX_STRING_SIZE];

  if(port[index].bM3gAndDtiConnected==M3G_FALSE) {
         return;
  }
  port[index].bM3gAndDtiConnected=M3G_FALSE;

  if (dev_Disconnect(port[index].m3gControlH, EV_ASYNC)!=DEV_SUCCESS ) {
      sprintf(str,"dev_Disconnect() failed for 3G");
  } else {
      sprintf(str,"dev_Disconnect() successful for 3G");
  }
  printandlog(index,DM,NULL,str,0);

  if (dev_Disconnect(port[index].networkH, EV_ASYNC)!=DEV_SUCCESS ) {
      sprintf(str,"dev_Disconnect() failed for DTI");
  } else {
      sprintf(str,"dev_Disconnect() successful for DTI");
  }
  printandlog(index,DM,NULL,str,0);
}
/****************************************************************
*   NAME:		void getLocalCapability(int index,int handle)
*   DESCRIPTION:
*   INPUT: index and handle
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void getLocalCapability(int index,int handle)
{
	char str[MAX_STRING_SIZE];

   if (m3g_GetLocalCaps(handle)!=M3G_SUCCESS) {
      sprintf(str,"m3g_GetLocalCaps() failure handle=%d",handle);
   } else {
      sprintf(str,"m3g_GetLocalCaps() successful Handle=%d",handle);
   }
	printandlog(index,M3G,NULL,str,0);
}
/****************************************************************
*   NAME:		void printLocalCapability(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void printLocalCapability(int index)
{
	char str[MAX_STRING_SIZE];
	int ctr;

	sprintf(str,"Channel Index# %d",index);
	printandlog(index,M3G,NULL,str,0);
	
	sprintf(str,"H223 Cap [ AL=%u | AL2Size=%u ]",
							port[index].simultaneousCapabilityList.pH223Capabilities->capability[0].h223Capability.adaptationLayerMedia,
							port[index].simultaneousCapabilityList.pH223Capabilities->capability[0].h223Capability.maxAL2SDUSize);
	printandlog(index,M3G,NULL,str,0);

	for(ctr=0;ctr<port[index].simultaneousCapabilityList.pAudioCapabilities->numCaps;ctr++) {
		sprintf(str,"Audio Cap#%u [coderType=%s|maxFramesPerSDU=%d|direction=%d]",ctr,
							strCoderType[port[index].simultaneousCapabilityList.pAudioCapabilities->capability[ctr].audioCapability.coderType],
							port[index].simultaneousCapabilityList.pAudioCapabilities->capability[ctr].audioCapability.maxFramesPerSDU,
							port[index].simultaneousCapabilityList.pAudioCapabilities->capability[ctr].audioCapability.direction);
		printandlog(index,M3G,NULL,str,0);
	}
	for(ctr=0;ctr<port[index].simultaneousCapabilityList.pVideoCapabilities->numCaps;ctr++) {
		sprintf(str,"Video Cap#%u [coderType=%s|maxBitRate=%d|direction=%d]",ctr,
							strCoderType[port[index].simultaneousCapabilityList.pVideoCapabilities->capability[ctr].videoCapability.coderType+2],
							port[index].simultaneousCapabilityList.pVideoCapabilities->capability[ctr].videoCapability.maxBitRate,
							port[index].simultaneousCapabilityList.pVideoCapabilities->capability[ctr].videoCapability.direction);	
		printandlog(index,M3G,NULL,str,0);
	}
}
/****************************************************************
*   NAME:			void setLocalCapability(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void setLocalCapability(int index)
{
	char str[MAX_STRING_SIZE];
	printLocalCapability(index);		// SKS print before setting it
   if(m3g_SetTCS(port[index].m3gControlH, 1, &port[index].simultaneousCapabilityList)==M3G_SUCCESS) {
      	sprintf(str,"Successful m3g_SetTCS() completion - %d",port[index].m3gControlH);
   } else {
      	sprintf(str,"m3g_SetTCS() failure - %d",port[index].m3gControlH);
   }
	printandlog(index,M3G,NULL,str,0);
}
/****************************************************************
*   NAME:		void storeLocalCapability(int index,int handle,M3G_CAPS_LIST *pDefaultCapsList)
*   DESCRIPTION:
*   INPUT: index, device handle and pointer to M3G_CAPS_LIST
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void storeLocalCapability(int index,int handle,M3G_CAPS_LIST *pDefaultCapsList)
{
	char str[MAX_STRING_SIZE];
	int ctr;

	if(port[index].m3gControlH==handle) {

		M3G_CAPS_LIST  preferredH223Caps = {0};

		preferredH223Caps.version=M3G_LIBRARY_VERSION;
		preferredH223Caps.numCaps=1;
		preferredH223Caps.capabilityType=M3G_E_H223_CAPABILITY;
				
		preferredH223Caps.capability[0].h223Capability=pDefaultCapsList->capability[0].h223Capability;

		preferredH223Caps.capability[0].h223Capability.version=M3G_LIBRARY_VERSION;
   	preferredH223Caps.capability[0].h223Capability.adaptationLayerMedia = M3G_AUDIO_AL2|M3G_VIDEO_AL2;
   	preferredH223Caps.capability[0].h223Capability.maxAL2SDUSize = 512;
   	preferredH223Caps.capability[0].h223Capability.maxAL3SDUSize = 512; 
   	preferredH223Caps.capability[0].h223Capability.frameH223AnnexA = M3G_FALSE; 
   	preferredH223Caps.capability[0].h223Capability.frameH223DoubleFlag = M3G_FALSE;
   	preferredH223Caps.capability[0].h223Capability.frameAnnexB = M3G_TRUE;
   	preferredH223Caps.capability[0].h223Capability.frameAnnexBWithHead = M3G_FALSE; 
   	preferredH223Caps.capability[0].h223Capability.rsCodeCapability = M3G_FALSE;
   	preferredH223Caps.capability[0].h223Capability.mobileOpXmitCap = M3G_TRUE; 
   	preferredH223Caps.capability[0].h223Capability.bitRate = 450;

   	port[index].simultaneousCapabilityList.pH223Capabilities = &preferredH223Caps;

		sprintf(str,"H223 Cap [ AL=%u | AL2Size=%u ]",
							port[index].simultaneousCapabilityList.pH223Capabilities->capability[0].h223Capability.adaptationLayerMedia,
							port[index].simultaneousCapabilityList.pH223Capabilities->capability[0].h223Capability.maxAL2SDUSize);
		printandlog(index,M3G,NULL,str,0);
	}
	else if(port[index].m3gAudioH==handle) {

   	M3G_CAPS_LIST    preferredAudioCaps = {0};
   	preferredAudioCaps.version = M3G_LIBRARY_VERSION;
   	preferredAudioCaps.numCaps = 0;
   	preferredAudioCaps.capabilityType = M3G_E_AUDIO_CAPABILITY;

   	for (ctr=0; ctr < pDefaultCapsList->numCaps; ctr++ ) {
      	if (pDefaultCapsList->capability[ctr].audioCapability.coderType==M3G_E_GSM_AMR_NB) {
				sprintf(str,"coderType=%s",strCoderType[pDefaultCapsList->capability[ctr].audioCapability.coderType]);
				printandlog(index,M3G,NULL,str,0);
            preferredAudioCaps.capability[preferredAudioCaps.numCaps].audioCapability=pDefaultCapsList->capability[ctr].audioCapability;
				preferredAudioCaps.capability[preferredAudioCaps.numCaps].audioCapability.maxFramesPerSDU=1;
            preferredAudioCaps.numCaps++;
      	}
   	}
   	port[index].simultaneousCapabilityList.pAudioCapabilities = &preferredAudioCaps;

	} 
	else if(port[index].m3gVideoH==handle) {
   	// configure the local video caps per the configured preferences:
   	M3G_CAPS_LIST    preferredVideoCaps = {0};
   	preferredVideoCaps.version = M3G_LIBRARY_VERSION;
   	preferredVideoCaps.numCaps = 0;
   	preferredVideoCaps.capabilityType = M3G_E_VIDEO_CAPABILITY;

  		for (ctr=0; ctr < pDefaultCapsList->numCaps; ctr++ ) {
         if (pDefaultCapsList->capability[ctr].videoCapability.coderType==port[index].cfgCoderType ) {
				sprintf(str,"coderType=%s",strCoderType[pDefaultCapsList->capability[ctr].videoCapability.coderType+2]);
				printandlog(index,M3G,NULL,str,0);
            preferredVideoCaps.capability[preferredVideoCaps.numCaps].videoCapability=pDefaultCapsList->capability[ctr].videoCapability;
				preferredVideoCaps.capability[preferredVideoCaps.numCaps].videoCapability.maxBitRate=430;
            preferredVideoCaps.numCaps++;
         }
   	}
   	port[index].simultaneousCapabilityList.pVideoCapabilities = &preferredVideoCaps;
	}
//	printandlog(index,M3G,NULL,str,0);
}

/****************************************************************
*   NAME:			void m3gSetParameter(int index,int handle)
*   DESCRIPTION:
*   INPUT: index and device handle
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void m3gSetParameter(int index,int handle)
{
	char str[MAX_STRING_SIZE];
	M3G_PARM_INFO parmInfo = {0};
	INIT_M3G_PARM_INFO(&parmInfo);

	parmInfo.parameterType = M3G_E_PRM_H245_TERMINAL_TYPE;
	parmInfo.parmValue.h245TerminalType = 127;
	if (m3g_SetParm(handle, &parmInfo) !=M3G_SUCCESS) {
         sprintf(str,"m3g_SetParm() failure on handle=%d",handle);
   } else {
         sprintf(str,"m3g_SetParm() successful on handle=%d",handle);
   }
	printandlog(index,M3G,NULL,str,0);

   parmInfo.parameterType = M3G_E_PRM_RELAY_DIGIT_TO_MEDIA_DEV;
   parmInfo.parmValue.boolParam= M3G_TRUE;
   if (m3g_SetParm(handle, &parmInfo) !=M3G_SUCCESS) {
         sprintf(str,"m3g_SetParm() failure on handle=%d",handle);
   } else {
         sprintf(str,"m3g_SetParm() successful on handle=%d",handle);
   }
	printandlog(index,M3G,NULL,str,0);

   parmInfo.parameterType = M3G_E_PRM_RELAY_FASTUPDATE_TO_H245;
   parmInfo.parmValue.boolParam =M3G_TRUE;
   if (m3g_SetParm(handle, &parmInfo)!=M3G_SUCCESS ) {
         sprintf(str,"m3g_SetParm() failure on handle=%d",handle);
   } else {
         sprintf(str,"m3g_SetParm() successful on handle=%d",handle);
   }
	printandlog(index,M3G,NULL,str,0);

 	parmInfo.parameterType = M3G_E_PRM_EARLY_MES;
   parmInfo.parmValue.boolParam= M3G_TRUE;
   if (m3g_SetParm(handle, &parmInfo) !=M3G_SUCCESS) {
         sprintf(str,"m3g_SetParm() failure on handle=%d",handle);
   } else {
         sprintf(str,"m3g_SetParm() successful on handle=%d",handle);
   }

	printandlog(index,M3G,NULL,str,0);
}
/****************************************************************
*   NAME:		void stopM3g(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void stopM3g(int index)
{
	if (m3g_Stop()!=M3G_SUCCESS) {
      printandlog(index,M3G,NULL,"m3g_Stop() failure",0);
   } else {
      printandlog(index,M3G,NULL,"m3g_Stop() Success",0);
	}
}
/****************************************************************
*   NAME:		int InitM3g(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: -1 when fails otherwise 0
*   CAUTIONS: NA
****************************************************************/
static int InitM3g(int index)
{
	char  str[MAX_STRING_SIZE];
  int nSRLMode = SR_POLLMODE;

	m3gBoardParmCtr=0;

  if ( sr_setparm(SRL_DEVICE, SR_MODEID, &nSRLMode) == -1 ) {
      printandlog(index,M3G,NULL,"Error sr_setparm(SRL_DEVICE, SR_MODEID, POLL) failure",0);
		exit(-1);
  }

  M3G_START_STRUCT startParams;
  INIT_M3G_START_STRUCT(&startParams);
  startParams.numEndpoints = num_devices;

  if (m3g_Start(&startParams)!=M3G_SUCCESS) {
      printandlog(index,M3G,NULL,"m3g_Start() failure",0);
      return -1;
  }
	sprintf(str,"m3g_Start() successful for %d devices",num_devices);
  printandlog(index,M3G,NULL,str,0);

  // start and allocate resources for the M3G library session:
  // Initialize the specified board defaults:
  	m3gBoardH = m3g_OpenEx("m3gB1", NULL, NULL, EV_SYNC);
  	sr_setparm(m3gBoardH, SR_USERCONTEXT, (void *)&index);
  	if ( m3gBoardH < 0 ) {
   	printandlog(index,M3G,NULL,"m3g_OpenEx(m3gB1) failed",0);
		return -1;
  	} else {
		sprintf(str,"Successful m3g_OpenEx(m3gB1) completion Handle=%d",m3gBoardH);
  		printandlog(index,M3G,NULL,str,0);
	}
   resetM3gDevice(BOARD_DEVICE,m3gBoardH);
	return 0;
}
/****************************************************************
*   NAME:		void open_mm_channel(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void open_mm_channel(int index)
{
	char str[MAX_STRING_SIZE];
	int ch=index+1;

	sprintf(port[index].mmDeviceName,"mmB1C%d",ch);
	if ((port[index].mmH = mm_Open(port[index].mmDeviceName, NULL,(void *) &port[index])) < 0) {
    	sprintf(str,"mm_Open(devicename=%s) Failed", port[index].mmDeviceName);
  	} else {
    	sprintf(str,"mm_Open(devicename=%s) successful", port[index].mmDeviceName);
  	}
   printandlog(index, MM, NULL, str,0);
   sr_setparm(port[index].mmH, SR_USERCONTEXT, (void *)&port[index].index);
}
/****************************************************************
*   NAME:		void open_m3g_channel(int index)
*   DESCRIPTION:
*   INPUT: index
*   RETURNS: None
*   CAUTIONS: NA
****************************************************************/
static void open_m3g_channel(int index)
{
	char	str[MAX_STRING_SIZE];
	int controlH,audioH,videoH;
	int ts=index+1;
	sprintf( port[index].m3gDeviceName,"m3gB1T%d",ts);

	
// SKS_SIMUL
	memset(&port[index].simultaneousCapabilityList, 0, sizeof(M3G_SIMULTANEOUS_CAP_SET));

   port[index].simultaneousCapabilityList.version = M3G_LIBRARY_VERSION;

	controlH = m3g_OpenEx(port[index].m3gDeviceName, NULL, (void *)&port[index],EV_ASYNC);
   if (controlH<0) {
		sprintf(str,"m3g_OpenEx(%s) failure", port[index].m3gDeviceName);
      printandlog(index, M3G, NULL, str,0);
   } else {
		sprintf(str,"Successful m3g_OpenEx(%s) completion Handle=%d", port[index].m3gDeviceName,controlH);
      printandlog(index,M3G,NULL,str,0);
		port[index].m3gControlH=controlH;
   }
   sr_setparm(port[index].m3gControlH, SR_USERCONTEXT, (void *)&port[index].index);
   char mediaName[30];
   sprintf(mediaName, "%s:AUDIO1", port[index].m3gDeviceName);
   audioH = m3g_OpenEx(mediaName, NULL,(void *) &port[index],EV_ASYNC);
   if ( audioH<0 ) {
		sprintf(str,"m3g_OpenEx(%s) failure",mediaName);
      printandlog(index, M3G, NULL, str,0);
   } else {
		sprintf(str,"Successful m3g_OpenEx(%s) completion Handle=%d",mediaName,audioH);
      printandlog(index,M3G,NULL,str,0);
		port[index].m3gAudioH= audioH;
   }
   sr_setparm(port[index].m3gAudioH, SR_USERCONTEXT, (void *)&port[index].index);
   sprintf(mediaName, "%s:VIDEO1", port[index].m3gDeviceName);
   videoH = m3g_OpenEx(mediaName, NULL,(void *) &port[index],EV_ASYNC);
   if ( videoH<0 ) {
		sprintf(str,"m3g_OpenEx(%s) failure",mediaName);
      printandlog(index, M3G, NULL, str,0);
   } else {
		sprintf(str,"Successful m3g_OpenEx(%s) completion Handle=%d",mediaName,videoH);
      printandlog(index,M3G,NULL,str,0);
		port[index].m3gVideoH=videoH;
   }
   sr_setparm(port[index].m3gVideoH, SR_USERCONTEXT, (void *)&port[index].index);
}

/****************************************************************
*			NAME: open_isdn_channel(index)
* DESCRIPTION: initialize a port used by an ISDN protocol
*		  INPUT: index into port[] for this channel
*		RETURNS: exits with exitdemo() if not successful
*	  CAUTIONS: 1. This will need to be modified for some configurations/environments
*					2. Many parameters are "commented" out. The purpose of this is to 
*					   give you a template of how to change default parameters if you need
*						to do so
*					3. Don't be intimidated by the size of this procedure.
*						As previously indicated, much of this procedure has
*						commented out code which is intended to give examples
*						for those who need it. Many developers will not.
****************************************************************/
static void open_isdn_channel(int index)
{
	char						str[MAX_STRING_SIZE];
	GCLIB_MAKECALL_BLK	*gclib_makecallp;
	GC_PARM_BLKP			parmblkp = NULL;


	printandlog(index, MISC, NULL, "ISDN device being opened", 0);

	/* Initial channel state is blocked */
	port[index].blocked = YES; 
	port[index].techtype = ISDN;
	port[index].makecall_timeout = 0;							/* timeout parameter is ignored for ISDN */

	/* initializes the GC_PARM_BLK with the respective set_id, parm_id and the values */

	/*******
	-- To set the bearer channel transfer mode information
	-- Commented out as this is the default. Change if you wish to use
	-- other than the default value
	--	gc_util_insert_parm_val(&parmblkp, GCIS_SET_BEARERCHNL, GCIS_PARM_TRANSFERMODE, 
	--		sizeof(int), ISDN_ITM_CIRCUIT);
	*******/

	/*******
	-- To set the bearer channel transfer rate information
	-- Commented out as this is the default. Change if you wish to use 
	-- other than the default value
	--	gc_util_insert_parm_val(&parmblkp, GCIS_SET_BEARERCHNL, GCIS_PARM_TRANSFERRATE, 
	--		sizeof(int), BEAR_RATE_64KBPS);
	*******/

	/*******
	-- To set Layer 1 protocol to use on bearer channel
	-- Commented out as this is the default. Change if you wish to use
	-- other than the default value
	--	gc_util_insert_parm_val(&parmblkp, GCSET_CHAN_CAPABILITY, GCPARM_TYPE, 
	--		sizeof(int), GCCAPTYPE_AUDIO);
	*******/

	/*******
	-- To set Layer 1 protocol to use on bearer channel
	-- The default is not good for all our E1 boards, for example Springware
	-- WARNING: this will need to be changed for T1. Commenting out this
	-- function call will work in many T1 cases
	*******/
	gc_util_insert_parm_val(&parmblkp, GCSET_CHAN_CAPABILITY, GCPARM_CAPABILITY, 
		sizeof(int), GCCAP_AUDIO_g711Alaw64k);

	/*******
	-- To set User rate to use on bearer channel (layer 1 rate)
	--  Commented out as this is the default. Change if you wish to use
	-- other than the default value
	--	gc_util_insert_parm_val(&parmblkp, GCSET_CHAN_CAPABILITY, GCPARM_RATE, 
	--		sizeof(int), GCCAPRATE_DEFAULT);
	*******/

	/*******
	-- To support the specific feature or service requested from the network
	-- Commented out as this is not supported by E1 and DM3.
	--	gc_util_insert_parm_val(&parmblkp, GCIS_SET_FACILITY, GCIS_PARM_FACILITY_CODINGVALUE, 
	--		sizeof(int), 0x03);
	*******/

	/*******
	-- To support the specific feature or service requested from the network
	-- Commented out as this is not supported by E1 and DM3.
	--	gc_util_insert_parm_val(&parmblkp, GCIS_SET_FACILITY, GCIS_PARM_FACILITY_FEATURESERVICE, 
	--		sizeof(int), ISDN_SERVICE);
	*******/

	/*******
	-- Note: gclib_makecallp is not freed since a "permanent" data
	-- structure. Assume program exit will free the resource
	*******/
	gclib_makecallp
		= (GCLIB_MAKECALL_BLKP) malloc(sizeof(GCLIB_MAKECALL_BLK) + parmblkp->parm_data_size);
	if (gclib_makecallp == NULL) {
		printandlog(ALL_DEVICES, MISC, NULL, "Could not allocate memory\n", 0);
		exitdemo(1);
	}
	port[index].makecallblk.cclib = NULL;
	port[index].makecallblk.gclib = gclib_makecallp;

	/* Set the extra data in the makecall block */
	gclib_makecallp->ext_datap = parmblkp;

	/*******
	-- Set the called number plan to unknown 
	-- Modified, as this causes issues on some switches
	-- - UNKNOWN works on 99% of E1 switches
	--	gclib_makecallp->destination.address_plan = GCADDRPLAN_ISDN;
	*******/
	gclib_makecallp->destination.address_plan = GCADDRPLAN_UNKNOWN;

	/*******
	-- Set the address type to transparent
	-- Modified, as this causes issues on some switches 
	-- - TRANSPARENT works on 99% of E1 switches
	--	gclib_makecallp->destination.address_type = GCADDRTYPE_NAT;
	*******/
	gclib_makecallp->destination.address_type = GCADDRTYPE_TRANSPARENT;

	/*******
	-- Removed sub-addresses, as this causes issues on some switches
	--	strcpy(gclib_makecallp->destination.sub_address, "456");
	--	gclib_makecallp->destination.sub_address_plan = 0;
	--	gclib_makecallp->destination.sub_address_type = GCSUBADDR_OSI;
	*******/
	gclib_makecallp->destination.sub_address[0] = '\x0';
	gclib_makecallp->destination.sub_address_plan = 0;
	gclib_makecallp->destination.sub_address_type = 0;

	/*******
	-- Note to customers: make sure that the Calling Address is set correctly, as some switches
	-- could reject the call if the calling address is not correct.
	*******/
	strcpy(gclib_makecallp->origination.address, "7654321");

	/*******
	-- Set the calling number plan to ISDN type
	-- Modified, as this causes issues on some switches
	-- gclib_makecallp->origination.address_plan = GCADDRPLAN_ISDN;
	*******/
	gclib_makecallp->origination.address_plan = GCADDRPLAN_UNKNOWN;

	/*******
   -- Set the address type
	-- Modified, as this causes issues on some switches
	-- gclib_makecallp->origination.address_type = GCADDRTYPE_NAT;
	*******/
	gclib_makecallp->origination.address_type = GCADDRTYPE_TRANSPARENT;

	/*******
	-- Removed sub-addresses, as this causes issues on some switches
	--	strcpy(gclib_makecallp->origination.sub_address, "456");
	-- gclib_makecallp->origination.sub_address_plan = 0;
	-- gclib_makecallp->origination.sub_address_type = GCSUBADDR_OSI;
	*******/
	gclib_makecallp->origination.sub_address[0] = '\x0';
	gclib_makecallp->origination.sub_address_plan = 0;
	gclib_makecallp->origination.sub_address_type = 0;

	gclib_makecallp->call_info.address_info = GCADDRINFO_ENBLOC;
	
	/* Open the line device */
	if (gc_OpenEx(&port[index].ldev, port[index].devname, EV_SYNC, (void *)&port[index]) != GC_SUCCESS) {
		sprintf(str, "gc_OpenEx(devicename=%s, mode=EV_SYNC) Failed", port[index].devname);
		printandlog(index, GC_APIERR, NULL, str, 0);
		exitdemo(1);
	}
	sprintf(str, "gc_OpenEx(devicename=%s, mode=EV_SYNC) Success", port[index].devname);
	printandlog(index, GC_APICALL, NULL, str, 0);

// SKS added on 14th September 2009
//	sr_setparm(port[index].ldev, SR_USERCONTEXT, (void *)&index);

	/* Due to implementation, must wait until get GCEV_UNBLOCKED before issueing the reset linedevice */
	port[index].resetlinedev_required_after_unblocked = YES;

	/* Enable alarm notification */
	enable_alarm_notification(&port[index]);
} /* End of Function */

/****************************************************************
*			NAME: open_ss7_channel(index)
* DESCRIPTION: initialize a port used by an SS7 protocol
*		  INPUT: index into port[] for this channel
*		RETURNS: exits with exitdemo if not successful
*	  CAUTIONS: none
****************************************************************/
static void open_ss7_channel(int index)
{
	char		str[MAX_STRING_SIZE];
	char		temp_devname[MAX_DEVNAME];
	printandlog(index, MISC, NULL, "SS7 device being opened", 0);

	/* Initial channel state is blocked */
	port[index].blocked = YES; 
	port[index].techtype = SS7;
	port[index].makecall_timeout = 60;							/* timeout is supported, try 1 minute */
	
	/* 
	NOTE:
	The SS7 MakeCallBlock is not initialized currently, as Gc_MakeCallBlk->cclib is set to 
	Null to use default values. For Live Testing, initialize the SS7MakeCallBlock and set 
	its address to Gc_MakeCallBlk->cclib field.
	*/

   sprintf(temp_devname,":L_SS7%s", port[index].devname);
	if (gc_OpenEx(&port[index].ldev, temp_devname, EV_SYNC, (void *)&port[index]) != GC_SUCCESS) {
		sprintf(str, "gc_OpenEx(devicename=%s, mode=EV_SYNC) Failed", temp_devname);
		printandlog(index, GC_APIERR, NULL, str, 0);
		exitdemo(1);
	}
	
	sprintf(str, "gc_OpenEx(devicename=%s, mode=EV_SYNC) Success", temp_devname);
	printandlog(index, GC_APICALL, NULL, str, 0);

// SKS added on 14th September 2009
//	sr_setparm(port[index].ldev, SR_USERCONTEXT, (void *)&index);

	GC_PARM gcParm;
	gcParm.longvalue=8;
	if(gc_SetParm(port[index].ldev,GCPR_RECEIVE_INFO_BUF,gcParm)==GC_SUCCESS) {
			printandlog(index, GC_APICALL, NULL, "gc_SetParm(GCPR_RECEIVE_INFO_BUF) to 8 successful", 0);
	}

} /* End of function */ 



/***************************************************************************
 *			NAME: void intr_hdlr()
 *DESCRIPTION: Handler called when one of the following signals is
 *					received: SIGHUP, SIGINT, SIGQUIT, SIGTERM.
 *					This function stops I/O activity on all channels and
 *					closes all the channels.
 *		  INPUT: None
 *		RETURNS: None.
  ***************************************************************************/
static void intr_hdlr(void)
{
	if (interrupted == NO) {
		printandlog(ALL_DEVICES, MISC, NULL, " *******Received User Interrupted Signal *******\n", 0);

		interrupted = YES;
	}
	/* will handle graceful shutdown outside this handler in main's forever loop */
}

//*****************************************************************************
// Function: int ::_kbhit()
// Description: Detect keyboard hit
// Return: int
// Parameters: none
//*****************************************************************************
int _kbhit()
{
   static const int STDIN = 0;
   static bool initialized = false;

   if ( !initialized )
   {
      // Use termios to turn off line buffering
      struct termios term;
      tcgetattr(STDIN, &term);
      term.c_lflag &= ~ICANON;
      tcsetattr(STDIN, TCSANOW, &term);
      setbuf(stdin, NULL);
      initialized = true;
   }

   int bytesWaiting;
   ioctl(STDIN, FIONREAD, &bytesWaiting);
   return bytesWaiting;
}

static void getDxDeviceName(int index)
{
   static int brd=1;
   static int ch=0;
   int tmpIndex=index+1;
   if(tmpIndex%5==0)
         brd++;
   ch++;
   if(ch>4)
      ch=1;
   sprintf(port[index].dxDeviceName,"dxxxB%dC%d",brd,ch);
}


/********************************************************************************
 *			NAME: load_config_file()
 *DESCRIPTION: This function reads the configuration file line
 *					by line. Ignores commented lines. For valid lines, 
 *					it fills the devname field of corresponding element 
 *					of port (global array of struct channel) as it is
 *					to be passed as an argument to gc_OpenEx(). It also
 *					fills the direction field in port[index]. 
 * 
 *		  INPUT: None
 *		RETURNS: None - exits if error
 *	  CAUTIONS: Assumes config file is in a "reasonable" format
 *					Not all fields are case insensitive
 *					The parsing in this procedure is simplified for demo purposes,
 *					i.e. error handling is weak to non-existent.
 *					Your application will need to implement parsing that is appropriate 
 *					for your application
 ********************************************************************************/
static void load_config_file(void)
{
	static	char	line[MAX_STRING_SIZE];
	static	char	netdevice[MAX_DEVNAME], outnetdevice[MAX_DEVNAME],protname[MAX_DEVNAME];
	static	char	devname[MAX_DEVNAME], outdevname[MAX_DEVNAME], destination_num[MAXPHONENUM];
	static	char	mrbtClipName[MAX_DEVNAME], invidcodec[MAX_DEVNAME],outvidcodec[MAX_DEVNAME] ,transcode[MAX_DEVNAME];
	int brd, ts;
	int		index, num_params;
	FILE		*cfg_fp;
	
	/* Open Configuration file in Read mode */
	cfg_fp = fopen(cfgfile, "r");
	
	if (cfg_fp == NULL) {
		printf("Unable to open %s for read\n", cfgfile);
		perror("Reason is: ");
		exit(1);
	}

	while (fgets(line, MAX_STRING_SIZE, cfg_fp)) {
		/* strip trailing white space */
		index = strlen(line) - 1;
		while (index && (line[index] <= ' ')) {
			line[index] = '\0';
			index--;
		}

		/* ignore blank lines */
		if (index == 0) {
			continue;
		}

		/* if 1st character is not alpha numeric, then treat the line as a comment & ignore it */
		if ((line[0] < 'A') || (line[0] > 'z')) {	/* simplifed non-alpha check */
			continue; /* Ignore the line and read next */
		}
		num_devices++; /* Increment Global Variable */

		if(num_devices > MAXCHAN)
		{
			printf(" [ERROR]:		 Number of devices in cfg file exceeded maximum. Increase MAXCHAN and recompile\n");
			num_devices--;
			fclose (cfg_fp);
			exitdemo(1);
		}

		/* Parse DTI Device, Protocol Name, Direction and Voice Device Name */ 
		num_params = sscanf(line, "%s%s%s%s%s%s%s%s",
			netdevice, 
			invidcodec,
			outnetdevice,
			outvidcodec, 
			protname, 
			destination_num,
			mrbtClipName,
			transcode);

//dtiB1T1     h263     dtiB1T16    mpeg4       ISDN     96546147       clip1       xcode


			if ((netdevice[0] == '\0') || (outnetdevice[0] == '\0') || (protname[0] == '\0') ) {
				/* Invalid entry in the configuration file */
				fclose (cfg_fp);
				printf("Invalid entry in %s\n", cfgfile);
				printf("Line is: %s\n", line);
				exitdemo(1);
			} else {
				brd=ts=-1;
				sscanf(netdevice,"dtiB%dT%d",&brd,&ts);
				if( (brd<1 || brd>8) || (ts<1 || ts>30)) {
					printf("Inavlid Incoming Device Name=%s\n",netdevice);
					fclose (cfg_fp);
					exitdemo(1);
				}
			}
			if( (strcasecmp(destination_num,"NONE")==0 || strlen(destination_num)<8) ) {
					printf("destination_num must not be NONE and must be at least >=8 digits\n");
					exitdemo(1);
			}
		 	
			if(strlen(mrbtClipName)==0) {
				strcpy(mrbtClipName,"clip1");
			}

			if(strlen(transcode)==0) {
				strcpy(transcode,"native");
      	}

		/* Fill devname to be passed as arg in gc_OpenEx() 
		in port array (global) of struct channel */
		
		if (strcmp(netdevice, "NONE") != 0) {
			sprintf(devname, ":N_%s:P_%s", netdevice, protname);
		} else {
			sprintf(devname, ":P_%s", protname);
		}

		getDxDeviceName(num_devices-1);
		
		port[num_devices - 1].direction = DIR_IN;

		/* Fill device index in port array (global) of struct channel */
		port[num_devices - 1].index = num_devices - 1;

		/* Copying all the local variable values to Global Variables */
		strcpy(port[num_devices - 1].devname, devname);
		strcpy(port[num_devices - 1].netdevice, netdevice);
		strcpy(port[num_devices - 1].protname, protname);
		strcpy(port[num_devices - 1].destination_num, destination_num);


		if(strcasecmp(invidcodec,"h263")==0) {
			port[num_devices - 1].cfgCoderType=M3G_E_H263;
		} else if (strcasecmp(invidcodec,"mpeg4")==0) {
			port[num_devices - 1].cfgCoderType=M3G_E_MPEG4;
    	} else {
			printf("ERROR: Invalid Video Codec specified for incoming channel\n Exiting ...........\n");
			exit(-1);
		}
		
		if(strcasecmp(transcode,"native")==0) {
			port[num_devices - 1].xcode=false;
		} else if(strcasecmp(transcode,"xcode")==0) {
         port[num_devices - 1].xcode=true;
      } 
	
 		sprintf(port[num_devices - 1].audioFileName,"avfiles/%s.amr", mrbtClipName);
    	sprintf(port[num_devices - 1].videoFileName,"avfiles/%s_%s.vid", mrbtClipName,invidcodec);


// SKS for MRBT
    /* Fill devname to be passed as arg in gc_OpenEx()
    in port array (global) of struct channel  for outgoing leg*/
    if ( strcmp(outnetdevice, "NONE") != 0 ) {

				// partner index is outgoing channel index for incoming leg
				port[num_devices - 1].nPartnerIndex=num_devices;
				num_devices++; /* Increment Global Variable */

        brd=ts=-1;
        sscanf(outnetdevice,"dtiB%dT%d",&brd,&ts);
        if( (brd<1 || brd>8) || (ts<1 || ts>30)) {
					printf("Inavlid Outgoing network Device Name=%s\n",outnetdevice);
          fclose (cfg_fp);
          exitdemo(1);
        }
      	sprintf(outdevname, ":N_%s:P_%s", outnetdevice, protname);

    		/* Fill Direction in port array (global) of struct channel */
      	port[num_devices - 1].direction = DIR_OUT;

    		/* Fill device index in port array (global) of struct channel */
    		port[num_devices - 1].index = num_devices - 1;
    		/* Copying all the local variable values to Global Variables */
    		strcpy(port[num_devices - 1].devname, outdevname);
    		strcpy(port[num_devices - 1].netdevice, outnetdevice);
    		strcpy(port[num_devices - 1].protname, protname);
    		strcpy(port[num_devices - 1].destination_num, destination_num);

    		if(strcasecmp(outvidcodec,"h263")==0) {
         	port[num_devices - 1].cfgCoderType=M3G_E_H263;
      	} else if (strcasecmp(outvidcodec,"mpeg4")==0) {
         	port[num_devices - 1].cfgCoderType=M3G_E_MPEG4;
      	} else {
         	printf("ERROR: Invalid Video Codec specified for incoming channel\n Exiting ...........\n");
         	exit(-1);
      	}

			if(strcasecmp(transcode,"native")==0) {
				port[num_devices - 1].xcode=false;
			} else if(strcasecmp(transcode,"xcode")==0) {
         	port[num_devices - 1].xcode=true;
      	} 

			// partner index is incoming channel index for outgoing leg
			port[num_devices - 1].nPartnerIndex=num_devices - 2;

			if( (strcasecmp(port[num_devices - 1].destination_num,"NONE")==0 || strlen(port[num_devices - 1].destination_num)<8) ) {
						printf("destination_num must not be NONE and must be at least >=8 digits\n");
						exitdemo(1);
			}
			printf("InIndex=%d|InDeviceName=%s|OutIndex=%d|OutDeviceName=%s\n",
								port[num_devices - 1].nPartnerIndex,port[num_devices - 2].netdevice,
								port[num_devices - 1].index,port[num_devices - 1].netdevice);
    }

	} /* End of read all lines loop */
	
	fclose (cfg_fp);
} /* End of Function load_config_file() */


/********************************************************************************
 *		 NAME: printandlog ()
 * DESCRIPTION: prints on the screen and to the log file. 
 *				 Put time stamp to the log file. If the
 *				 message to be logged is specific to a device,
 *				 it is logged to the device specific log file.
 * 
 *		INPUT: index is the device index
 *				 log_type valid values are ERR, MISC, EVENT, GC_APICALL, GC_APIERR and GC_RESULT_INFO
 *				 metaeventp = pointer to metaevent data structure - only used for GC_RESULT_INFO
 *				 msg_str message string to be printed
 *				 callindex is the CRN index - used for retrieving callstate.
 *	 RETURNS: NA
 ********************************************************************************/
static void printandlog(int index, int log_type, METAEVENT *metaeventp, char *msg_str, int callindex)
{	
	struct tm			*Time;
	time_t				Timet;
	int					i;

#ifdef _WIN32	/* Defined in Windows */  
	struct _timeb	  Timeb;										/* used to get milliseconds */
#else			/* Defined in Unix	*/
	struct timeb		Timeb;	  
#endif
	 
	char					TimeStr[MAX_STRING_SIZE];			/* will hold the time string when built */
	char					buffer[MAX_STRING_SIZE];			/* will have everything but the time */
	char					timestamp[MAX_STRING_SIZE];		/* holds the formatted time */
	char					str[MAX_STRING_SIZE];
	char					gc_apierr[MAX_STRING_SIZE];		/* iff have GC API error - most of time will be NULL */
	char					gc_result_info[MAX_STRING_SIZE]; /* iff have GC result info request - most of time will be null */
		 
	/* initialize temporary buffers */
	buffer[0] = '\0';
	gc_apierr[0] = gc_result_info[0] = timestamp[0] = '\0';
	 
	/* Put time stamp	 - everything but the milliseconds */
	TimeStr[0] = '\0';
	time(&Timet);
	Time = localtime(&Timet);

#ifdef _WIN32	/* Defined in Windows */  
	_ftime(&Timeb);												/* needed to get the milliseconds */
#else			/* Defined in Unix	 */  
	ftime(&Timeb); 
#endif

	if (log_type == MISC_WITH_NL) {
		strcpy(timestamp, "\n");
	} 

	sprintf(TimeStr, "%02d/%02d %02d:%02d:%02d.%03d ", Time->tm_mon+1, Time->tm_mday, Time->tm_hour, Time->tm_min, Time->tm_sec, Timeb.millitm);
	strcat(timestamp, TimeStr);

	switch(log_type)
	{
		case GC_APICALL:
				strcat(buffer, " [GC_APICALL ]:\t\t");
				break;

		case GC_APIERR:
				strcat(buffer, " [GC_APIERR  ]:\t\t");
				format_error_info(gc_apierr);
				break;

		case GC_RESULT_INFO:
				strcat(buffer, " [GC_RESULT_INFO]:\t\t");
				format_result_info(index, metaeventp, gc_result_info);
				break;

		case NON_GC_APIERR:
				strcat(buffer, " [NON_GC_APIERR]:\t\t");
				break;

		case EVENT:
				strcat(buffer, " [EVENT      ]:\t\t");
				break;

		case MISC:
				strcat(buffer, " [MISC       ]:\t\t");
				break;

		case MISC_WITH_NL:
				strcat(buffer, " [MISC       ]:\t\t");
				break;
		case STATE:
				strcat(buffer, " [GC_STATE   ]:\t\t");
				append_call_state_name(buffer, index, callindex);
				break;
		case APP_STATE:
				strcat(buffer, " [APP_STATE  ]:\t\t");
				break;
		case DX:
				strcat(buffer, " [DX         ]:\t\t");
				break;
		case M3G:
				strcat(buffer, " [M3G_APICALL]:\t\t");
				break;
		case M3GEV:
				strcat(buffer, " [M3GEV      ]:\t\t");
				break;
		case MM:
				strcat(buffer, " [MM_APICALL ]:\t\t");
				break;
		case MMEV:
				strcat(buffer, " [MMEV       ]:\t\t");
				break;
		case DM:
				strcat(buffer, " [DM_APICALL ]:\t\t");
				break;
		case DMEV:
				strcat(buffer, " [DMEV       ]:\t\t");
				break;
		default:
			sprintf(str, " [UNEXPECTED LOG TYPE %d]:", log_type);
			strcat(buffer, str);
			break;
	} /* End of Switch */
	 
	strcat(buffer, msg_str);
	 

	/* now do the actual logging */
	/* 1st to screen */
	printf("%s\n", buffer);

	/* print GC error or GC result information if required */
	if (gc_apierr[0]) {
		printf("%s\n", gc_apierr);
	}
	if (gc_result_info[0]) {
		printf("%s\n", gc_result_info);
	}

	/* then to log file(s) */
	if (index == ALL_DEVICES) {
		for (i = 0; i < num_devices; i++) {
			if (port[i].log_fp) {
				fprintf(port[i].log_fp, "%s ", timestamp);
				fprintf(port[i].log_fp, "%s\n", buffer);
            fflush(port[i].log_fp);
			}
		}
	} else if (index==BOARD_DEVICE) {
		fprintf(boardLogFd,"%s",timestamp);
		fprintf(boardLogFd,"%s\n",buffer);
		fflush(boardLogFd);
	} else if (port[index].log_fp) {
		fprintf(port[index].log_fp, "%s", timestamp);
		fprintf(port[index].log_fp, "%s\n", buffer);

		/* print GC error or GC result information if required */
		if (gc_apierr[0]) {
			fprintf(port[index].log_fp, "%s\n", gc_apierr);
		}
		if (gc_result_info[0]) {
			fprintf(port[index].log_fp, "%s\n", gc_result_info);
		}
      fflush(port[index].log_fp);
	}
}

/********************************************************************************
 *			NAME: format_error_info()
 *DESCRIPTION: formats error information using gc_ErrorInfo()
 *		  INPUT: buffer - buffer to format into
 *		RETURNS: None
 *	  CAUTIONS: No integrity checking done on dest_buffer overflow
 ********************************************************************************/
static void format_error_info(char *dest_buffer)
{
	GC_INFO			  t_info;	  /* Structure which stores information about GC related errors */ 

	gc_ErrorInfo(&t_info);
			 
	strcpy(dest_buffer, "					 GC ERROR: ");
	strcat(dest_buffer, t_info.gcMsg);
	strcat(dest_buffer, "\n");


	strcat(dest_buffer, "					 CC Name: ");
	strcat(dest_buffer, t_info.ccLibName);
	strcat(dest_buffer, "\n");


	strcat(dest_buffer, "					 CC ERROR: ");
	strcat(dest_buffer, t_info.ccMsg);
	strcat(dest_buffer, "\n");

}

/********************************************************************************
 *			NAME: format_result_info()
 *DESCRIPTION: formats result information using gc_ResultInfo()
 *		  INPUT: index - index into port data structure
 *             metaeventp - pointer to metaevent structure
 *					buffer - buffer to format into
 *		RETURNS: None
 *	  CAUTIONS: No integrity checking done on dest_buffer overflow
 ********************************************************************************/
static void format_result_info(int index, METAEVENT *metaeventp, char *dest_buffer)
{
	char					str[MAX_STRING_SIZE];
	int					rc;
	GC_INFO				t_info;	  /* Structure which stores information about GC related errors */ 

	rc = gc_ResultInfo(metaeventp, &t_info);
	if (rc < 0) {
		sprintf(str, "gc_ResultInfo() call failed");
		printandlog(index, GC_APIERR, NULL, str, 0);
		exitdemo(1);
	}
			 
	strcpy(dest_buffer, "					 GC Result: ");
	strcat(dest_buffer, t_info.gcMsg);
	strcat(dest_buffer, "\n");


	strcat(dest_buffer, "					 CC Name: ");
	strcat(dest_buffer, t_info.ccLibName);
	strcat(dest_buffer, "\n");


	strcat(dest_buffer, "					 CC Result: ");
	strcat(dest_buffer, t_info.ccMsg);
	strcat(dest_buffer, "\n");
}

/********************************************************************************
 *			NAME: append_call_state_name ()
 *DESCRIPTION: Appends the call state to a given buffer for a specified channel index
 *		  INPUT: buffer - buffer to append to
 *					index - index into port structure (from 0)
 *		RETURNS: None
 *	  CAUTIONS: No integrity checking done on buffer overflow
 ********************************************************************************/
static void append_call_state_name(char *buffer, int index, int callindex)
{
	struct channel		*pline;
	char					str[100];

	if ((callindex < 0) || (callindex > MAX_CALLS)) {
		sprintf(str, "Invalid callindex(%d)", callindex);
		strcat(buffer, str);
		return;
	}

	pline = &port[index];
	switch(pline->call[callindex].call_state)
	{ 
		case GCST_NULL:
			strcat(buffer, "GCST_NULL");
			break;

		case GCST_IDLE:
			strcat(buffer, "GCST_IDLE");
			break;

		case GCST_OFFERED:
			strcat(buffer, "GCST_OFFERED");
			break;

		case GCST_ALERTING:
			strcat(buffer, "GCST_ALERTING");
			break;

    case GCST_PROCEEDING:
			strcat(buffer, "GCST_PROCEEDING");
			break;

		case GCST_ACCEPTED:
			strcat(buffer, "GCST_ACCEPTED");
			break;
				
		case GCST_CONNECTED:
			strcat(buffer, "GCST_CONNECTED");
			break;

		case GCST_DISCONNECTED:
			strcat(buffer, "GCST_DISCONNECTED");
			break;

		default:
			sprintf(str, "UNKNOWN GC CALL STATE (%d)", pline->call[callindex].call_state);
			strcat(buffer, str);
			break;
	}
	
}

/*******************************************************************************
 *			NAME: existCRN()
 *DESCRIPTION: Search the CRN array for a valid CRN.
 *		  INPUT: CRN crn - CRN associated with the call.
 *						Note: if crn == -1 then this means return the index of the 1st valid
 *						crn in the call array.
 *		  			struct channel *pline - pointer to data structure for the channel
 *		RETURNS: if CRN exists, then return callindex; otherwise return OUT_OF_RANGE.		
 ******************************************************************************/
static int existCRN(CRN crn, struct channel *pline)
{
	int callindex;

	if (crn == -1) {
		/* For drop_outbound_calls_if_required() case */
		/* The function does not have access to metaevent structure. */
		for (callindex = 0; callindex < MAX_CALLS; callindex++) {
			/* if crn is found, then break */
			if (pline->call[callindex].crn > 0) {
				break;
			}
		}
	} else {
		/* All other cases. */
		for (callindex = 0; callindex < MAX_CALLS; callindex++) {
			if (pline->call[callindex].crn == crn) {
				/* crn is found, break out of for loop and continue. */
				break;
			}
		}
	}

	/* If the callindex < MAX_CALLS, then return CRN. */
	/* Otherwise return (-1) indicating CRN was not found. */
	if (callindex < MAX_CALLS) {
		/* CRN found */
		return (callindex);
	}
	else {
		/* CRN not found */
		return (-1);
	}
}

/********************************************************************************
 *			NAME: initiate_exit()
 * DESCRIPTION: Initiate termination of active calls.
 * 			This will be done via gc_ResetLineDev()
 * 			which will clean up active calls
 *		  INPUT: none
 *		RETURNS: none
 *
 *******************************************************************************/
static void initiate_exit()
{
	int				index;
	char				str[MAX_STRING_SIZE]; 
	struct channel	*pline;
	static int		exit_active = NO;

	if (exit_active == YES) {
		return;
	}
	printandlog(ALL_DEVICES, MISC, NULL, "********** Exit initiated ***********\n", 0);

	exit_active = YES;
	for(index = 0; index < num_devices; index++) {
		pline = &port[index];
		if (pline->ldev) {
			if (pline->resetlinedev_active == NO) {
				if (gc_ResetLineDev(port[index].ldev, EV_ASYNC) == GC_SUCCESS) {
					reset_linedevs_left_to_complete++;
					pline->resetlinedev_active = YES;
					sprintf(str, "gc_ResetLineDev(linedev=%ld, mode=EV_ASYNC) Success",
						 port[index].ldev);
					printandlog(index, GC_APICALL, NULL, str, 0);
				} else {
					sprintf(str, "gc_ResetLineDev(linedev=%ld, mode=EV_ASYNC) failure ignored while exiting (reload boards after exit)",
						port[index].ldev);
					printandlog(index, GC_APIERR, NULL, str, 0);
				}
			} else {
				reset_linedevs_left_to_complete++;			/* reset linedev already active */
			}															/* just wait for it to complete */
		}
	}
	if (reset_linedevs_left_to_complete) {
		sprintf(str, "Please wait while %d gc_ResetLineDev()s are being done...",
			reset_linedevs_left_to_complete);
		printandlog(pline->index, MISC, NULL, str, 0);
	}
}

/********************************************************************************
 *			NAME: exitdemo()
 *DESCRIPTION: Closes all the logfiles,
 *					Closes all the devices
 *					Disables event handler
 *					Stops GlobalCall
 *					And Exits
 *		  INPUT: exitlevel 0 success, 1 some error 
 *					Exit from program with the exitlevel as specified by exitlevel
 *		RETURNS: exits		
 ********************************************************************************/
static void exitdemo(int exitlevel)
{
	int					index;
	char					str[MAX_STRING_SIZE]; 
	struct channel		*pline;
	time_t				finish_time;
	double				duration, min_time = 0;
	static int			recursive = 0;					/* called recursively. e.g. Multiple ^c. If yes, skip clean-up logic and exit */

	printandlog(ALL_DEVICES, MISC_WITH_NL, NULL, "********* Program Exiting *********", 0);
	if (recursive == 0) {
		recursive++;
		for (index = 0; index < num_devices; index++) {
			pline = &port[index];
		 
			/* The total calls run on Inbound & Outbound side */
			if (pline->direction == DIR_IN) {
				sprintf(str, "The total no of Inbound calls on this device is: %d", pline->numb_calls);
				printandlog(pline->index, MISC, NULL, str, 0);
			} else { 
				sprintf(str, "The total no of Outbound calls on this device is: %d", pline->numb_calls);
				printandlog(pline->index, MISC, NULL, str, 0);
			}
			sprintf(str, "The total no of Inbound calls over all devices is: %d", in_calls);
			printandlog(pline->index, MISC, NULL, str, 0);
			sprintf(str, "The total no of Outbound calls over all devices is: %d", out_calls);
			printandlog(pline->index, MISC, NULL, str, 0);

			/* Note down the finish time for the test */
			time(&finish_time);
			/* Calculate the duration taken by the test */
			duration = difftime(finish_time,	 start_time);
			min_time = duration / 60;
			sprintf(str, "The total duration taken by the test is: %5.2f minutes", min_time);
			printandlog(pline->index, MISC, NULL, str, 0);
		}
	}

	/* code that can never be called recursively */

	/* close all log files */
	for (index = 0; index < num_devices; index++) {
		pline = &port[index];
		if (pline->log_fp != NULL) {
			fclose(pline->log_fp);
			pline->log_fp = NULL;			
		}
	}
	if(boardLogFd!=NULL) {
		fclose(boardLogFd);
		boardLogFd=NULL;
	}
	if (gc_started == YES) {
		for(index = 0; index < num_devices; index++) {
			pline = &port[index];

			if (pline->ldev) {
				/* nothing can do about errors, so ignore them */			
				gc_Close(pline->ldev);
				pline->ldev = 0;
			}
			/* if GC makecall block being used, then free it */
			if (port[index].makecallblk.gclib) {
				/* free the parm block if used using the utility functions */
				/* CAUTION: this must be done before freeing the GC makecall block */
				if (port[index].makecallblk.gclib->ext_datap) {
					gc_util_delete_parm_blk(port[index].makecallblk.gclib->ext_datap);
				}

				/* now free the GC makecall block */
				free(port[index].makecallblk.gclib);
				port[index].makecallblk.gclib = NULL;
			}
		}
		gc_Stop();
	}
	exit(exitlevel);
} /* End of function exitdemo() */
/*****<< End of File >>*****/
